@@ -7,7 +7,7 @@
 --  U     6   1   bb   y    U    F    O   O   T                             --
 --                                                                          --
 -- U61 is another block based game                                          --
--- Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   --
+-- Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              --
 --                                                                          --
 -- This program is free software; you can redistribute it and/or            --
 -- modify it under the terms of the GNU General Public License              --
@@ -23,7 +23,7 @@
 -- along with this program; if not, write to the Free Software              --
 -- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA--
 --                                                                          --
--- This project is also available on SourceForge  (http://sourceforge.net)  --
+-- This project is also available on Savannah (http://savannah.gnu.org)     --
 ------------------------------------------------------------------------------
 
 -- tetramino game, the goal is to have an horizontal line filled with
@@ -31,6 +31,12 @@
 -- colors do not have any purpose except make the game look nicer
 
 
+-- initialization ------------------------------------------------------------
+
+function user_start()
+end
+
+
 -- shape definition ----------------------------------------------------------
 
 function user_new_shape(num)
