@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -41,6 +41,7 @@
 #include "text.h"
 #include "const.h"
 #include "version.h"
+#include "macro.h"
 
 #include <stdio.h>
 
@@ -64,11 +65,11 @@ U61_AboutMenu::U61_AboutMenu(U61_Menu *parent)
     char buf2[U61_CONST_STRING_SIZE];
 
     U61_Version::get_version_text(buf1);
-    sprintf(buf2,"Release %s",buf1);
+    U61_MACRO_SPRINTF1(buf2,"Release %s",buf1);
 
     add_item(new U61_Text(buf2));
     add_item(new U61_Text(" "));
-    add_item(new U61_Text("Copyright (C) 2000-2002"));
+    add_item(new U61_Text("Copyright (C) 2000-2003"));
     add_item(new U61_Text(" "));
     add_item(new U61_Text("Author: Christian Mauduit"));
     add_item(new U61_Text("License: free software (GPL)"));
@@ -77,21 +78,3 @@ U61_AboutMenu::U61_AboutMenu(U61_Menu *parent)
     select(0);
 }
 
-/*--------------------------------------------------------------------------*/
-/*
- * this function does nothing, this way the player sees plain text
- */
-void U61_AboutMenu::select_up()
-{
-    
-}
-
-/*--------------------------------------------------------------------------*/
-/*
- * this function does nothing, this way the player sees plain text
- */
-void U61_AboutMenu::select_down()
-{
-    
-}
-
