@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -33,32 +33,28 @@
  */
 
 /*---------------------------------------------------------------------------
- includes
- ---------------------------------------------------------------------------*/
+  includes
+  ---------------------------------------------------------------------------*/
 
 #include "application.h"
 #include "global.h" 
 #include "version.h" 
-#include "debug.h"
+#include "log.h"
 #include "platform.h"
-#include "script.h"
-#include "sound.h"
+#include "setup.h"
+#include "macro.h"
 
 #include <stdio.h> 
-#include <ClanLib/display.h>
-#include <ClanLib/sound.h>
-#include <ClanLib/network.h>
-#include <ClanLib/mikmod.h>
 
 /*---------------------------------------------------------------------------
- globals
- ---------------------------------------------------------------------------*/
+  globals
+  ---------------------------------------------------------------------------*/
 
 U61_Application application;
 
 /*---------------------------------------------------------------------------
- functions
- ---------------------------------------------------------------------------*/
+  functions
+  ---------------------------------------------------------------------------*/
 
 /*--------------------------------------------------------------------------*/
 /*
@@ -69,9 +65,9 @@ char *U61_Application::get_title()
   char buf[U61_CONST_STRING_SIZE];        
     
   U61_Version::get_version_text(buf);
-  sprintf(app_name,
-	  "U61 %s",
-	  buf);
+  U61_MACRO_SPRINTF1(app_name,
+		     "U61 %s",
+		     buf);
 
   return app_name; 
 }
@@ -85,17 +81,11 @@ int U61_Application::main(int argc, char **argv)
   int retcode=0;
   char buf[U61_CONST_STRING_SIZE];
 
-  CL_SetupCore::init();
-  U61_Debug::init();
+  U61_Setup::init1();
 
   if (process_options(argc,argv))
     {
-      CL_SetupDisplay::init();
-      CL_SetupSound::init();
-      CL_SetupNetwork::init();
-      CL_SetupMikMod::init();
-      U61_Script::open();
-      U61_Platform::init();
+      U61_Setup::init2();
 
       U61_Version::get_version_text(buf);
       U61_LOG_MESSAGE("Starting U61 "<<buf<<" using ClanLib "<<CL_VERSION_STRING<<"...");
@@ -121,22 +111,14 @@ int U61_Application::main(int argc, char **argv)
 	  retcode=1;
 	}
 
-      U61_LOG_MESSAGE("Game over...");
+      U61_LOG_MESSAGE("Exiting program.");
 
-      U61_Sound::wait_no_playback();
-
-      U61_Platform::deinit();
-      U61_Script::close();
-      CL_SetupMikMod::deinit();
-      CL_SetupNetwork::deinit();    
-      CL_SetupSound::deinit();
-      CL_SetupDisplay::deinit();
+      U61_Setup::deinit2();
     }
   
-  U61_Debug::display_close_message();
+  U61_Log::display_close_message();
 
-  U61_Debug::deinit();
-  CL_SetupCore::deinit();
+  U61_Setup::deinit1();
 
   return retcode;
 }
@@ -147,12 +129,11 @@ int U61_Application::main(int argc, char **argv)
  */ 
 bool U61_Application::process_options(int argc,char **argv)
 {
-  char buf[U61_CONST_STRING_SIZE];        
   char c;
   bool result=true;
   int i;
 
-  for (i=1;i<argc;++i)
+  for (i=1;i<argc && result;++i)
     {
       if (argv[i][0]=='-' || argv[i][0]=='/')
 	{
@@ -160,49 +141,105 @@ bool U61_Application::process_options(int argc,char **argv)
 	  switch (c)
 	    {
 	    case 'v':
-	      U61_Version::get_version_text(buf);
-	      std::cout<<buf<<"\n";
+	      display_version();
 	      result=false;
 	      break;
 	    case '?': 
 	    case 'h':
-	      U61_Version::get_version_text(buf);
-	      std::cout<<"\n";
-	      std::cout<<"U61 - another block-based game\n";
-	      std::cout<<"\n";
-	      std::cout<<"v"<<buf<<" compiled on "<<__DATE__<<" at "__TIME__"\n";
-	      std::cout<<"Copyright (C) 2000-2002 Christian Mauduit\n";
-	      std::cout<<"\n";
-	      std::cout<<"This program is free software; you can redistribute it and/or modify\n";
-	      std::cout<<"it under the terms of the GNU General Public License as published by\n";
-	      std::cout<<"the Free Software Foundation; either version 2 of the License, or\n";
-	      std::cout<<"(at your option) any later version.\n";
-	      std::cout<<"\n";
-
-	      std::cout<<"This program is distributed in the hope that it will be useful,\n";
-	      std::cout<<"but WITHOUT ANY WARRANTY; without even the implied warranty of\n";
-	      std::cout<<"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n";
-	      std::cout<<"GNU General Public License for more details.\n";
-
-	      std::cout<<"\n";
-	      std::cout<<"You should have received a copy of the GNU General Public License\n";
-	      std::cout<<"along with this program; if not, write to the Free Software\n";
-	      std::cout<<"Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA\n";
-	      std::cout<<"\n";
-	      std::cout<<"Documentation is available in HTML, ascii text, PostScript, PDF... Read it 8-)\n";
-	      std::cout<<"\n";
-	      std::cout<<"U61 homepage   : http://www.ufoot.org/u61\n";
-	      std::cout<<"Contact author : ufoot@ufoot.org\n";
+	      display_help();
 	      result=false;
 	      break;
+	    case '-':
+	      result=process_long_option(argv[i]+2);
+	      break;
 	    default:
-	      std::cout<<"Unknown option '"<<c<<"' \n";
+	      std::cout<<"Unknown option '-"<<(argv[i]+1)<<"'\n";
 	      result=false;
 	      break;
 	    }
 	}
+      else
+	{
+	  std::cout<<"Unknown option '"<<argv[i]<<"'\n";
+	  result=false;
+	}
     }
 
   return result;
 }
 
+/*--------------------------------------------------------------------------*/
+/*
+ * Handles "long" options such as "--version" or "--help"
+ */ 
+bool U61_Application::process_long_option(char *opt)
+{
+  bool result=true;
+
+  if (strcmp(opt,"version")==0) 
+    {
+      display_version();
+      result=false;
+    }
+  else if (strcmp(opt,"help")==0) 
+    {
+      display_help();
+      result=false;
+    }
+  else
+    {
+      std::cout<<"Unknown option '--"<<opt<<"'\n";
+      result=false;
+    }
+
+  return result;
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * Displays the game version
+ */ 
+void U61_Application::display_version()
+{	
+  char buf[U61_CONST_STRING_SIZE];        
+
+  U61_Version::get_version_text(buf);
+  std::cout<<buf<<"\n";
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * Displays a short help message
+ */ 
+void U61_Application::display_help()
+{
+  char buf[U61_CONST_STRING_SIZE];        
+
+  U61_Version::get_version_text(buf);
+  std::cout<<"\n";
+  std::cout<<"U61 - another block-based game\n";
+  std::cout<<"\n";
+  std::cout<<"v"<<buf<<" compiled on "<<__DATE__<<" at "__TIME__"\n";
+  std::cout<<"Copyright (C) 2000-2003 Christian Mauduit\n";
+  std::cout<<"\n";
+  std::cout<<"This program is free software; you can redistribute it and/or modify\n";
+  std::cout<<"it under the terms of the GNU General Public License as published by\n";
+  std::cout<<"the Free Software Foundation; either version 2 of the License, or\n";
+  std::cout<<"(at your option) any later version.\n";
+  std::cout<<"\n";
+
+  std::cout<<"This program is distributed in the hope that it will be useful,\n";
+  std::cout<<"but WITHOUT ANY WARRANTY; without even the implied warranty of\n";
+  std::cout<<"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n";
+  std::cout<<"GNU General Public License for more details.\n";
+
+  std::cout<<"\n";
+  std::cout<<"You should have received a copy of the GNU General Public License\n";
+  std::cout<<"along with this program; if not, write to the Free Software\n";
+  std::cout<<"Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA\n";
+  std::cout<<"\n";
+  std::cout<<"U61 homepage   : http://www.ufoot.org/u61\n";
+  std::cout<<"Contact author : ufoot@ufoot.org\n";
+  std::cout<<"\n";
+  std::cout<<"Documentation is available in HTML, ascii text, PostScript, PDF... Read it 8-)\n";
+}
