@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -32,8 +32,8 @@
  * description: main file, where the ClanLin application object is defined 
  */
 
-#ifndef INC_U61_APPLICATION
-#define INC_U61_APPLICATION
+#ifndef U61_INC_APPLICATION
+#define U61_INC_APPLICATION
 
 /*---------------------------------------------------------------------------
  includes
@@ -56,6 +56,9 @@ public:
 protected:
     char app_name[U61_CONST_STRING_SIZE];
     bool process_options(int argc, char **argv);
+    bool process_long_option(char *opt);
+    void display_version();
+    void display_help();
 };
 
 #endif
