@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -34,8 +34,8 @@
  *              it usually has a recognizable shape 
  */
 
-#ifndef INC_U61_BLOCK
-#define INC_U61_BLOCK
+#ifndef U61_INC_BLOCK
+#define U61_INC_BLOCK
 
 /*---------------------------------------------------------------------------
  includes
@@ -45,6 +45,7 @@
 
 #include "blockitem.h"
 #include "data.h"
+#include "utils.h"
 
 /*---------------------------------------------------------------------------
  constants
@@ -54,6 +55,8 @@
 #define U61_BLOCK_PREVIEW_WIDTH   4
 #define U61_BLOCK_PREVIEW_HEIGHT  4
 
+#define U61_BLOCK_SERIALIZED_SIZE (U61_BLOCK_MAX_ITEMS*U61_BLOCKITEM_SERIALIZED_SIZE + 3*U61_SERIAL_INT_SERIALIZED_SIZE)
+
 /*---------------------------------------------------------------------------
  classes
  ---------------------------------------------------------------------------*/
@@ -88,9 +91,14 @@ public:
     void center();
     void corner();
 
-    void draw(int x,int y,int size=U61_DATA_SIZE_BIG);
+    void draw(int x,int y,int size,
+	      int map_width, int map_height);
+    void draw_anticipation(int x,int y,int size,
+	      int map_width, int map_height);
     void draw_preview(int x,int y,int size=U61_DATA_SIZE_BIG);
-    void draw_anticipation(int x,int y,int size=U61_DATA_SIZE_BIG);
+
+    bool serialize(unsigned char **buffer, unsigned char *buffer_limit);
+    bool unserialize(unsigned char **buffer, unsigned char *buffer_limit);
     
 protected:
     U61_BlockItem items[U61_BLOCK_MAX_ITEMS];
