@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -40,6 +40,8 @@
  ---------------------------------------------------------------------------*/
 
 #include "blockitem.h"
+#include "serial.h"
+#include "log.h"
 
 /*---------------------------------------------------------------------------
  variants
@@ -116,3 +118,46 @@ int U61_BlockItem::get_y()
     return y;
 }
 
+/*--------------------------------------------------------------------------*/
+/*
+ * Serializes a blockitem into a byte strem
+ */
+bool U61_BlockItem::serialize(unsigned char **buffer, unsigned char *buffer_limit)
+{
+  bool result=true;
+  unsigned char *buffer_start=*buffer;
+
+  result=result && square.serialize(buffer,buffer_limit);
+
+  result=result && U61_Serial::serialize_int(x,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_int(y,buffer,buffer_limit);
+
+  if ((*buffer) - buffer_start != U61_BLOCKITEM_SERIALIZED_SIZE)
+    {
+      U61_LOG_WARNING("Wrong buffer size for blockitem serialization (diff="<<(buffer_start - (*buffer) + U61_BLOCKITEM_SERIALIZED_SIZE)<<")!");
+    }
+
+  return result;
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * Un-serializes a blockitem from a byte strem
+ */
+bool U61_BlockItem::unserialize(unsigned char **buffer, unsigned char *buffer_limit)
+{
+  bool result=true;
+  unsigned char *buffer_start=*buffer;
+
+  result=result && square.unserialize(buffer,buffer_limit);
+
+  result=result && U61_Serial::unserialize_int(&x,buffer,buffer_limit);
+  result=result && U61_Serial::unserialize_int(&y,buffer,buffer_limit);
+
+  if ((*buffer) - buffer_start != U61_BLOCKITEM_SERIALIZED_SIZE)
+    {
+      U61_LOG_WARNING("Wrong buffer size for blockitem unserialization (diff="<<(buffer_start - (*buffer) + U61_BLOCKITEM_SERIALIZED_SIZE)<<")!");
+    }
+
+  return result;
+}
