@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -35,8 +35,8 @@
  *              holding the coordinates of these squares
  */
 
-#ifndef INC_U61_BLOCKITEM
-#define INC_U61_BLOCKITEM
+#ifndef U61_INC_BLOCKITEM
+#define U61_INC_BLOCKITEM
 
 /*---------------------------------------------------------------------------
  includes
@@ -52,6 +52,8 @@
 
 #define U61_BLOCKITEM_COORD_MAX 10
 
+#define U61_BLOCKITEM_SERIALIZED_SIZE (U61_SQUARE_SERIALIZED_SIZE + 2*U61_SERIAL_INT_SERIALIZED_SIZE)
+
 /*---------------------------------------------------------------------------
  classes
  ---------------------------------------------------------------------------*/
@@ -71,6 +73,9 @@ public:
 
     void reset();
 
+    bool serialize(unsigned char **buffer, unsigned char *buffer_limit);
+    bool unserialize(unsigned char **buffer, unsigned char *buffer_limit);
+
 protected:
     int x;
     int y;
