@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -39,6 +39,7 @@
  ---------------------------------------------------------------------------*/
 
 #include "clientdispatcher.h"
+#include "log.h"
 
 /*---------------------------------------------------------------------------
  variants
@@ -63,6 +64,8 @@ U61_ClientDispatcher::U61_ClientDispatcher() : U61_Dispatcher()
  */
 void U61_ClientDispatcher::open(U61_Connection *c)
 {
+  U61_LOG_MESSAGE("Accepted by server.");
+
   connection=c;
 }
 
@@ -77,7 +80,7 @@ void U61_ClientDispatcher::close()
 
 /*--------------------------------------------------------------------------*/
 /*
- * sends an event to no one since we're in a local game
+ * sends an event 
  */
 bool U61_ClientDispatcher::send(U61_Event evt)
 {
@@ -93,8 +96,7 @@ bool U61_ClientDispatcher::send(U61_Event evt)
 
 /*--------------------------------------------------------------------------*/
 /*
- * returns always false since no events can be from remote machines when
- * in local mode. Should never be called.
+ * receives events
  */
 bool U61_ClientDispatcher::recv(U61_Event *evt)
 {
@@ -110,15 +112,16 @@ bool U61_ClientDispatcher::recv(U61_Event *evt)
 
 /*--------------------------------------------------------------------------*/
 /*
- * returns true if a packet is available
+ * returns true if an event is available
  */
 bool  U61_ClientDispatcher::peek()
 {
   bool result=false;
+  U61_Event evt;
 
   if (connection!=NULL)
     {
-      result=connection->peek();
+      result=connection->peek_event();
     }
 
   return result;
