@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -36,64 +36,67 @@
 
 
 /*---------------------------------------------------------------------------
- includes
- ---------------------------------------------------------------------------*/
+  includes
+  ---------------------------------------------------------------------------*/
 
 #include "clientprotocol.h"
 #include "global.h"
 #include "time.h"
 #include "platform.h"
-#include "debug.h"
+#include "log.h"
+#include "macro.h"
+#include "checksum.h"
 
 /*---------------------------------------------------------------------------
- constants
- ---------------------------------------------------------------------------*/
+  constants
+  ---------------------------------------------------------------------------*/
 
-#define U61_CLIENTPROTOCOL_SEARCHING                10
-#define U61_CLIENTPROTOCOL_CONNECTED_TO_SERVER      11
-#define U61_CLIENTPROTOCOL_JOIN_DELAY_ELAPSED       12
-#define U61_CLIENTPROTOCOL_CHANNEL_FOUND            13
-#define U61_CLIENTPROTOCOL_SCRIPT_NAME_RECEIVED     30
-#define U61_CLIENTPROTOCOL_SCRIPT_RECEIVED          31
-#define U61_CLIENTPROTOCOL_ID0_RECEIVED             40
-#define U61_CLIENTPROTOCOL_ID1_RECEIVED             41
-#define U61_CLIENTPROTOCOL_INITIAL_SPEED_RECEIVED   42
-#define U61_CLIENTPROTOCOL_ACCELERATION_RECEIVED    43
-#define U61_CLIENTPROTOCOL_CURSE_DELAY_RECEIVED     44
-#define U61_CLIENTPROTOCOL_TIME_RECEIVED            45
-#define U61_CLIENTPROTOCOL_PLAYING                  60   
-#define U61_CLIENTPROTOCOL_CLOSED                   61
-
-#define U61_CLIENTPROTOCOL_BROADCAST_TIMEOUT      3000
+#define U61_CLIENTPROTOCOL_CONNECTED_TO_SERVER      10
+#define U61_CLIENTPROTOCOL_PASSWORD_SENT            30
+#define U61_CLIENTPROTOCOL_PASSWORD_OK_RECEIVED     31
+#define U61_CLIENTPROTOCOL_SCRIPT_NAME_RECEIVED     40
+#define U61_CLIENTPROTOCOL_SCRIPT_RECEIVED          41
+#define U61_CLIENTPROTOCOL_ID0_RECEIVED             50
+#define U61_CLIENTPROTOCOL_ID1_RECEIVED             51
+#define U61_CLIENTPROTOCOL_INITIAL_SPEED_RECEIVED   52
+#define U61_CLIENTPROTOCOL_ACCELERATION_RECEIVED    53
+#define U61_CLIENTPROTOCOL_CURSE_DELAY_RECEIVED     54
+#define U61_CLIENTPROTOCOL_TIME_RECEIVED            55
+#define U61_CLIENTPROTOCOL_PLAYING                  70   
+#define U61_CLIENTPROTOCOL_CLOSED                   71
 
 /*---------------------------------------------------------------------------
- functions
- ---------------------------------------------------------------------------*/
+  functions
+  ---------------------------------------------------------------------------*/
 
 /*--------------------------------------------------------------------------*/
 /*
  * creates a client protocol
  */
 U61_ClientProtocol::U61_ClientProtocol(U61_ClientDispatcher *d, 
-				       int p, int ch,
-                                       char *id, bool bcast)
-  : U61_Protocol(p,ch)
+				       int p,
+                                       char *srv)
+  : U61_Protocol(p)
 {
   char *pos;
 
   /*
-   * we remove spaces from the server name
+   * we remove spaces at the end of the server name
    */
-  pos=strchr(id,' ');
+  pos=strchr(srv,' ');
   if (pos!=NULL)
     {
       (*pos)='\0';
     }
 
   dispatcher=d;
-  server_id=id;
-  broadcast=bcast;
+  server_id=srv;
   connection=NULL;
+
+  connecting=false;
+  connect_thread=NULL;
+  connect_error=false;
+  password_error=false;
 }
 
 /*--------------------------------------------------------------------------*/
@@ -102,17 +105,7 @@ U61_ClientProtocol::U61_ClientProtocol(U61_ClientDispatcher *d,
  */
 U61_ClientProtocol::~U61_ClientProtocol()
 {
-  if (connection!=NULL)
-    {
-      delete connection;
-      connection=NULL;
-    }
-
-  if (session!=NULL)
-    {
-      delete session; 
-      session=NULL;
-    }
+  stop_connect_thread(true);
 }
 
 /*--------------------------------------------------------------------------*/
@@ -123,24 +116,53 @@ void U61_ClientProtocol::check_connection()
 {
   int i;
   U61_Player *p;
+  U61_Event evt;
 
   if (stage!=U61_CLIENTPROTOCOL_CLOSED 
-      && session!=NULL 
-      && session->receive_session_closed())
+      && connection)
     {
-      stage=U61_CLIENTPROTOCOL_CLOSED;
-      U61_LOG_DEBUG("stage=CLOSED");
-      connection->close();
-
-      /*
-       * We kill all the remote players and let the game keep going locally
-       */
-      for (i=0;i<U61_DISPATCHER_MAX_PLAYER_ID;++i)
+      if (!connection->is_alive())
 	{
-	  p=U61_Global::game.find_player_by_id(i);
-          if (p!=NULL && p->is_available() && !p->is_local())
+	  if (stage!=U61_CLIENTPROTOCOL_PLAYING)
 	    {
-	      p->kill();
+	      if (password_error)
+		{
+		  U61_LOG_WARNING("Wrong password.");
+		  U61_Menu::display_info("Wrong password");	      
+		}
+	      else
+		{
+		  U61_Menu::display_info("Connection closed");	      
+		}
+	    }
+	  
+	  stage=U61_CLIENTPROTOCOL_CLOSED;
+	  U61_LOG_DEBUG("stage=CLOSED");
+	  connection->close();
+	  
+	  /*
+	   * We kill all the remote players and let the game keep going locally
+	   */
+	  for (i=0;i<U61_DISPATCHER_MAX_PLAYER_ID;++i)
+	    {
+	      p=U61_Global::game.find_player_by_id(i);
+	      if (p!=NULL && p->is_available() && !p->is_local())
+		{
+		  evt.time=0;
+		  evt.author=evt.target=i;
+		  
+		  evt.code=U61_EVENT_READY;
+		  dispatcher->put_out(evt);   
+		  
+		  evt.code=U61_EVENT_KILL;
+		  dispatcher->put_out(evt);   
+	      
+		  p=U61_Global::game.find_player_by_id(i);
+		  if (p!=NULL)
+		    {
+		      p->kill();
+		    }
+		}
 	    }
 	}
     }
@@ -157,21 +179,8 @@ int U61_ClientProtocol::send_next_packet()
 
   switch(stage)
     {
-    case U61_PROTOCOL_BEGIN:
-      if ((result=start_searching())==U61_PROTOCOL_DONE)
-	{
-	  stage=U61_CLIENTPROTOCOL_SEARCHING;
-	  U61_LOG_DEBUG("stage=SEARCHING");
-	}
-      break;
     case U61_CLIENTPROTOCOL_CONNECTED_TO_SERVER:
-      if ((result=join_delay())==U61_PROTOCOL_DONE)
-	{
-	  stage=U61_CLIENTPROTOCOL_JOIN_DELAY_ELAPSED;
-	  U61_LOG_DEBUG("stage=JOIN_DELAY_ELAPSED");
-	}
-      break;
-    case U61_CLIENTPROTOCOL_CHANNEL_FOUND:
+      U61_Menu::display_info("Sending program id...");	      
       if ((result=send_program(connection))==U61_PROTOCOL_DONE)
 	{
 	  stage=U61_PROTOCOL_PROGRAM_SENT;
@@ -179,13 +188,23 @@ int U61_ClientProtocol::send_next_packet()
 	}
       break;
     case U61_PROTOCOL_PROGRAM_RECEIVED:
+      U61_Menu::display_info("Sending version id...");	      
       if ((result=send_version(connection))==U61_PROTOCOL_DONE)
 	{
 	  stage=U61_PROTOCOL_VERSION_SENT;
 	  U61_LOG_DEBUG("stage=VERSION_SENT");
 	}
       break;
+    case U61_PROTOCOL_VERSION_RECEIVED:
+      U61_Menu::display_info("Sending password...");	      
+      if ((result=send_password())==U61_PROTOCOL_DONE)
+	{
+	  stage=U61_CLIENTPROTOCOL_PASSWORD_SENT;
+	  U61_LOG_DEBUG("stage=PASSWORD_SENT");
+	}
+      break;
     case U61_CLIENTPROTOCOL_TIME_RECEIVED:
+      U61_Menu::display_info("Sending ready signal...");	      
       if ((result=send_ready(connection))==U61_PROTOCOL_DONE)
 	{
 	  stage=U61_PROTOCOL_READY_SENT;
@@ -193,6 +212,7 @@ int U61_ClientProtocol::send_next_packet()
 	}
       break;
     case U61_PROTOCOL_READY_RECEIVED:
+      U61_Menu::display_info("");	      
       start_game();
       dispatcher->open(connection);
       
@@ -217,21 +237,16 @@ int U61_ClientProtocol::recv_next_packet()
 
   switch(stage)
     {
-    case U61_CLIENTPROTOCOL_SEARCHING:
+    case U61_PROTOCOL_BEGIN:
+      U61_Menu::display_info("Connecting...");	      
       if ((result=connect_to_server())==U61_PROTOCOL_DONE)
 	{
 	  stage=U61_CLIENTPROTOCOL_CONNECTED_TO_SERVER;
 	  U61_LOG_DEBUG("stage=CONNECTED_TO_SERVER");
 	}
       break;
-    case U61_CLIENTPROTOCOL_JOIN_DELAY_ELAPSED:
-      if ((result=find_channel())==U61_PROTOCOL_DONE)
-	{
-	  stage=U61_CLIENTPROTOCOL_CHANNEL_FOUND;
-	  U61_LOG_DEBUG("stage=CHANNEL_FOUND");
-	}
-      break;
     case U61_PROTOCOL_PROGRAM_SENT:
+      U61_Menu::display_info("Receiving program id...");	      
       if ((result=recv_program(connection))==U61_PROTOCOL_DONE)
 	{  
 	  stage=U61_PROTOCOL_PROGRAM_RECEIVED;
@@ -239,13 +254,23 @@ int U61_ClientProtocol::recv_next_packet()
 	}
       break;
     case U61_PROTOCOL_VERSION_SENT:
+      U61_Menu::display_info("Receiving version id...");	      
       if ((result=recv_version(connection))==U61_PROTOCOL_DONE)
 	{
 	  stage=U61_PROTOCOL_VERSION_RECEIVED;
 	  U61_LOG_DEBUG("stage=VERSION_RECEIVED");
 	}
       break;
-    case U61_PROTOCOL_VERSION_RECEIVED:
+    case U61_CLIENTPROTOCOL_PASSWORD_SENT:
+      U61_Menu::display_info("Checking password...");	      
+      if ((result=recv_password_ok())==U61_PROTOCOL_DONE)
+	{
+	  stage=U61_CLIENTPROTOCOL_PASSWORD_OK_RECEIVED;
+	  U61_LOG_DEBUG("stage=PASSWORD_OK_RECEIVED");
+	}
+      break;
+    case U61_CLIENTPROTOCOL_PASSWORD_OK_RECEIVED:
+      U61_Menu::display_info("Receiving script name...");	      
       if ((result=recv_script_name())==U61_PROTOCOL_DONE)
 	{
 	  stage=U61_CLIENTPROTOCOL_SCRIPT_NAME_RECEIVED;
@@ -253,6 +278,7 @@ int U61_ClientProtocol::recv_next_packet()
 	}
       break;
     case U61_CLIENTPROTOCOL_SCRIPT_NAME_RECEIVED:
+      U61_Menu::display_info("Receiving script code...");	      
       if ((result=recv_script())==U61_PROTOCOL_DONE)
 	{
 	  stage=U61_CLIENTPROTOCOL_SCRIPT_RECEIVED;
@@ -260,6 +286,7 @@ int U61_ClientProtocol::recv_next_packet()
 	}
       break;
     case U61_CLIENTPROTOCOL_SCRIPT_RECEIVED:
+      U61_Menu::display_info("Receiving id0...");	      
       if ((result=recv_id0())==U61_PROTOCOL_DONE)
 	{
 	  stage=U61_CLIENTPROTOCOL_ID0_RECEIVED;
@@ -267,6 +294,7 @@ int U61_ClientProtocol::recv_next_packet()
 	}
       break;
     case U61_CLIENTPROTOCOL_ID0_RECEIVED:
+      U61_Menu::display_info("Receiving id1...");	      
       if ((result=recv_id1())==U61_PROTOCOL_DONE)
 	{
 	  stage=U61_CLIENTPROTOCOL_ID1_RECEIVED;
@@ -274,6 +302,7 @@ int U61_ClientProtocol::recv_next_packet()
 	}
       break;
     case U61_CLIENTPROTOCOL_ID1_RECEIVED:
+      U61_Menu::display_info("Receiving init speed...");	      
       if ((result=recv_initial_speed())==U61_PROTOCOL_DONE)
 	{
 	  stage=U61_CLIENTPROTOCOL_INITIAL_SPEED_RECEIVED;
@@ -281,6 +310,7 @@ int U61_ClientProtocol::recv_next_packet()
 	}
       break;
     case U61_CLIENTPROTOCOL_INITIAL_SPEED_RECEIVED:
+      U61_Menu::display_info("Receiving init accel...");	      
       if ((result=recv_acceleration())==U61_PROTOCOL_DONE)
 	{
 	  stage=U61_CLIENTPROTOCOL_ACCELERATION_RECEIVED;
@@ -288,6 +318,7 @@ int U61_ClientProtocol::recv_next_packet()
 	}
       break;
     case U61_CLIENTPROTOCOL_ACCELERATION_RECEIVED:
+      U61_Menu::display_info("Receiving curse delay...");	      
       if ((result=recv_curse_delay())==U61_PROTOCOL_DONE)
 	{
 	  stage=U61_CLIENTPROTOCOL_CURSE_DELAY_RECEIVED;
@@ -295,6 +326,7 @@ int U61_ClientProtocol::recv_next_packet()
 	}
       break;
     case U61_CLIENTPROTOCOL_CURSE_DELAY_RECEIVED:
+      U61_Menu::display_info("Receiving time...");	      
       if ((result=recv_time())==U61_PROTOCOL_DONE)
 	{
 	  stage=U61_CLIENTPROTOCOL_TIME_RECEIVED;
@@ -302,6 +334,7 @@ int U61_ClientProtocol::recv_next_packet()
 	}
       break;
     case U61_PROTOCOL_READY_SENT:
+      U61_Menu::display_info("Waiting ready signal...");	      
       if ((result=recv_ready(connection))==U61_PROTOCOL_DONE)
 	{
 	  stage=U61_PROTOCOL_READY_RECEIVED;
@@ -313,39 +346,6 @@ int U61_ClientProtocol::recv_next_packet()
   return result;
 }
 
-/*--------------------------------------------------------------------------*/
-/*
- * enter the server search state
- */
-int U61_ClientProtocol::start_searching()
-{
-  int result=U61_PROTOCOL_LATER;
-
-  /*
-   * We mark the protocol as being "connecting". This is usefull because
-   * the "abort" procedure is different
-   */
-  connecting=true;
-
-  try 
-    {
-      if (broadcast)
-	{
-	  // commented out because the ClanLib functions are broken
-	  //	  CL_Network::find_sessions_broadcast(U61_PROTOCOL_GAME_ID,port);
-	}
-      result=U61_PROTOCOL_DONE;
-    }
-  catch (CL_Error e)
-    {
-      U61_LOG_WARNING("Find error : "<<e.message.c_str()<<"");
-      diagnostic="unable to broadcast";
-      result=U61_PROTOCOL_FAILED;
-    }
-
-  return result;
-}
-
 /*--------------------------------------------------------------------------*/
 /*
  * tries to find out if a game has been found on the server
@@ -355,100 +355,194 @@ int U61_ClientProtocol::connect_to_server()
 {
   int result=U61_PROTOCOL_LATER;
 
-  session=NULL;
-  if (broadcast)
+  connect_mutex.enter();
+
+  if (!connection)
     {
-      try 
+      if (connect_error)
+	{
+	  U61_MACRO_STRCPY(diagnostic,"Unable to connect on server");
+	  result=U61_PROTOCOL_FAILED;
+	}
+      else
 	{
 	  /*
-	   * first case, we have broadcasted servers on the LAN, so
-	   * now we connect on the first of these "broadcasted"
-	   * servers
+	   * connection is NULL, we're not connected yet...
 	   */
-	  // commented out because the ClanLib functions are broken
-	  //session=CL_Network::receive_session_found
-	  //  (U61_CLIENTPROTOCOL_BROADCAST_TIMEOUT);
-	}
-      catch (CL_Error e)
-	{
-	  U61_LOG_WARNING("Broadcast error : "<<e.message.c_str()<<"");
-	  diagnostic="unable to find server";
-	  result=U61_PROTOCOL_FAILED;
+	  if (!connecting)
+	    {
+	      stop_connect_thread(true);
 
-	  session=NULL;
+	      try 
+		{
+		  connecting = true;
+		  connect_thread=CL_Thread::create(&connect_to_server_callback, (void *) this);
+		  if (connect_thread)
+		    {
+		      connect_thread->start();
+		      U61_LOG_DEBUG("U61_ClientProtocol connect thread started");
+		    }
+		  else
+		    {
+		      U61_LOG_DEBUG("Could not create CL_Thread object");
+		      U61_MACRO_STRCPY(diagnostic,"Could not create connect thread");
+		      stop_connect_thread(true);
+		      result=U61_PROTOCOL_FAILED;
+		    }
+		}
+	      catch (CL_Error e)
+		{
+		  U61_LOG_DEBUG("Could not launch U61_ClientProtocol connect thread");
+		  U61_MACRO_STRCPY(diagnostic,"Could not start connect thread");
+		  stop_connect_thread(true);
+		  result=U61_PROTOCOL_FAILED;
+		}
+	    }
 	}
     }
   else
-    {
-      try 
-	{
-	  /*
-	   * second case, we do not broadcast but call a given
-	   * server, specified by name or ip
-	   */
-	  session=new CL_NetSession(U61_CONST_PROGRAM,server_id,port);
-	}
-      catch (CL_Error e)
-	{
-	  U61_LOG_WARNING("Connect error : "<<e.message.c_str()<<"");
-	  diagnostic="unable to connect on server";
-	  result=U61_PROTOCOL_FAILED;
-
-	  session=NULL;
-	}
-    }
-
-  if (session!=NULL)
     {
       /*
-       * OK, a game has been found
+       * connection is not NULL, we consider we're connected
        */
-      join_delay_start=U61_Time::for_effect();
+      stop_connect_thread(false);
       result=U61_PROTOCOL_DONE;
-    }
+    }    
+
+  connect_mutex.leave();
 
   return result;
 }
 
 /*--------------------------------------------------------------------------*/
 /*
- * This function just waits one second before trying to send stuff
- * to the server, this is for ClanLib to find the time to give
- * network access on the right channel
+ * Callnack called by connect_to_server in a separate thread
  */
-int U61_ClientProtocol::join_delay()
+int U61_ClientProtocol::connect_to_server_callback(void *data)
 {
-  int result=U61_PROTOCOL_LATER;
+  int result=0;
+  U61_ClientProtocol *client_protocol;
+  U61_BufferedSocket *sock;
+  U61_Connection *connection=NULL;
 
-  if (U61_Time::for_effect()>join_delay_start+U61_CLIENTPROTOCOL_JOIN_DELAY)
+  U61_LOG_DEBUG("Client connection thread started");
+
+  client_protocol=(U61_ClientProtocol *) data;
+
+  U61_LOG_DEBUG("Connecting on \""<<client_protocol->server_id<<":"<<client_protocol->port<<"\".");
+
+  sock=new U61_BufferedSocket();
+  if (sock) 
     {
-      channel_to_test=0;
-      result=U61_PROTOCOL_DONE;
+      if (sock->connect(client_protocol->server_id,client_protocol->port))
+	{
+	  result = 1;
+	  /*
+	   * The connection object will handle the destruction of
+	   * the socket objet
+	   */
+	  connection=new U61_Connection(sock);
+	}
+      else
+	{
+	  delete sock;
+	}
     }
 
+  if (connection != NULL)
+    {
+      connection->authorize_all();
+    }
+
+  client_protocol->connect_mutex.enter();
+
+  client_protocol->connect_error = (connection==NULL);
+  client_protocol->connection = connection;
+  client_protocol->connecting = false;
+
+  client_protocol->connect_mutex.leave();
+
+  U61_LOG_DEBUG("Client connection thread finished");
+
   return result;
 }
 
 /*--------------------------------------------------------------------------*/
 /*
- * So now we search a free channel in order to open a connection object
+ * Sends the password
  */
-int U61_ClientProtocol::find_channel()
+int U61_ClientProtocol::send_password()
 {
   int result=U61_PROTOCOL_LATER;
-  CL_NetComputer server;
+  U61_Checksum checksum;
+  char *ascii_sig;
 
-  if (session->is_writable(channel_to_test) 
-      && session->is_readable(channel_to_test))
+  if (connection!=NULL)
     {
-      server=session->get_server();
-      connection=new U61_Connection(session,server,
-                                    channel_to_test,false);
-      result=U61_PROTOCOL_DONE;
+      if (U61_Global::config.use_password)
+	{
+	  checksum.calc(U61_Global::config.password,
+			strlen(U61_Global::config.password));
+	  ascii_sig=checksum.get_ascii_sig();
+	}
+      else
+	{
+	  ascii_sig="";
+	}
+
+      if (connection->send(ascii_sig, strlen(ascii_sig)))
+	{
+	  /*
+	   * We set password error to true, so from now
+	   * any error will be interpreted as a bad
+	   * password. This is necessary for the server does
+	   * not answer "password is wrong" but simply closes
+	   * the connection...
+	   */
+	  password_error=true;
+
+	  result=U61_PROTOCOL_DONE;
+	}
     }
   else
     {
-      channel_to_test++;
+      result=U61_PROTOCOL_FAILED;
+    }
+ 
+  return result;
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * Receives the answer "password is correct".
+ */
+int U61_ClientProtocol::recv_password_ok()
+{
+  int result=U61_PROTOCOL_LATER;
+  char password_ok[U61_CONST_STRING_SIZE];
+
+  if (connection!=NULL) 
+    {
+      if (connection->peek_str())
+	{
+	  if (connection->recv(password_ok,sizeof(password_ok)-1))
+	    {
+	      if (strcmp(password_ok,U61_PROTOCOL_OK_MESSAGE)==0)
+		{
+		  password_error=false;
+		  result=U61_PROTOCOL_DONE;		  
+		  U61_LOG_DEBUG("Password is OK");
+		}
+	      else
+		{
+		  result=U61_PROTOCOL_FAILED;		  
+		}
+	    }
+	}      
+    }
+  else
+    {
+      result=U61_PROTOCOL_FAILED;
     }
 
   return result;
@@ -463,16 +557,23 @@ int U61_ClientProtocol::recv_script_name()
 {
   int result=U61_PROTOCOL_LATER;
 
-  if (connection->peek())
+  if (connection!=NULL) 
     {
-      if (connection->recv(U61_Global::remote_script.name_buffer,
-			   sizeof(U61_Global::remote_script.name_buffer)-1))
+      if (connection->peek_str())
 	{
-	  U61_Platform::strip_remote_script_path
-	    (U61_Global::remote_script.name_buffer);
-	  result=U61_PROTOCOL_DONE;
-          U61_LOG_DEBUG("script_name=\""<<U61_Global::remote_script.name_buffer<<"\"");
-	}
+	  if (connection->recv(U61_Global::remote_script.name_buffer,
+			       sizeof(U61_Global::remote_script.name_buffer)-1))
+	    {
+	      U61_Platform::strip_remote_script_path
+		(U61_Global::remote_script.name_buffer);
+	      result=U61_PROTOCOL_DONE;
+	      U61_LOG_DEBUG("script_name=\""<<U61_Global::remote_script.name_buffer<<"\"");
+	    }
+	}      
+    }
+  else
+    {
+      result=U61_PROTOCOL_FAILED;
     }
 
   return result;
@@ -488,14 +589,21 @@ int U61_ClientProtocol::recv_script()
 {
   int result=U61_PROTOCOL_LATER;
 
-  if (connection->peek())
+  if (connection!=NULL) 
     {
-      if (connection->recv(U61_Global::remote_script.code_buffer,
-			   sizeof(U61_Global::remote_script.code_buffer)-1))
+      if (connection->peek_str())
 	{
-	  result=U61_PROTOCOL_DONE;
+	  if (connection->recv(U61_Global::remote_script.code_buffer,
+			       sizeof(U61_Global::remote_script.code_buffer)-1))
+	    {
+	      result=U61_PROTOCOL_DONE;
+	    }
 	}
     }
+  else
+    {
+      result=U61_PROTOCOL_FAILED;
+    }
 
   return result;
 }
@@ -508,14 +616,22 @@ int U61_ClientProtocol::recv_id0()
 {
   int result=U61_PROTOCOL_LATER;
 
-  if (connection->peek())
+  if (connection!=NULL) 
     {
-      if (connection->recv(&id0))
+      if (connection->peek_int())
 	{
-	  result=U61_PROTOCOL_DONE;
-	  U61_LOG_DEBUG("id0="<<id0);
+	  if (connection->recv(&(id[0])))
+	    {
+	      connection->authorize_player_id(id[0],false);
+	      result=U61_PROTOCOL_DONE;
+	      U61_LOG_DEBUG("id0="<<id[0]);
+	    }
 	}
     }
+  else
+    {
+      result=U61_PROTOCOL_FAILED;
+    }
 
   return result;
 }
@@ -528,14 +644,22 @@ int U61_ClientProtocol::recv_id1()
 {
   int result=U61_PROTOCOL_LATER;
 
-  if (connection->peek())
+  if (connection!=NULL) 
     {
-      if (connection->recv(&id1))
+      if (connection->peek_int())
 	{
-	  result=U61_PROTOCOL_DONE;
-	  U61_LOG_DEBUG("id1="<<id1);
+	  if (connection->recv(&(id[1])))
+	    {
+	      connection->authorize_player_id(id[1],false);
+	      result=U61_PROTOCOL_DONE;
+	      U61_LOG_DEBUG("id1="<<id[1]);
+	    }
 	}
     }
+  else
+    {
+      result=U61_PROTOCOL_FAILED;
+    }
 
   return result;
 }
@@ -548,14 +672,21 @@ int U61_ClientProtocol::recv_initial_speed()
 {
   int result=U61_PROTOCOL_LATER;
 
-  if (connection->peek())
+  if (connection!=NULL) 
     {
-      if (connection->recv(&initial_speed))
+      if (connection->peek_int())
 	{
-	  result=U61_PROTOCOL_DONE;
-	  U61_LOG_DEBUG("initial_speed="<<initial_speed);
+	  if (connection->recv(&initial_speed))
+	    {
+	      result=U61_PROTOCOL_DONE;
+	      U61_LOG_DEBUG("initial_speed="<<initial_speed);
+	    }
 	}
     }
+  else
+    {
+      result=U61_PROTOCOL_FAILED;
+    }
 
   return result;
 }
@@ -568,14 +699,21 @@ int U61_ClientProtocol::recv_acceleration()
 {
   int result=U61_PROTOCOL_LATER;
 
-  if (connection->peek())
+  if (connection!=NULL) 
     {
-      if (connection->recv(&acceleration))
+      if (connection->peek_int())
 	{
-	  result=U61_PROTOCOL_DONE;
-	  U61_LOG_DEBUG("acceleration="<<acceleration);
+	  if (connection->recv(&acceleration))
+	    {
+	      result=U61_PROTOCOL_DONE;
+	      U61_LOG_DEBUG("acceleration="<<acceleration);
+	    }
 	}
     }
+  else
+    {
+      result=U61_PROTOCOL_FAILED;
+    }
 
   return result;
 }
@@ -588,14 +726,21 @@ int U61_ClientProtocol::recv_curse_delay()
 {
   int result=U61_PROTOCOL_LATER;
 
-  if (connection->peek())
+  if (connection!=NULL) 
     {
-      if (connection->recv(&curse_delay))
+      if (connection->peek_int())
 	{
-	  result=U61_PROTOCOL_DONE;
-	  U61_LOG_DEBUG("curse_delay="<<curse_delay);
+	  if (connection->recv(&curse_delay))
+	    {
+	      result=U61_PROTOCOL_DONE;
+	      U61_LOG_DEBUG("curse_delay="<<curse_delay);
+	    }
 	}
     }
+  else
+    {
+      result=U61_PROTOCOL_FAILED;
+    }
 
   return result;
 }
@@ -608,14 +753,21 @@ int U61_ClientProtocol::recv_time()
 {
   int result=U61_PROTOCOL_LATER;
 
-  if (connection->peek())
+  if (connection!=NULL) 
     {
-      if (connection->recv(&time))
+      if (connection->peek_int())
 	{
-	  result=U61_PROTOCOL_DONE;
-	  U61_LOG_DEBUG("time="<<time);
+	  if (connection->recv(&time))
+	    {
+	      result=U61_PROTOCOL_DONE;
+	      U61_LOG_DEBUG("time="<<time);
+	    }
 	}
     }
+  else
+    {
+      result=U61_PROTOCOL_FAILED;
+    }
 
   return result;
 }
@@ -626,21 +778,56 @@ int U61_ClientProtocol::recv_time()
  */
 void U61_ClientProtocol::start_game()
 {
-  /*
-   * We mark the protocol as not being connecting to the server
-   */
-  connecting=false;
+  stop_connect_thread(false);
 
   U61_Global::game.do_network_join(U61_Global::remote_script.code_buffer,
-				   id0,id1,
+				   id[0],id[1],
 				   initial_speed,acceleration,curse_delay,
 				   time);
 }
 
+/*--------------------------------------------------------------------------*/
+/*
+ * Stops the connection thread
+ */
+void U61_ClientProtocol::stop_connect_thread(bool close_connection)
+{
+  connect_mutex.enter();
 
+  if (connect_thread!=NULL)
+    {
+      U61_LOG_DEBUG("Stopping client connection thread");
 
+      //#ifdef U61_DEF_WIN32
+      /*
+       * Threads suck on Win98, so we disable the "force terminate"
+       * feature on this platform and wait for the thread to
+       * terminate alone instead
+       */
+      //      connect_thread->wait();
+      //#else
+      connect_thread->terminate();
+      //#endif
 
+      U61_LOG_DEBUG("Client connection thread stopped");
 
+      delete connect_thread;
+      connect_thread = NULL;
+    }
 
+  if (close_connection)
+    {
+      if (connection!=NULL)
+	{
+	  delete connection;
+	  connection=NULL;
+	}
+    }
+      
+  connecting = false;
+  connect_error = false;
+
+  connect_mutex.leave();
+}
 
 
