@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -41,23 +41,24 @@
 
 
 /*---------------------------------------------------------------------------
- includes
- ---------------------------------------------------------------------------*/
+  includes
+  ---------------------------------------------------------------------------*/
 
 #include "config.h"
 #include "platform.h"
 #include "script.h"
-#include "debug.h"
+#include "log.h"
 #include "sound.h"
 #include "music.h"
 #include "global.h"
 #include "version.h"
 #include "utils.h"
 #include "inputmonitor.h"
+#include "macro.h"
 
 /*---------------------------------------------------------------------------
- constants
- ---------------------------------------------------------------------------*/
+  constants
+  ---------------------------------------------------------------------------*/
 
 #define U61_CONFIG_ID_PROGRAM "cfg_program"
 #define U61_CONFIG_ID_VERSION "cfg_version"
@@ -75,7 +76,10 @@
 
 #define U61_CONFIG_ID_SERVER_ID "cfg_server_id"
 #define U61_CONFIG_ID_NETWORK_PORT "cfg_network_port"
-#define U61_CONFIG_ID_NETWORK_CHANNEL "cfg_network_channel"
+#define U61_CONFIG_ID_SERVER_COMMENT "cfg_server_comment"
+
+#define U61_CONFIG_ID_USE_PASSWORD "cfg_use_password"
+#define U61_CONFIG_ID_PASSWORD "cfg_password"
 
 #define U61_CONFIG_ID_INITIAL_SPEED "cfg_initial_speed"
 #define U61_CONFIG_ID_ACCELERATION "cfg_acceleration"
@@ -87,6 +91,7 @@
 #define U61_CONFIG_ID_MAX_FPS "cfg_max_fps"
 
 #define U61_CONFIG_ID_NETWORK_ANTICIPATION "cfg_network_anticipation"
+#define U61_CONFIG_ID_NETWORK_CONSOLE_DUMP "cfg_network_console_dump"
 
 #define U61_CONFIG_ID_FIRST_REPEAT_MOVE "first_repeat_move"
 #define U61_CONFIG_ID_STANDARD_REPEAT_MOVE "standard_repeat_move"
@@ -115,8 +120,8 @@
 #define U61_CONFIG_ID_LABEL "label"
 
 /*---------------------------------------------------------------------------
- functions
- ---------------------------------------------------------------------------*/
+  functions
+  ---------------------------------------------------------------------------*/
 
 /*--------------------------------------------------------------------------*/
 /* 
@@ -190,7 +195,7 @@ bool U61_Config::read()
   else
     {
       U61_LOG_MESSAGE("Loading config from \""<<filename<<"\".");
-      
+
       if (!U61_Script::do_file(filename))
 	{
 	  ok=false;
@@ -241,12 +246,12 @@ void U61_Config::reset()
   char default_data_file[U61_CONST_STRING_SIZE];
   char default_script_file[U61_CONST_STRING_SIZE];
 
-  strcpy(default_data_dir,U61_Platform::get_builtin_theme_path());
-  strcpy(default_script_dir,U61_Platform::get_builtin_script_path());
-  strcpy(default_data_file,default_data_dir);
-  strcat(default_data_file,"default.dat");
-  strcpy(default_script_file,default_script_dir);
-  strcat(default_script_file,"classic.lua");
+  U61_MACRO_STRCPY(default_data_dir,U61_Platform::get_builtin_theme_path());
+  U61_MACRO_STRCPY(default_script_dir,U61_Platform::get_builtin_script_path());
+  U61_MACRO_STRCPY(default_data_file,default_data_dir);
+  U61_MACRO_STRCAT(default_data_file,"default.dat");
+  U61_MACRO_STRCPY(default_script_file,default_script_dir);
+  U61_MACRO_STRCAT(default_script_file,"classic.lua");
 
   launch_count=read_int(U61_CONFIG_ID_LAUNCH_COUNT,0);
   read_str(data_dir,U61_CONFIG_ID_DATA_DIR,default_data_dir);
@@ -258,9 +263,12 @@ void U61_Config::reset()
   show_system_info=read_bool(U61_CONFIG_ID_SHOW_SYSTEM_INFO,false);
 
   read_str(server_id,U61_CONFIG_ID_SERVER_ID,"127.0.0.1");
-  network_port=read_int(U61_CONFIG_ID_NETWORK_PORT,8061);
-  network_channel=read_int(U61_CONFIG_ID_NETWORK_CHANNEL,1);
+  network_port=read_int(U61_CONFIG_ID_NETWORK_PORT,U61_CONFIG_DEFAULT_PORT);
+  read_str(server_comment,U61_CONFIG_ID_SERVER_COMMENT,"");
         
+  use_password=read_bool(U61_CONFIG_ID_USE_PASSWORD,false);
+  read_str(password,U61_CONFIG_ID_PASSWORD,"hello");
+
   initial_speed=read_int(U61_CONFIG_ID_INITIAL_SPEED,5);
   acceleration=read_int(U61_CONFIG_ID_ACCELERATION,5);
   curse_delay=read_int(U61_CONFIG_ID_CURSE_DELAY,20);
@@ -268,9 +276,10 @@ void U61_Config::reset()
   sound_volume=read_int(U61_CONFIG_ID_SOUND_VOLUME,U61_SOUND_VOLUME_MAX/2);
   music_volume=read_int(U61_CONFIG_ID_MUSIC_VOLUME,U61_MUSIC_VOLUME_MAX/2);
 
-  max_fps=read_int(U61_CONFIG_ID_MAX_FPS,18);
+  max_fps=read_int(U61_CONFIG_ID_MAX_FPS,25);
 
   network_anticipation=read_bool(U61_CONFIG_ID_NETWORK_ANTICIPATION,false);
+  network_console_dump=read_bool(U61_CONFIG_ID_NETWORK_CONSOLE_DUMP,false);
 
   for (pl=0;pl<U61_CONFIG_NB_PLAYERS;++pl)
     {
@@ -345,7 +354,7 @@ void U61_Config::reset()
   for (k=0;k<U61_INPUTMONITOR_NUM_KEYS_SCANCODE;++k)
     {
       key[k].ascii=U61_KEYDEF_ASCII_UNKNOWN;
-      strcpy(key[k].label,U61_KEYDEF_LABEL_UNKNOWN);
+      U61_MACRO_STRCPY(key[k].label,U61_KEYDEF_LABEL_UNKNOWN);
     }
 
   key[CL_KEY_A].ascii=read_key_int(CL_KEY_A,U61_CONFIG_ID_ASCII,'a');
@@ -660,7 +669,10 @@ bool U61_Config::write()
 
 	  ok=ok && write_str(file,U61_CONFIG_ID_SERVER_ID, server_id);
 	  ok=ok && write_int(file,U61_CONFIG_ID_NETWORK_PORT, network_port);
-	  ok=ok && write_int(file,U61_CONFIG_ID_NETWORK_CHANNEL, network_channel);
+	  ok=ok && write_str(file,U61_CONFIG_ID_SERVER_COMMENT, server_comment);
+
+	  ok=ok && write_int(file,U61_CONFIG_ID_USE_PASSWORD, use_password);
+	  ok=ok && write_str(file,U61_CONFIG_ID_PASSWORD, password);
 
 	  ok=ok && write_int(file,U61_CONFIG_ID_INITIAL_SPEED, initial_speed);
 	  ok=ok && write_int(file,U61_CONFIG_ID_ACCELERATION, acceleration);
@@ -672,6 +684,7 @@ bool U61_Config::write()
 	  ok=ok && write_int(file,U61_CONFIG_ID_MAX_FPS, max_fps);
 
 	  ok=ok && write_int(file,U61_CONFIG_ID_NETWORK_ANTICIPATION, network_anticipation);
+	  ok=ok && write_int(file,U61_CONFIG_ID_NETWORK_CONSOLE_DUMP, network_console_dump);
 
 	  for (pl=0;pl<U61_CONFIG_NB_PLAYERS;++pl)
 	    {
@@ -755,13 +768,16 @@ bool U61_Config::load_script()
   return ok;
 }
 
+typedef char U61_TYPE_STRING[U61_CONST_STRING_SIZE];
+
 /*--------------------------------------------------------------------------*/
 /*
  * loads a string entry from the config file
  */
-void U61_Config::read_str(char *buf, char *name,char *def)
+void U61_Config::read_str(char buf[U61_CONST_STRING_SIZE], 
+			  char *name,char *def)
 {
-  strcpy(buf,U61_Script::get_global_str(name,def));
+  U61_MACRO_STRNCPY(buf,U61_Script::get_global_str(name,def),U61_CONST_STRING_SIZE);
 }
 
 /*--------------------------------------------------------------------------*/
@@ -806,6 +822,7 @@ bool U61_Config::write_str(FILE *file,char *name,char *value)
       switch (*value)
 	{
 	case '\\':
+	case '\"':
 	  buffer_value[pos]='\\';
 	  pos++;
 	  /*
@@ -820,10 +837,10 @@ bool U61_Config::write_str(FILE *file,char *name,char *value)
     }
   buffer_value[pos]=0;
 
-  strcpy(buffer,name);
-  strcat(buffer," = \"");
-  strcat(buffer,buffer_value);
-  strcat(buffer,"\"\n");
+  U61_MACRO_STRCPY(buffer,name);
+  U61_MACRO_STRCAT(buffer," = \"");
+  U61_MACRO_STRCAT(buffer,buffer_value);
+  U61_MACRO_STRCAT(buffer,"\"\n");
 
   ok=(fwrite(buffer,strlen(buffer),1,file)>0);
 
@@ -840,12 +857,12 @@ bool U61_Config::write_int(FILE *file,char *name,int value)
   char buffer2[20];
   bool ok;
 
-  sprintf(buffer2,"%d",value);
+  U61_MACRO_SPRINTF1(buffer2,"%d",value);
 
-  strcpy(buffer,name);
-  strcat(buffer," = ");
-  strcat(buffer,buffer2);
-  strcat(buffer,"\n");
+  U61_MACRO_STRCPY(buffer,name);
+  U61_MACRO_STRCAT(buffer," = ");
+  U61_MACRO_STRCAT(buffer,buffer2);
+  U61_MACRO_STRCAT(buffer,"\n");
 
   ok=(fwrite(buffer,strlen(buffer),1,file)>0);
 
@@ -856,12 +873,13 @@ bool U61_Config::write_int(FILE *file,char *name,int value)
 /*
  * loads a string entry from the config file, for a specific player
  */
-void U61_Config::read_player_str(char *buf,int pl,char *name,char *def)
+void U61_Config::read_player_str(char buf[U61_CONST_STRING_SIZE],
+				 int pl,char *name,char *def)
 {
   char buffer[U61_CONST_STRING_SIZE];
 
-  sprintf(buffer,U61_CONFIG_ID_PLAYER,pl);
-  strcat(buffer,name);
+  U61_MACRO_SPRINTF1(buffer,U61_CONFIG_ID_PLAYER,pl);
+  U61_MACRO_STRCAT(buffer,name);
 
   read_str(buf,buffer,def);
 }
@@ -874,8 +892,8 @@ int U61_Config::read_player_int(int pl,char *name,int def)
 {
   char buffer[U61_CONST_STRING_SIZE];
 
-  sprintf(buffer,U61_CONFIG_ID_PLAYER,pl);
-  strcat(buffer,name);
+  U61_MACRO_SPRINTF1(buffer,U61_CONFIG_ID_PLAYER,pl);
+  U61_MACRO_STRCAT(buffer,name);
 
   return read_int(buffer,def);
 }
@@ -888,8 +906,8 @@ bool U61_Config::read_player_bool(int pl,char *name,bool def)
 {
   char buffer[U61_CONST_STRING_SIZE];
 
-  sprintf(buffer,U61_CONFIG_ID_PLAYER,pl);
-  strcat(buffer,name);
+  U61_MACRO_SPRINTF1(buffer,U61_CONFIG_ID_PLAYER,pl);
+  U61_MACRO_STRCAT(buffer,name);
 
   return read_bool(buffer,def);
 }
@@ -902,8 +920,8 @@ bool U61_Config::write_player_str(int pl,FILE *file,char *name,char *value)
 {
   char buffer[U61_CONST_STRING_SIZE];
 
-  sprintf(buffer,U61_CONFIG_ID_PLAYER,pl);
-  strcat(buffer,name);
+  U61_MACRO_SPRINTF1(buffer,U61_CONFIG_ID_PLAYER,pl);
+  U61_MACRO_STRCAT(buffer,name);
 
   return write_str(file,buffer,value);
 } 
@@ -916,8 +934,8 @@ bool U61_Config::write_player_int(int pl,FILE *file,char *name,int value)
 {
   char buffer[U61_CONST_STRING_SIZE];
 
-  sprintf(buffer,U61_CONFIG_ID_PLAYER,pl);
-  strcat(buffer,name);
+  U61_MACRO_SPRINTF1(buffer,U61_CONFIG_ID_PLAYER,pl);
+  U61_MACRO_STRCAT(buffer,name);
 
   return write_int(file,buffer,value);
 } 
@@ -926,12 +944,13 @@ bool U61_Config::write_player_int(int pl,FILE *file,char *name,int value)
 /*
  * loads a string entry from the config file, for a specific key
  */
-void U61_Config::read_key_str(char *buf,int k,char *name,char *def)
+void U61_Config::read_key_str(char buf[U61_CONST_STRING_SIZE],
+			      int k,char *name,char *def)
 {
   char buffer[U61_CONST_STRING_SIZE];
 
-  sprintf(buffer,U61_CONFIG_ID_KEY,k);
-  strcat(buffer,name);
+  U61_MACRO_SPRINTF1(buffer,U61_CONFIG_ID_KEY,k);
+  U61_MACRO_STRCAT(buffer,name);
 
   read_str(buf,buffer,def);
 }
@@ -944,8 +963,8 @@ int U61_Config::read_key_int(int k,char *name,int def)
 {
   char buffer[U61_CONST_STRING_SIZE];
 
-  sprintf(buffer,U61_CONFIG_ID_KEY,k);
-  strcat(buffer,name);
+  U61_MACRO_SPRINTF1(buffer,U61_CONFIG_ID_KEY,k);
+  U61_MACRO_STRCAT(buffer,name);
 
   return read_int(buffer,def);
 }
@@ -958,8 +977,8 @@ bool U61_Config::read_key_bool(int k,char *name,bool def)
 {
   char buffer[U61_CONST_STRING_SIZE];
 
-  sprintf(buffer,U61_CONFIG_ID_KEY,k);
-  strcat(buffer,name);
+  U61_MACRO_SPRINTF1(buffer,U61_CONFIG_ID_KEY,k);
+  U61_MACRO_STRCAT(buffer,name);
 
   return read_bool(buffer,def);
 }
@@ -972,8 +991,8 @@ bool U61_Config::write_key_str(int k,FILE *file,char *name,char *value)
 {
   char buffer[U61_CONST_STRING_SIZE];
 
-  sprintf(buffer,U61_CONFIG_ID_KEY,k);
-  strcat(buffer,name);
+  U61_MACRO_SPRINTF1(buffer,U61_CONFIG_ID_KEY,k);
+  U61_MACRO_STRCAT(buffer,name);
 
   return write_str(file,buffer,value);
 } 
@@ -986,8 +1005,8 @@ bool U61_Config::write_key_int(int k,FILE *file,char *name,int value)
 {
   char buffer[U61_CONST_STRING_SIZE];
 
-  sprintf(buffer,U61_CONFIG_ID_KEY,k);
-  strcat(buffer,name);
+  U61_MACRO_SPRINTF1(buffer,U61_CONFIG_ID_KEY,k);
+  U61_MACRO_STRCAT(buffer,name);
 
   return write_int(file,buffer,value);
 } 
