@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -39,8 +39,8 @@
  *              settings in general
  */
 
-#ifndef INC_U61_CONFIG
-#define INC_U61_CONFIG
+#ifndef U61_INC_CONFIG
+#define U61_INC_CONFIG
 
 /*---------------------------------------------------------------------------
  includes
@@ -58,7 +58,8 @@
  constants
  ---------------------------------------------------------------------------*/
 
-#define U61_CONFIG_NB_PLAYERS 4
+#define U61_CONFIG_NB_PLAYERS      4
+#define U61_CONFIG_DEFAULT_PORT 8061
 
 /*---------------------------------------------------------------------------
  classes
@@ -89,7 +90,10 @@ class U61_Config
 
   char server_id[U61_CONST_STRING_SIZE];
   int network_port;
-  int network_channel;
+  char server_comment[U61_CONST_STRING_SIZE];
+
+  bool use_password;
+  char password[U61_CONST_STRING_SIZE];
     
   int initial_speed;
   int acceleration;
@@ -101,23 +105,24 @@ class U61_Config
   int max_fps;
 
   bool network_anticipation;
+  bool network_console_dump;
 
   U61_PlayerConfig player[U61_CONFIG_NB_PLAYERS];
   U61_KeyConfig key[CL_NUM_KEYS];
 
 
  protected:
-  void read_str(char *buf,char *name,char *def);
+  void read_str(char buf[U61_CONST_STRING_SIZE],char *name,char *def);
   int read_int(char *name,int def);
   bool read_bool(char *name,bool def);
   bool write_str(FILE *file,char *name, char *value); 
   bool write_int(FILE *file,char *name,int value);
-  void read_player_str(char *buf,int pl,char *name,char *def);
+  void read_player_str(char buf[U61_CONST_STRING_SIZE],int pl,char *name,char *def);
   int read_player_int(int pl,char *name,int def);
   bool read_player_bool(int pl,char *name,bool def);
   bool write_player_str(int pl,FILE *file,char *name, char *value); 
   bool write_player_int(int pl,FILE *file,char *name,int value);
-  void read_key_str(char *buf,int k,char *name,char *def);
+  void read_key_str(char buf[U61_CONST_STRING_SIZE],int k,char *name,char *def);
   int read_key_int(int k,char *name,int def);
   bool read_key_bool(int k,char *name,bool def);
   bool write_key_str(int k,FILE *file,char *name, char *value); 
