@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -33,8 +33,8 @@
  *              no peculiar class, such as the standard string buffer size
  */
 
-#ifndef INC_U61_CONST
-#define INC_U61_CONST
+#ifndef U61_INC_CONST
+#define U61_INC_CONST
 
 /*---------------------------------------------------------------------------
  defines
@@ -43,5 +43,9 @@
 #define U61_CONST_STRING_SIZE 1000
 #define U61_CONST_SCRIPT_SIZE 1000000
 #define U61_CONST_PROGRAM "u61"
+#define U61_CONST_METASERVER_GAME_SIZE 25
+#define U61_CONST_METASERVER_VERSION_SIZE 25
+#define U61_CONST_METASERVER_ADDRESS_SIZE 15
+#define U61_CONST_METASERVER_COMMENT_SIZE 100
 
 #endif
