@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -45,10 +45,11 @@
 
 #include "data.h"
 #include "const.h"
-#include "debug.h"
+#include "log.h"
 #include "platform.h"
 #include "version.h"
 #include "utils.h"
+#include "macro.h"
 
 /*---------------------------------------------------------------------------
  constants
@@ -131,6 +132,7 @@
 #define U61_DATA_ID_SQUARE_CURSE "curse"
 #define U61_DATA_ID_SQUARE_ANTICIPATION "anticipation"
 #define U61_DATA_ID_SQUARE_EXPLOSION "explosion"
+#define U61_DATA_ID_SQUARE_FILLER "filler"
 
 #define U61_DATA_ID_MAP_BIG "map/big_"
 #define U61_DATA_ID_MAP_SMALL "map/small_"
@@ -142,6 +144,12 @@
 #define U61_DATA_ID_INFO_BIG "info/big"
 #define U61_DATA_ID_INFO_SMALL "info/small"
 
+#define U61_DATA_ID_MOUSE_CURSOR "mouse/cursor"
+
+
+#define U61_DATA_MOUSE_CURSOR_NB_FRAMES         16
+#define U61_DATA_MOUSE_CURSOR_FRAME_DELAY       30
+
 /*---------------------------------------------------------------------------
  functions
  ---------------------------------------------------------------------------*/
@@ -152,15 +160,100 @@
  */
 U61_Data::U61_Data()
 {
+  int i,j;
+
+  /*
+   * Resetting public stuff
+   */
+  screen_w = 0;
+  screen_h = 0;
+
+  nb_map = 0;
+  nb_music = 0;
+
+  for (i=0;i<U61_DATA_NB_SIZE;++i)
+    {
+      square_w[i] = 0;
+      square_h[i] = 0;
+      map_w[i] = 0;
+      map_h[i] = 0;
+      info_w[i] = 0;
+      info_h[i] = 0;
+      player_w[i] = 0;
+      player_h[i] = 0;
+      
+      offset_map_x[i] = 0;
+      offset_map_y[i] = 0;
+      offset_info_x[i] = 0;
+      offset_info_y[i] = 0;
+      offset_squares_x[i] = 0;
+      offset_squares_y[i] = 0;
+      offset_name_x[i] = 0;
+      offset_name_y[i] = 0;
+      offset_score_x[i] = 0;
+      offset_score_y[i] = 0;
+      offset_target_x[i] = 0;
+      offset_target_y[i] = 0;
+      offset_preview_x[i] = 0;
+      offset_preview_y[i] = 0;
+      offset_curses_x[i] = 0;
+      offset_curses_y[i] = 0;
+      offset_antidotes_x[i] = 0;
+      offset_antidotes_y[i] = 0;
+      offset_next_curse_x[i] = 0;
+      offset_next_curse_y[i] = 0;
+
+      font_info[i] = NULL;
+      font_symbol[i] = NULL;
+
+      for (j=0;j<U61_DATA_NB_SQUARE_TYPE;++j)
+	{
+	  square[j][i] = NULL;
+	}
+
+      curse[i] = NULL;
+      anticipation[i] = NULL;
+      explosion[i] = NULL;
+      filler[i] = NULL;
+
+      for (j=0;j<U61_DATA_NB_MAP_MAX;++j)
+	{
+	  map[j][i] = NULL;
+	}
+
+      info[i] = NULL;
+    }
+
+  font_menu = NULL;
+
+  sound_game_start = NULL;
+  sound_menu_move = NULL;
+  sound_menu_validate = NULL;
+  sound_block_touch = NULL;
+  sound_block_pattern = NULL;
+  sound_player_start = NULL;
+  sound_curse_receive = NULL;
+  sound_player_loose = NULL;
+
+  back = NULL;
+  shade = NULL;
+
+  mouse_cursor=NULL;
+
+  /*
+   * Protected stuff
+   */
+
   manager=NULL;
+  packed_data=true;
   
   /*
    * We keep the initial working directory in memory, to
    * chdir back to it later, if needed...
    */
-  strcpy(initial_dir,U61_Platform::getcwd());
-  strcpy(data_dir,U61_Platform::getcwd());
-  strcpy(data_file,"");
+  U61_MACRO_STRCPY(initial_dir,U61_Platform::getcwd());
+  U61_MACRO_STRCPY(data_dir,U61_Platform::getcwd());
+  U61_MACRO_STRCPY(data_file,"");
 }
 
 /*--------------------------------------------------------------------------*/
@@ -258,12 +351,12 @@ bool U61_Data::load1(char *name)
 	   * ClanLib datafile reader whill pathetically fail since
 	   * it opens "." as the main parent directory.
 	   */
-	  strcpy(data_dir,name);
+	  U61_MACRO_STRCPY(data_dir,name);
 	  U61_Platform::keep_dir_only(data_dir);
 	  /*
 	   * We store the name of the datafile, without the path
 	   */
-	  strcpy(data_file,name);
+	  U61_MACRO_STRCPY(data_file,name);
 	  U61_Platform::keep_file_only(data_file);
 
 	  chdir_to_data_dir();
@@ -374,6 +467,13 @@ bool U61_Data::load2()
 	      U61_LOG_DEBUG("Loading info graphics");
 	      success=success && load_info();
 	    }
+
+	  if (success)
+	    {
+	      U61_LOG_DEBUG("Loading mouse graphics");
+	      success=success && load_mouse();
+	    }
+
 	  if (!success)
 	    {
 	      U61_LOG_ERROR("Unknown error reading datafile, step 2");
@@ -638,7 +738,7 @@ bool U61_Data::load_musics()
 
   for (i=0;i<nb_music && success;++i)
     { 
-      sprintf(buffer,"%s%d",U61_DATA_ID_MUSIC_MOD,i);
+      U61_MACRO_SPRINTF2(buffer,"%s%d",U61_DATA_ID_MUSIC_MOD,i);
       music[i]=CL_Streamed_MikModSample::load(buffer,manager); 
       cl_assert(music[i]!=NULL);
     } 
@@ -659,53 +759,65 @@ bool U61_Data::load_squares()
 
   for (type=0; type<U61_DATA_NB_SQUARE_TYPE; ++type)
     {
-      strcpy(id,U61_DATA_ID_SQUARE_SMALL);
+      U61_MACRO_STRCPY(id,U61_DATA_ID_SQUARE_SMALL);
       id[strlen(id)-1]='0'+type;
       square[type][U61_DATA_SIZE_SMALL]=CL_Surface::load(id,manager);
       cl_assert(square[type][U61_DATA_SIZE_SMALL]!=NULL);
 
-      strcpy(id,U61_DATA_ID_SQUARE_BIG);
+      U61_MACRO_STRCPY(id,U61_DATA_ID_SQUARE_BIG);
       id[strlen(id)-1]='0'+type;
       square[type][U61_DATA_SIZE_BIG]=CL_Surface::load(id,manager);
       cl_assert(square[type][U61_DATA_SIZE_BIG]!=NULL);
     }
 
-  strcpy(id,U61_DATA_ID_SQUARE_SMALL);
+  U61_MACRO_STRCPY(id,U61_DATA_ID_SQUARE_SMALL);
   id[strlen(id)-1]=0;
-  strcat(id,U61_DATA_ID_SQUARE_CURSE);
+  U61_MACRO_STRCAT(id,U61_DATA_ID_SQUARE_CURSE);
   curse[U61_DATA_SIZE_SMALL]=CL_Surface::load(id,manager);
   cl_assert(curse[U61_DATA_SIZE_SMALL]!=NULL);
 
-  strcpy(id,U61_DATA_ID_SQUARE_BIG);
+  U61_MACRO_STRCPY(id,U61_DATA_ID_SQUARE_BIG);
   id[strlen(id)-1]=0;
-  strcat(id,U61_DATA_ID_SQUARE_CURSE);
+  U61_MACRO_STRCAT(id,U61_DATA_ID_SQUARE_CURSE);
   curse[U61_DATA_SIZE_BIG]=CL_Surface::load(id,manager);
   cl_assert(curse[U61_DATA_SIZE_BIG]!=NULL);
 
-  strcpy(id,U61_DATA_ID_SQUARE_SMALL);
+  U61_MACRO_STRCPY(id,U61_DATA_ID_SQUARE_SMALL);
   id[strlen(id)-1]=0;
-  strcat(id,U61_DATA_ID_SQUARE_ANTICIPATION);
+  U61_MACRO_STRCAT(id,U61_DATA_ID_SQUARE_ANTICIPATION);
   anticipation[U61_DATA_SIZE_SMALL]=CL_Surface::load(id,manager);
   cl_assert(anticipation[U61_DATA_SIZE_SMALL]!=NULL);
 
-  strcpy(id,U61_DATA_ID_SQUARE_BIG);
+  U61_MACRO_STRCPY(id,U61_DATA_ID_SQUARE_BIG);
   id[strlen(id)-1]=0;
-  strcat(id,U61_DATA_ID_SQUARE_ANTICIPATION);
+  U61_MACRO_STRCAT(id,U61_DATA_ID_SQUARE_ANTICIPATION);
   anticipation[U61_DATA_SIZE_BIG]=CL_Surface::load(id,manager);
   cl_assert(anticipation[U61_DATA_SIZE_BIG]!=NULL);
 
-  strcpy(id,U61_DATA_ID_SQUARE_SMALL);
+  U61_MACRO_STRCPY(id,U61_DATA_ID_SQUARE_SMALL);
   id[strlen(id)-1]=0;
-  strcat(id,U61_DATA_ID_SQUARE_EXPLOSION);
+  U61_MACRO_STRCAT(id,U61_DATA_ID_SQUARE_EXPLOSION);
   explosion[U61_DATA_SIZE_SMALL]=CL_Surface::load(id,manager);
   cl_assert(explosion[U61_DATA_SIZE_SMALL]!=NULL);
 
-  strcpy(id,U61_DATA_ID_SQUARE_BIG);
+  U61_MACRO_STRCPY(id,U61_DATA_ID_SQUARE_BIG);
   id[strlen(id)-1]=0;
-  strcat(id,U61_DATA_ID_SQUARE_EXPLOSION);
+  U61_MACRO_STRCAT(id,U61_DATA_ID_SQUARE_EXPLOSION);
   explosion[U61_DATA_SIZE_BIG]=CL_Surface::load(id,manager);
   cl_assert(explosion[U61_DATA_SIZE_BIG]!=NULL);
 
+  U61_MACRO_STRCPY(id,U61_DATA_ID_SQUARE_SMALL);
+  id[strlen(id)-1]=0;
+  U61_MACRO_STRCAT(id,U61_DATA_ID_SQUARE_FILLER);
+  filler[U61_DATA_SIZE_SMALL]=CL_Surface::load(id,manager);
+  cl_assert(filler[U61_DATA_SIZE_SMALL]!=NULL);
+
+  U61_MACRO_STRCPY(id,U61_DATA_ID_SQUARE_BIG);
+  id[strlen(id)-1]=0;
+  U61_MACRO_STRCAT(id,U61_DATA_ID_SQUARE_FILLER);
+  filler[U61_DATA_SIZE_BIG]=CL_Surface::load(id,manager);
+  cl_assert(filler[U61_DATA_SIZE_BIG]!=NULL);
+
   if (success)
     {
       square_w[U61_DATA_SIZE_SMALL]=square[0][U61_DATA_SIZE_SMALL]->get_width();
@@ -735,12 +847,12 @@ bool U61_Data::load_maps()
   
   for (i=0; i<nb_map; ++i)
     {
-      sprintf(id,"%s%d",U61_DATA_ID_MAP_SMALL,i);
+      U61_MACRO_SPRINTF2(id,"%s%d",U61_DATA_ID_MAP_SMALL,i);
       map[i][U61_DATA_SIZE_SMALL]=CL_Surface::load(id,manager);
       cl_assert(map[i][U61_DATA_SIZE_SMALL]!=NULL);
       map[i][U61_DATA_SIZE_SMALL]->convert_system();
 
-      sprintf(id,"%s%d",U61_DATA_ID_MAP_BIG,i);
+      U61_MACRO_SPRINTF2(id,"%s%d",U61_DATA_ID_MAP_BIG,i);
       map[i][U61_DATA_SIZE_BIG]=CL_Surface::load(id,manager);
       cl_assert(map[i][U61_DATA_SIZE_BIG]!=NULL);
       map[i][U61_DATA_SIZE_BIG]->convert_system();
@@ -803,3 +915,32 @@ bool U61_Data::load_info()
   return success;
 }
 
+/*--------------------------------------------------------------------------*/
+/*
+ * loads the mouse sprites
+ */
+bool U61_Data::load_mouse()
+{
+  bool success=true;
+  CL_SurfaceProvider *surface_provider;
+  static std::vector<int> frame_delays;
+  int i;
+
+  frame_delays.clear();
+  for (i=0;i<U61_DATA_MOUSE_CURSOR_NB_FRAMES;++i)
+    {
+      frame_delays.push_back(U61_DATA_MOUSE_CURSOR_FRAME_DELAY);
+    }
+
+  surface_provider=CL_SurfaceProvider::load(U61_DATA_ID_MOUSE_CURSOR,manager);
+  cl_assert(surface_provider!=NULL);
+  
+  mouse_cursor=CL_MouseCursorProvider::create(surface_provider,
+					      true,
+					      true,
+					      &frame_delays);
+  cl_assert(mouse_cursor!=NULL);
+
+  return success;
+}
+
