@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -36,8 +36,8 @@
  *              classes to access the game resources
  */
 
-#ifndef INC_U61_DATA
-#define INC_U61_DATA
+#ifndef U61_INC_DATA
+#define U61_INC_DATA
 
 /*---------------------------------------------------------------------------
  includes
@@ -64,6 +64,11 @@
 #define U61_DATA_SYMBOL_CURSE_GOOD "+"
 #define U61_DATA_SYMBOL_ANTIDOTE "a"
 
+#define U61_DATA_PLAYER_NAME_SIZE     6
+#define U61_DATA_SERVER_ADDRESS_SIZE 80
+#define U61_DATA_SCRIPT_NAME_SIZE   128
+#define U61_DATA_PASSWORD_SIZE       16
+
 /*---------------------------------------------------------------------------
  classes
  ---------------------------------------------------------------------------*/
@@ -136,6 +141,7 @@ class U61_Data
   CL_Surface *curse[U61_DATA_NB_SIZE];
   CL_Surface *anticipation[U61_DATA_NB_SIZE];
   CL_Surface *explosion[U61_DATA_NB_SIZE];
+  CL_Surface *filler[U61_DATA_NB_SIZE];
 
   CL_Surface *map[U61_DATA_NB_MAP_MAX][U61_DATA_NB_SIZE];
 
@@ -144,6 +150,8 @@ class U61_Data
 
   CL_Surface *info[U61_DATA_NB_SIZE];
 
+  CL_MouseCursorProvider *mouse_cursor;
+
  protected:
   CL_ResourceManager *manager;
   bool packed_data;
@@ -161,6 +169,7 @@ class U61_Data
   bool load_maps();
   bool load_back();
   bool load_info();
+  bool load_mouse();
 };
 
 #endif
