@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -35,40 +35,43 @@
 
 
 
-#ifndef INC_U61_EDIT
-#define INC_U61_EDIT
+#ifndef U61_INC_EDIT
+#define U61_INC_EDIT
 
 /*---------------------------------------------------------------------------
- includes
- ---------------------------------------------------------------------------*/
+  includes
+  ---------------------------------------------------------------------------*/
 
 #include <ClanLib/core.h>
 #include "menuitem.h"
 
 /*---------------------------------------------------------------------------
- constants
- ---------------------------------------------------------------------------*/
+  constants
+  ---------------------------------------------------------------------------*/
 
-#define U61_EDIT_MAX_SIZE 50
+#define U61_EDIT_MAX_SIZE 150
 
 /*---------------------------------------------------------------------------
- classes
- ---------------------------------------------------------------------------*/
+  classes
+  ---------------------------------------------------------------------------*/
 
 class U61_Edit: public U61_MenuItem
 {
  public:
-  U61_Edit(char *label, char *val);
+  U61_Edit(char *label, char *val, int size);
     
   virtual bool action(int key);
   virtual void update();
-  virtual void select();
+  virtual void select(int pos);
+  virtual bool check_value();
 
  protected:
+  int size_max;
   char *name;
   char *value;
   char buffer[U61_EDIT_MAX_SIZE+1];
   int pos;
+  static bool insert_mode;
 
   virtual int translate_to_ascii(int key);
 
