@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -34,20 +34,21 @@
  */
 
 /*---------------------------------------------------------------------------
- includes
- ---------------------------------------------------------------------------*/
+  includes
+  ---------------------------------------------------------------------------*/
 
 #include "emptymenu.h"
-#include "button.h"
+#include "mouse.h"
+#include "log.h"
 
 /*---------------------------------------------------------------------------
- globals
- ---------------------------------------------------------------------------*/
+  globals
+  ---------------------------------------------------------------------------*/
 
 
 /*---------------------------------------------------------------------------
- functions
- ---------------------------------------------------------------------------*/
+  functions
+  ---------------------------------------------------------------------------*/
 
 /*--------------------------------------------------------------------------*/
 /* 
@@ -55,21 +56,21 @@
  */ 
 U61_EmptyMenu::U61_EmptyMenu(U61_Menu *parent) : U61_Menu(parent, " ")
 {
-    message_active=false;
+  message_active=false;
 
-    add_item(new U61_Button(" "));
-    select(0);
+  select(0);
+
+  U61_Mouse::hide();
 }
 
 /*--------------------------------------------------------------------------*/
-/*
- * validate is called when the user presses enter
- */
-U61_Menu *U61_EmptyMenu::validate()
+/* 
+ * destruction of the empty menu
+ */ 
+U61_EmptyMenu::~U61_EmptyMenu()
 {
-    U61_Menu *next_menu;
+  U61_LOG_DEBUG("U61_EmptyMenu destructor called");
 
-    next_menu=this;
-
-    return next_menu;
+  U61_Mouse::show();
 }
+
