@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -44,6 +44,7 @@
  ---------------------------------------------------------------------------*/
 
 #include "event.h"
+#include "log.h"
 
 /*---------------------------------------------------------------------------
  globals
@@ -80,33 +81,61 @@ U61_Event::~U61_Event()
 /*
  * Puts the event into a portable char buffer (for network transmission)
  */
-void U61_Event::serialize(unsigned char *buffer)
+bool U61_Event::serialize(unsigned char **buffer, unsigned char *buffer_limit)
 {
-  buffer[0]=code;
-  buffer[1]=(author<<4) | target;
-  buffer[2]=0xFF & par;
-  buffer[3]=par>>8;
-  buffer[4]=0xFF & time;
-  buffer[5]=0xFF & time>>8;
-  buffer[6]=0xFF & time>>16;
-  buffer[7]=time>>24;
+  bool result=false;
+
+  if (buffer_limit-(*buffer)>=U61_EVENT_SERIALIZED_SIZE)
+    {
+      (*buffer)[0]=code;
+      (*buffer)[1]=(author<<4) | target;
+      (*buffer)[2]=0xFF & par;
+      (*buffer)[3]=par>>8;
+      (*buffer)[4]=0xFF & time;
+      (*buffer)[5]=0xFF & time>>8;
+      (*buffer)[6]=0xFF & time>>16;
+      (*buffer)[7]=time>>24;
+
+      (*buffer)+=U61_EVENT_SERIALIZED_SIZE;
+      result=true;
+    }
+  else
+    {
+      U61_LOG_DEBUG("Buffer too short to serialize event!");
+    }
+
+  return result;
 }
 
 /*--------------------------------------------------------------------------*/
 /*
  * Gets the event from a portable char buffer (for network transmission)
  */
-void U61_Event::unserialize(unsigned char *buffer)
+bool U61_Event::unserialize(unsigned char **buffer, unsigned char *buffer_limit)
 {
-  code=buffer[0];
-  target=buffer[1] & 0x0F;
-  author=buffer[1]>>4;
-  par=(((unsigned int) buffer[3])<<8)
-    | ((unsigned int) buffer[2]);
-  time=(((unsigned int) buffer[7])<<24) 
-    | (((unsigned int) buffer[6])<<16) 
-    | (((unsigned int) buffer[5])<<8) 
-    | ((unsigned int) buffer[4]);
+  bool result=false;
+
+  if (buffer_limit-(*buffer)>=U61_EVENT_SERIALIZED_SIZE)
+    {
+      code=(*buffer)[0];
+      target=(*buffer)[1] & 0x0F;
+      author=(*buffer)[1]>>4;
+      par=(((unsigned int) (*buffer)[3])<<8)
+	| ((unsigned int) (*buffer)[2]);
+      time=(((unsigned int) (*buffer)[7])<<24) 
+	| (((unsigned int) (*buffer)[6])<<16) 
+	| (((unsigned int) (*buffer)[5])<<8) 
+	| ((unsigned int) (*buffer)[4]);
+
+      (*buffer)+=U61_EVENT_SERIALIZED_SIZE;
+      result=true;
+    }
+  else
+    {
+      U61_LOG_DEBUG("Buffer too short to unserialize event!");
+    }
+
+  return result;
 }
 
 /*------------------------------------------------------------------------- */
@@ -115,10 +144,107 @@ void U61_Event::unserialize(unsigned char *buffer)
  */
 ostream &operator <<(ostream &o, U61_Event event)
 {
+  char *event_label="";
+
+  switch (event.code)
+    {
+    case U61_EVENT_NOTHING :
+      event_label="NOTHING";
+      break;
+    case U61_EVENT_READY :
+      event_label="READY";
+      break;
+    case U61_EVENT_READY_ALL :
+      event_label="READY_ALL";
+      break;
+    case U61_EVENT_START_STOP :
+      event_label="START_STOP";
+      break;
+    case U61_EVENT_KILL :
+      event_label="KILL";
+      break;
+    case U61_EVENT_NAME_LETTER :
+      event_label="NAME_LETTER";
+      break;
+    case U61_EVENT_CHAT_LETTER :
+      event_label="CHAT_LETTER";
+      break;
+    case U61_EVENT_CHANGE_BACKGROUND:
+      event_label="CHANGE_BACKGROUND";
+      break; 
+    case U61_EVENT_MOVE_RIGHT :
+      event_label="MOVE_RIGHT";
+      break;
+    case U61_EVENT_ROTATE_LEFT :
+      event_label="ROTATE_LEFT";
+      break;
+    case U61_EVENT_ROTATE_RIGHT :
+      event_label="ROTATE_RIGHT";
+      break;
+    case U61_EVENT_MOVE_DOWN :
+      event_label="MOVE_DOWN";
+      break;
+    case U61_EVENT_DROP :
+      event_label="DROP";
+      break;
+    case U61_EVENT_SET_VICTIM :
+      event_label="SET_VICTIM";
+      break;
+    case U61_EVENT_NEXT_VICTIM :
+      event_label="NEXT_VICTIM";
+      break;
+    case U61_EVENT_PREV_VICTIM :
+      event_label="PREV_VICTIM";
+      break;
+    case U61_EVENT_USE_ANTIDOTE :
+      event_label="USE_ANTIDOTE";
+      break;
+    case U61_EVENT_NEW_BLOCK :
+      event_label="NEW_BLOCK";
+      break;
+    case U61_EVENT_LOOSE :
+      event_label="LOOSE";
+      break;
+    case U61_EVENT_NEW_CURSE :
+      event_label="NEW_CURSE";
+      break;
+    case U61_EVENT_REQUEST_CURSE :
+      event_label="REQUEST_CURSE";
+      break;
+    case U61_EVENT_DO_CURSE :
+      event_label="DO_CURSE";
+      break;
+    case U61_EVENT_HISTORY_BEGIN :
+      event_label="HISTORY_BEGIN";
+      break;
+    case U61_EVENT_HISTORY_END :
+      event_label="HISTORY_END";
+      break;
+    case U61_EVENT_SERIAL_BEGIN :
+      event_label="SERIAL_BEGIN";
+      break;
+    case U61_EVENT_SERIAL_DATA :
+      event_label="SERIAL_DATA";
+      break;
+    case U61_EVENT_SERIAL_END :
+      event_label="SERIAL_END";
+      break;
+    case U61_EVENT_REQUEST_CHECKSUM :
+      event_label="REQUEST_CHECKSUM";
+      break;
+    case U61_EVENT_VERIFY_CHECKSUM :
+      event_label="VERIFY_CHECKSUM";
+      break;
+    default :
+      event_label="UNKNOWN";
+      break;
+    }
+
   return o<<"code="<<event.code
-	  <<"/author="<<event.author
-	  <<"/target="<<event.target
-	  <<"/par="<<event.par
-	  <<"/time="<<event.time;
+	  <<" ("<<event_label<<")"
+	  <<" / author="<<event.author
+	  <<" / target="<<event.target
+	  <<" / par="<<event.par
+	  <<" / time="<<event.time;
 }
 
