@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -39,8 +39,8 @@
  *              almost all events are dated.
  */
 
-#ifndef INC_U61_EVENT
-#define INC_U61_EVENT
+#ifndef U61_INC_EVENT
+#define U61_INC_EVENT
 
 /*---------------------------------------------------------------------------
  includes
@@ -49,33 +49,56 @@
 #include <ClanLib/core.h>
 //#include <iostream.h>
 
+/*---------------------------------------------------------------------------
+ constants
+ ---------------------------------------------------------------------------*/
+
+/*
+ * The list of event codes
+ */
 #define U61_EVENT_NOTHING            0
 
-#define U61_EVENT_START_STOP         1
-#define U61_EVENT_KILL               2
+#define U61_EVENT_READY              1
+#define U61_EVENT_READY_ALL          2
 
-#define U61_EVENT_NAME_LETTER        3
-#define U61_EVENT_CHANGE_BACKGROUND  4
+#define U61_EVENT_START_STOP         3
+#define U61_EVENT_KILL               4
 
-#define U61_EVENT_MOVE_LEFT          5
-#define U61_EVENT_MOVE_RIGHT         6
-#define U61_EVENT_ROTATE_LEFT        7
-#define U61_EVENT_ROTATE_RIGHT       8
-#define U61_EVENT_MOVE_DOWN          9
-#define U61_EVENT_DROP              10
-#define U61_EVENT_SET_VICTIM        11
-#define U61_EVENT_NEXT_VICTIM       12
-#define U61_EVENT_PREV_VICTIM       13
-#define U61_EVENT_USE_ANTIDOTE      14
+#define U61_EVENT_NAME_LETTER        5
+#define U61_EVENT_CHAT_LETTER        6
+#define U61_EVENT_CHANGE_BACKGROUND  7
+
+#define U61_EVENT_MOVE_LEFT          8
+#define U61_EVENT_MOVE_RIGHT         9
+#define U61_EVENT_ROTATE_LEFT       10
+#define U61_EVENT_ROTATE_RIGHT      11
+#define U61_EVENT_MOVE_DOWN         12
+#define U61_EVENT_DROP              13
+#define U61_EVENT_SET_VICTIM        14
+#define U61_EVENT_NEXT_VICTIM       15
+#define U61_EVENT_PREV_VICTIM       16
+#define U61_EVENT_USE_ANTIDOTE      17
  
-#define U61_EVENT_NEW_BLOCK         15
-#define U61_EVENT_LOOSE             16
-#define U61_EVENT_NEW_CURSE         17
-#define U61_EVENT_REQUEST_CURSE     18
-#define U61_EVENT_DO_CURSE          19
+#define U61_EVENT_NEW_BLOCK         18
+#define U61_EVENT_LOOSE             19
+#define U61_EVENT_NEW_CURSE         20
+#define U61_EVENT_REQUEST_CURSE     21
+#define U61_EVENT_DO_CURSE          22
 
-#define U61_EVENT_HISTORY_BEGIN     20
-#define U61_EVENT_HISTORY_END       21
+#define U61_EVENT_HISTORY_BEGIN     23
+#define U61_EVENT_HISTORY_END       24
+
+#define U61_EVENT_SERIAL_BEGIN      25
+#define U61_EVENT_SERIAL_DATA       26
+#define U61_EVENT_SERIAL_END        27
+
+#define U61_EVENT_REQUEST_CHECKSUM  28
+#define U61_EVENT_VERIFY_CHECKSUM   29
+
+/*
+ * The size (in bytes) of a serialized event
+ */
+#define U61_EVENT_SERIALIZED_SIZE    8
 
 /*---------------------------------------------------------------------------
  classes
@@ -93,8 +116,8 @@ class U61_Event
   unsigned int par : 16;
   unsigned int time : 32;
 
-  void serialize(unsigned char *buffer);
-  void unserialize(unsigned char *buffer);
+  bool serialize(unsigned char **buffer, unsigned char *buffer_limit);
+  bool unserialize(unsigned char **buffer, unsigned char *buffer_limit);
   
  protected:  
 };
