@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -41,7 +41,7 @@
  ---------------------------------------------------------------------------*/
 
 #include "eventcopy.h"
-#include "debug.h"
+#include "log.h"
 
 /*---------------------------------------------------------------------------
  globals
@@ -110,23 +110,30 @@ void U61_EventCopy::put(U61_Event event)
   /*
    * If an event does not come from the right player, we do not let
    * it pass...
-   * We also block NOTHING events, for they would waste bandwidth
+   * We also block 90% of NOTHING events, for they would waste bandwidth
    */
   if (event.author==(unsigned int) player)
     {
-      if (event.code==U61_EVENT_NOTHING)
-	{  
+      switch (event.code)
+	{
+	case U61_EVENT_NOTHING:
 	  if (dummy_counter>=U61_EVENTCOPY_DUMMY_PASS)
 	    { 
 	      queue.push_back(event);
 	      dummy_counter=0;
 	    }
-	  dummy_counter++;
-	}
-      else    
-	{
+	  dummy_counter++;	  
+	  break;
+	case U61_EVENT_READY_ALL:
+	  /*
+	   * We do not copy this event otherwise it replicates
+	   * forever on the network...
+	   */
+	  break;
+	default:
 	  U61_LOG_DEBUG("Copying event "<<event);
 	  queue.push_back(event);
+	  break;
 	}
     }
 }
