@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -40,8 +40,8 @@
 
 
 
-#ifndef INC_U61_FILE
-#define INC_U61_FILE
+#ifndef U61_INC_FILE
+#define U61_INC_FILE
 
 /*---------------------------------------------------------------------------
  includes
@@ -58,9 +58,9 @@
 class U61_File: public U61_MenuItem
 {
  public:
-  U61_File(char *label, char *name, char *buf);
+  U61_File(char *label, char *name, char buf[U61_CONST_STRING_SIZE]);
     
-  virtual void select();
+  virtual void select(int pos);
  protected:
   char *buffer;
   char filename[U61_CONST_STRING_SIZE];
