@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -48,8 +48,9 @@
 #include "button.h"
 #include "platform.h"
 #include "const.h"
-#include "debug.h"
+#include "log.h"
 #include "global.h"
+#include "macro.h"
 
 /*---------------------------------------------------------------------------
  globals
@@ -141,15 +142,15 @@ bool U61_FileMenu::add_files(char *dir,char *ext)
   
   U61_Global::data.chdir_to_initial_dir();
 
-  strcpy(last_chosen_file,chosen_file);
+  U61_MACRO_STRCPY(last_chosen_file,chosen_file);
   U61_LOG_DEBUG("Searching "<<dir<<" for \""<<ext<<"\" files.");
   if (U61_Platform::open_dir(dir,ext))
     {
       while ((name=U61_Platform::next_file())!=NULL)
         {
-          strcpy(buffer_name,dir);
-	  strcat(buffer_name,name);
-          strcpy(buffer_label,name);
+          U61_MACRO_STRCPY(buffer_name,dir);
+	  U61_MACRO_STRCAT(buffer_name,name);
+          U61_MACRO_STRCPY(buffer_label,name);
           buffer_label[strlen(buffer_label)-strlen(ext)]=0;
 	      
 	  add_item(new U61_File(buffer_label,buffer_name,chosen_file));
@@ -164,7 +165,7 @@ bool U61_FileMenu::add_files(char *dir,char *ext)
       U61_Platform::close_dir();
     }
   
-  strcpy(chosen_file,last_chosen_file);
+  U61_MACRO_STRCPY(chosen_file,last_chosen_file);
   
   U61_Global::data.chdir_to_data_dir();
 
