@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -47,6 +47,7 @@
 #include "game.h"
 #include "time.h"
 #include "mainmenu.h"
+#include "errormenu.h"
 #include "sound.h"
 #include "music.h"
 #include "square.h"
@@ -61,19 +62,21 @@
 #include "localdispatcher.h"
 #include "utils.h"
 #include "game.h"
-#include "debug.h"
+#include "log.h"
+#include "macro.h"
+#include "mouse.h"
+#include "system.h"
 
 /*---------------------------------------------------------------------------
  constants
  ---------------------------------------------------------------------------*/
 
-#define U61_GAME_MAX_LOGIC_PER_DISPLAY 1000
+#define U61_GAME_MAX_LOGIC_PER_DISPLAY  1000
 #define U61_GAME_LIMIT_LOGIC_PER_DISPLAY 100
-#define U61_GAME_MENU_POLL_DELAY 33
-
-#define U61_GAME_STATE_RUN               0
-#define U61_GAME_STATE_QUIT              1
+#define U61_GAME_MENU_POLL_DELAY          33
 
+#define U61_GAME_STATE_RUN                 0
+#define U61_GAME_STATE_QUIT                1
 
 /*---------------------------------------------------------------------------
  functions
@@ -123,8 +126,11 @@ bool U61_Game::init(char *filename)
 
   protocol=NULL;
   dispatcher=NULL;
+  www_register=NULL;
+  playing_session=false;
+  server_mode=false;
 
-  strcpy(exe_name,filename);
+  U61_MACRO_STRCPY(exe_name,filename);
 
   U61_Global::config.read();
   
@@ -151,10 +157,15 @@ bool U61_Game::init(char *filename)
       
   if (success)
     {
+      U61_Global::mouse_monitor.init();
+      U61_Mouse::show();
+
       U61_Global::input_monitor.init();
       U61_Global::input_monitor.read_config(&U61_Global::config);
     }
 
+  slot_quit = CL_System::sig_quit().connect(this, &U61_Game::on_quit);
+
   return success;
 }
 
@@ -330,7 +341,7 @@ void U61_Game::poll()
   int clanlib_time;
   int u61_time;
 
-  CL_System::keep_alive();
+  U61_System::keep_alive();
   clanlib_time=CL_System::get_time();
   if (last_clanlib_time_polled<clanlib_time)
     {
@@ -338,7 +349,7 @@ void U61_Game::poll()
        * required by ClanLib, polls the keyboard and does 
        * some system low level stuff
        */
-      CL_System::keep_alive();
+      U61_System::keep_alive();
       last_clanlib_time_polled=clanlib_time;
       calls_to_poll_clanlib++;
 
@@ -367,9 +378,14 @@ void U61_Game::poll()
 	  last_menu_time_polled=clanlib_time;   
 
 	  menu_input.poll(u61_time);
-          menu_input.poll_mouse();
 	  current_menu->poll();
 
+	  /*
+	   * we poll the "chat" object together with the menu object,
+	   * since it does not need to be polled "very often".
+	   */
+	  chat_input.poll(u61_time);
+
 	  calls_to_poll_menu++;     
         }
     }
@@ -445,21 +461,22 @@ void U61_Game::menu()
 {
   U61_Menu *next_menu;
   U61_Event key;
-  int mouse_y;
 
-  next_menu=current_menu;
+  next_menu=current_menu->poll();
 
-  if (!menu_input.empty())
+  if (next_menu==current_menu)
     {
-      key=menu_input.get();
-      next_menu=current_menu->action(key.code);
+      if (!menu_input.empty())
+	{
+	  key=menu_input.get();
+	  next_menu=current_menu->action(key.code);
+	}      
     }
 
-  mouse_y=menu_input.get_mouse_y();
-  if (mouse_y>=0)
+  if (U61_Global::mouse_monitor.moved())
     {
-      U61_LOG_DEBUG("mouse_y="<<mouse_y);
-      next_menu=current_menu->mouse_action(mouse_y);
+      next_menu->mouse_move(U61_Global::mouse_monitor.get_x(),
+			    U61_Global::mouse_monitor.get_y());
     }
 
   if (next_menu==NULL)
@@ -496,18 +513,34 @@ void U61_Game::network()
       if (protocol->send_all()==U61_PROTOCOL_FAILED 
 	  || protocol->recv_all()==U61_PROTOCOL_FAILED)
 	{
-	  protocol->print_diagnostic();
-	  if (protocol->is_connecting())
-	    {
-	      stop();
+	  U61_Menu *temp_menu;
 
-	      /*
-	       * We go back of 2 levels in the menu history to get back
-	       * to the "join menu"
-	       */
-	      parent_menu();
-	      parent_menu();
-	    }
+	  /*
+	   * Back to the "In game option menu"
+	   */
+	  parent_menu();
+	  /*
+	   * Back to where the player should be (score menu in
+	   * most cases, but the score menu will disappear
+	   * automatically if there are no interesting scores
+	   * to display).
+	   */
+	  parent_menu();
+
+	  /*
+	   * Important to create the menu now for the
+	   * protocol object is still here, but will be
+	   * deleted after a call to stop()
+	   */
+	  temp_menu=new U61_ErrorMenu
+	    (current_menu,protocol->get_diagnostic());
+
+	  stop();
+
+	  /*
+	   * Display the error on the screen
+	   */
+	  current_menu=temp_menu;
 	}
     }
 
@@ -522,7 +555,7 @@ void U61_Game::network()
               slot=find_free_network_slot();
               if (slot>=0)
 		{
-                  player[slot].init_network(dispatcher,i);
+                  player[slot].init_network(dispatcher,i,server_mode);
 		  new_player_created=true;
 		}
 	    }
@@ -537,6 +570,12 @@ void U61_Game::network()
 	  U61_Sound::play_player_start();
 	}
     }
+
+  if (www_register!=NULL)
+    {
+      www_register->set_busy_players(nb_active_players());
+      www_register->execute(false);
+    }
 }
 
 /*--------------------------------------------------------------------------*/
@@ -632,19 +671,20 @@ void U61_Game::start_local_alone()
       U61_Script::check_id();
 
       slow_down_allowed=true;
+      server_mode=false;
 
       dispatcher=new U61_LocalDispatcher();
 
       num=get_local_player(0);
       U61_LOG_DEBUG("num="<<num);
       U61_LOG_DEBUG("name="<<U61_Global::config.player[num].name);
-      player[0].init_local(&(player_input[num]),dispatcher,
-			   0,0,
-			   &(U61_Global::config.player[num]));
+      player[0].init_local(&(player_input[num]),NULL,
+			   dispatcher,
+			   0,server_mode,U61_TIME_START,
+			   &(U61_Global::config.player[num]),
+			   true);
       U61_Global::layout.init_alone(&(player[0]));
     }
-
-  U61_Music::start();
 }
 
 /*--------------------------------------------------------------------------*/
@@ -660,13 +700,16 @@ void U61_Game::start_local_multiplayer()
       U61_Script::check_id();
 
       slow_down_allowed=true;
+      server_mode=false;
 
       dispatcher=new U61_LocalDispatcher();
 
       num0=get_local_player(0);
-      player[0].init_local(&(player_input[num0]),dispatcher,
-			   0,0,
-			   &(U61_Global::config.player[num0]));
+      player[0].init_local(&(player_input[num0]),NULL,
+			   dispatcher,
+			   0,server_mode,U61_TIME_START,
+			   &(U61_Global::config.player[num0]),
+			   true);
 
       num1=get_local_player(1);
       if (num1<0)
@@ -691,18 +734,22 @@ void U61_Game::start_local_multiplayer()
 	      num1=0;
 	    }
 
-	  player[1].init_local(&(player_input[num1]),dispatcher,
-			       1,0,
-			       &(U61_Global::config.player[num1]));
+	  player[1].init_local(&(player_input[num1]),NULL,
+			       dispatcher,
+			       1,server_mode,U61_TIME_START,
+			       &(U61_Global::config.player[num1]),
+			       true);
 
 	  U61_Global::layout.init_multiplayer_2(&(player[0]),
 						&(player[1]));
         }
       else
         {
-	  player[1].init_local(&(player_input[num1]),dispatcher,
-			       1,0,
-			       &(U61_Global::config.player[num1]));
+	  player[1].init_local(&(player_input[num1]),NULL,
+			       dispatcher,
+			       1,server_mode,U61_TIME_START,
+			       &(U61_Global::config.player[num1]),
+			       true);
 
 	  num2=get_local_player(2);
 	  if (num2<0)
@@ -712,9 +759,11 @@ void U61_Game::start_local_multiplayer()
             }
 	  else
             {
-	      player[2].init_local(&(player_input[num2]),dispatcher,
-				   2,0,
-				   &(U61_Global::config.player[num2]));
+	      player[2].init_local(&(player_input[num2]),NULL,
+				   dispatcher,
+				   2,server_mode,U61_TIME_START,
+				   &(U61_Global::config.player[num2]),
+				   true);
                 
 	      num3=get_local_player(3);
 	      if (num3<0)
@@ -725,9 +774,11 @@ void U61_Game::start_local_multiplayer()
                 }
 	      else
                 {
-		  player[3].init_local(&(player_input[num3]),dispatcher,
-				       3,0,
-				       &(U61_Global::config.player[num3]));
+		  player[3].init_local(&(player_input[num3]),NULL,
+				       dispatcher,
+				       3,server_mode,U61_TIME_START,
+				       &(U61_Global::config.player[num3]),
+				       true);
                     
 		  U61_Global::layout.init_multiplayer_4(&(player[0]),
 							&(player[1]),
@@ -737,8 +788,6 @@ void U61_Game::start_local_multiplayer()
             }
         }
     }
-
-  U61_Music::start();
 }
 
 /*--------------------------------------------------------------------------*/
@@ -749,25 +798,29 @@ void U61_Game::start_network_new()
 {
   int num0,num1;
   U61_ServerDispatcher *d;
+  int nbp=0;
 
   U61_LOG_DEBUG("new network game started");
   if (standard_init() && U61_Global::config.load_script())
     {
       U61_Script::check_id();
+      chat_input.set_active();      
 
       slow_down_allowed=false;
+      server_mode=true;
 
       d=new U61_ServerDispatcher();
       dispatcher=d;     
-      
+
       protocol=new U61_ServerProtocol(d,
-				      U61_Global::config.network_port,
-                                      U61_Global::config.network_channel);
+				      U61_Global::config.network_port);
       
       num0=get_local_player(0);
-      player[0].init_local(&(player_input[num0]),dispatcher,
-			   0,0,
-			   &(U61_Global::config.player[num0]));
+      player[0].init_local(&(player_input[num0]),&chat_input,
+			   dispatcher,
+			   0,server_mode,U61_TIME_START,
+			   &(U61_Global::config.player[num0]),
+			   false);
 
       num1=get_local_player(1);
       if (num1<0)
@@ -775,22 +828,30 @@ void U61_Game::start_network_new()
 	  U61_Global::layout.init_network_1
 	    (&(player[0]),
 	     &(player[U61_LAYOUT_NB_PLAYERS_NET_LOCAL]));
+	  nbp=1;
         }
       else
         {
-	  player[1].init_local(&(player_input[num1]),dispatcher,
-			       1,0,
-			       &(U61_Global::config.player[num1]));
+	  player[1].init_local(&(player_input[num1]),NULL,
+			       dispatcher,
+			       1,server_mode,U61_TIME_START,
+			       &(U61_Global::config.player[num1]),
+			       false);
 
 	 
 	  U61_Global::layout.init_network_2
 	    (&(player[0]),
 	     &(player[1]),
 	     &(player[U61_LAYOUT_NB_PLAYERS_NET_LOCAL]));
+	  nbp=2;
 	}
-    }
 
-  U61_Music::start();
+      www_register=new U61_WwwRegister();
+      www_register->set_port(U61_Global::config.network_port);
+      www_register->set_busy_players(nbp);
+      www_register->set_comment(U61_Global::config.server_comment);
+      www_register->execute(true);
+    }
 }
 
 /*--------------------------------------------------------------------------*/
@@ -806,42 +867,14 @@ void U61_Game::start_network_join()
   if (standard_init())
     {
       slow_down_allowed=false;
+      server_mode=false;
 
       d=new U61_ClientDispatcher();
       dispatcher=d;     
 
       protocol=new U61_ClientProtocol(d,
 				      U61_Global::config.network_port,
-                                      U61_Global::config.network_channel,
-                                      U61_Global::config.server_id,
-				      false);
-    }
-}
-
-/*--------------------------------------------------------------------------*/
-/*
- * joins an existing network game by broadcasting a LAN
- */
-void U61_Game::start_network_broadcast()
-{
-  U61_ClientDispatcher *d;
-
-  U61_LOG_DEBUG("broadcasting network game");
-  
-  if (standard_init() && U61_Global::config.load_script())
-    {
-      U61_Script::check_id();
-
-      slow_down_allowed=false;
-
-      d=new U61_ClientDispatcher();
-      dispatcher=d;     
-
-      protocol=new U61_ClientProtocol(d,
-				      U61_Global::config.network_port,
-                                      U61_Global::config.network_channel,
-                                      U61_Global::config.server_id,
-				      true);
+                                      U61_Global::config.server_id);
     }
 }
 
@@ -861,15 +894,18 @@ void U61_Game::do_network_join(char *script,
 
   U61_Script::do_string(script);
   U61_Script::check_id();
+  chat_input.set_active();      
 
   initial_speed=speed;
   acceleration=accel;
   curse_delay=delay;
 
   num0=get_local_player(0);
-  player[0].init_local(&(player_input[num0]),dispatcher,
-		       id0,time,
-		       &(U61_Global::config.player[num0]));
+  player[0].init_local(&(player_input[num0]),&chat_input,
+		       dispatcher,
+		       id0,server_mode,time,
+		       &(U61_Global::config.player[num0]),
+		       false);
 
   num1=get_local_player(1);
   if (num1<0)
@@ -880,9 +916,11 @@ void U61_Game::do_network_join(char *script,
     }
   else
     {
-      player[1].init_local(&(player_input[num1]),dispatcher,
-			   id1,time,
-			   &(U61_Global::config.player[num1]));
+      player[1].init_local(&(player_input[num1]),NULL,
+			   dispatcher,
+			   id1,server_mode,time,
+			   &(U61_Global::config.player[num1]),
+			   false);
 
 	 
       U61_Global::layout.init_network_2
@@ -890,8 +928,6 @@ void U61_Game::do_network_join(char *script,
 	 &(player[1]),
 	 &(player[U61_LAYOUT_NB_PLAYERS_NET_LOCAL]));
     }
-
-  U61_Music::start();
 }
 
 /*--------------------------------------------------------------------------*/
@@ -914,6 +950,13 @@ void U61_Game::stop_network()
       U61_LOG_DEBUG("dispatcher deleted");
       dispatcher=NULL;
     }
+  if (www_register!=NULL)
+    {
+      U61_LOG_DEBUG("deleting www_register");
+      delete www_register;
+      U61_LOG_DEBUG("www_register deleted");
+      www_register=NULL;
+    }
 }
 
 /*--------------------------------------------------------------------------*/
@@ -932,10 +975,12 @@ void U61_Game::stop()
   U61_Global::layout.reset();
   stop_network();
   U61_Time::resume();
-  U61_Music::stop();
 
   U61_Platform::close_score_file();
 
+  notify_session_stop();
+  chat_input.reset();
+
   U61_LOG_DEBUG("game stopped");
 }
 
@@ -960,6 +1005,7 @@ bool U61_Game::standard_init()
   U61_Global::data.chdir_to_data_dir();
   
   U61_Global::score.reset();
+  chat_input.reset();
 
   return ok;
 }
@@ -1086,26 +1132,19 @@ int U61_Game::find_free_player_id1()
  */
 int U61_Game::find_free_player_id(int start,int step)
 {
-  int i,j;
+  int i;
   int result=-1;
-  bool taken;
 
-  for (i=start;i<=U61_DISPATCHER_MAX_PLAYER_ID-step;i+=step)
+  for (i=start;i<U61_DISPATCHER_MAX_PLAYER_ID;i+=step)
     {
-      taken=false;
-      for (j=0;j<step;++j)
-	{
-          if (exists_player_id(i+j))
-	    {
-	      taken=true;
-	    }
-	}
-      if (!taken)
+      if (!exists_player_id(i))
 	{
 	  result=i;
 	}
     }
 
+  U61_LOG_DEBUG("find_free_player_id "<<result);
+
   return result;
 }
 
@@ -1199,4 +1238,48 @@ U61_Player *U61_Game::find_player_by_id(int id)
   return result;
 }
 
+/*--------------------------------------------------------------------------*/
+/*
+ * Function fired when a sig_quit signal is generated by CL_System
+ */
+void U61_Game::on_quit()
+{
+  U61_LOG_MESSAGE("Quit signal received, exit forced.");
+  /*
+   * It's safe to call quit() directly quit() here, since quit
+   * only positions an integer (game_state) which will fire the
+   * game shutdown at next game loop cycle only.
+   */
+  quit();
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * Callback called when the game session *really* starts, ie when all
+ * players have pressed "ready"
+ */
+void U61_Game::notify_session_start()
+{
+  if (!playing_session)
+    {
+      playing_session = true;
+      U61_Music::start();
+      chat_input.set_active(false);
+      U61_LOG_MESSAGE("Game start.");
+    }
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * Callback called when a game session ends
+ */
+void U61_Game::notify_session_stop()
+{
+  if (playing_session)
+    {
+      playing_session = false;
+      U61_Music::stop();
+      U61_LOG_MESSAGE("Game over.");
+    }
+}
 
