@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -39,8 +39,8 @@
  *              then launches the menu etc...
  */
 
-#ifndef INC_U61_GAME
-#define INC_U61_GAME
+#ifndef U61_INC_GAME
+#define U61_INC_GAME
 
 /*---------------------------------------------------------------------------
  includes
@@ -58,12 +58,15 @@
 #include "player.h"
 #include "protocol.h"
 #include "dispatcher.h"
+#include "wwwregister.h"
+#include "chatinput.h"
 
 /*---------------------------------------------------------------------------
  constants
  ---------------------------------------------------------------------------*/
 
-#define U61_GAME_NB_NET_GROUPS 5
+#define U61_GAME_NB_NET_GROUPS            5
+#define U61_GAME_NB_PLAYERS_PER_NET_GROUP 2
 
 /*---------------------------------------------------------------------------
  classes
@@ -88,7 +91,6 @@ class U61_Game
   void start_local_multiplayer();
   void start_network_new();
   void start_network_join();
-  void start_network_broadcast();
   void do_network_join(char *script,
 		       int id0,int id1,
 		       int speed,int accel,int delay,
@@ -106,6 +108,15 @@ class U61_Game
 
   U61_Menu *get_current_menu();
   void force_menu(U61_Menu *m);
+
+  void notify_session_start();
+  void notify_session_stop();
+
+  /*
+   * Public cause needs to be manipulated in the network menu
+   * to activate/disactivate chat mode.
+   */
+  U61_ChatInput chat_input;
   
  protected:
   char exe_name[U61_CONST_STRING_SIZE];
@@ -114,6 +125,7 @@ class U61_Game
   U61_Player player[U61_LAYOUT_NB_PLAYERS_TOTAL];
   U61_Protocol *protocol;
   U61_Dispatcher *dispatcher;
+  U61_WwwRegister *www_register;
 
   bool standard_init();
   void stop_network();
@@ -137,10 +149,16 @@ class U61_Game
   bool exists_player_id(int player_id);
   int nb_active_players();
 
+  void on_quit();
+
   U61_Menu *current_menu;
 
   int state;
   bool slow_down_allowed;
+  bool playing_session;
+  bool server_mode;
+
+  CL_Slot slot_quit;
 
   int last_clanlib_time_polled;
   int last_u61_time_polled;
