@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -91,6 +91,14 @@ U61_RemoteScript U61_Global::remote_script;
  */
 U61_InputMonitor U61_Global::input_monitor;
 
+/*--------------------------------------------------------------------------*/
+/*
+ * This object is a wrapper placed over ClanLib's API, it's responsible
+ * for setting hooks on the mouse events and maintain the various
+ * mouse parameters up-to-date.
+ */
+U61_MouseMonitor U61_Global::mouse_monitor;
+
 /*--------------------------------------------------------------------------*/
 /*
  * This object holds all the scores for the current game. Scores have been
