@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -37,8 +37,8 @@
  *              and then choose to use one or another
  */
 
-#ifndef INC_U61_GLOBAL
-#define INC_U61_GLOBAL
+#ifndef U61_INC_GLOBAL
+#define U61_INC_GLOBAL
 
 /*---------------------------------------------------------------------------
  includes
@@ -52,6 +52,7 @@
 #include "layout.h"
 #include "remotescript.h"
 #include "inputmonitor.h"
+#include "mousemonitor.h"
 #include "score.h"
 #include "systeminfo.h"
 
@@ -68,6 +69,7 @@ public:
     static U61_Layout layout;
     static U61_RemoteScript remote_script;
     static U61_InputMonitor input_monitor;
+    static U61_MouseMonitor mouse_monitor;
     static U61_Score score;
     static U61_SystemInfo system_info;
 protected:
