@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -36,20 +36,24 @@
  *              happened to each player.
  */
 
-#ifndef INC_U61_HISTORY
-#define INC_U61_HISTORY
+#ifndef U61_INC_HISTORY
+#define U61_INC_HISTORY
 
 /*---------------------------------------------------------------------------
- includes
- ---------------------------------------------------------------------------*/
+  includes
+  ---------------------------------------------------------------------------*/
 
 #include <ClanLib/core.h>
 
 #include "event.h"
 
 /*---------------------------------------------------------------------------
- classes
- ---------------------------------------------------------------------------*/
+  constants
+  ---------------------------------------------------------------------------*/
+
+/*---------------------------------------------------------------------------
+  classes
+  ---------------------------------------------------------------------------*/
 
 class U61_History
 { 
@@ -63,13 +67,21 @@ class U61_History
   int size();
   U61_Event get(int i);
   bool playing();
+  void set_player_id(int pl_id);
+
  protected:   
-  std::vector<U61_Event> events;
+  std::vector<U61_Event> events_playing;
+  std::vector<U61_Event> events_init;
   bool state;
+  bool ready_all;
+  bool put_ready_all;
+  unsigned int last_map_serialization_time;
+  int player_id;
 
   void loose(U61_Event *evt);
-  void start_stop();
+  void start_stop(U61_Event evt);
   void kill();
+  void serialize_map();
 };
 
 #endif
