@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -40,7 +40,8 @@
  ---------------------------------------------------------------------------*/
 
 #include "inputmonitor.h"
-#include "debug.h"
+#include "log.h"
+#include "macro.h"
 
 /*---------------------------------------------------------------------------
  constants
@@ -122,7 +123,7 @@ void U61_InputMonitor::write_config(U61_Config *conf)
   for (i=0;i<U61_INPUTMONITOR_NUM_KEYS_SCANCODE;++i)
     {    
       conf->key[i].ascii=scancode[i].get_ascii();
-      strcpy(conf->key[i].label,scancode[i].get_label());
+      U61_MACRO_STRCPY(conf->key[i].label,scancode[i].get_label());
 
       U61_LOG_DEBUG("writing to config, key scancode="<<i<<", ascii="<<((int) conf->key[i].ascii)<<", label=\""<<conf->key[i].label<<"\"");
     }
