@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -40,12 +40,15 @@
 #include "joinmenu.h"
 #include "networkmenu.h"
 #include "nonpausingmenu.h"
+#include "findgamemenu.h"
 #include "button.h"
 #include "editaddress.h"
+#include "editnumber.h"
+#include "editport.h"
 #include "global.h"
 #include "savescriptmenu.h"
 #include "scoremenu.h"
-#include "debug.h"
+#include "log.h"
 
 /*---------------------------------------------------------------------------
  globals
@@ -63,8 +66,13 @@
 U61_JoinMenu::U61_JoinMenu(U61_Menu *parent)
   : U61_Menu(parent,"Choose your server")
 {
-  add_item(new U61_Button("Join server"));
-  add_item(new U61_EditAddress("Server",U61_Global::config.server_id));
+  add_item(new U61_Button("Find Internet games"));
+  add_item(new U61_Button("Join server now"));
+  add_item(new U61_EditAddress("Server",
+			       U61_Global::config.server_id));
+  add_item(new U61_EditPort("Port",
+			    &(U61_Global::config.network_port)));
+
   select(0);
 }
 
@@ -80,6 +88,9 @@ U61_Menu *U61_JoinMenu::validate()
   switch (selection)
     {
     case 0:
+      next_menu=new U61_FindGameMenu(this);
+      break;
+    case 1:
       next_menu=new U61_NonPausingMenu
 	(new U61_NetworkMenu
 	 (new U61_SaveScriptMenu
