@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -202,6 +202,7 @@ void U61_Layout::init_network_1(U61_Player *p1,
 		U61_DATA_SIZE_BIG);
 
   init_network_common(net);
+  chat_display.set_player_id(p1->get_id());
   U61_Music::set_change_limit(1);
 }
 
@@ -225,6 +226,7 @@ void U61_Layout::init_network_2(U61_Player *p1,
 		U61_DATA_SIZE_BIG);
 
   init_network_common(net);
+  chat_display.set_player_id(p1->get_id());
   U61_Music::set_change_limit(2);
 }
 
@@ -243,8 +245,13 @@ void U61_Layout::init_network_common(U61_Player *net)
    */
   for (i=0;i<U61_LAYOUT_SHUFFLE_COUNT;++i)
     {
-      slot1=U61_Utils::random(8);
-      slot2=U61_Utils::random(8);
+      /*
+       * The 2+random(6) is important we want the 1 & 2 slots
+       * to remain free whenever possible. This is because this
+       * is the place where chat messages are drawn.
+       */
+      slot1=2+U61_Utils::random(6);
+      slot2=2+U61_Utils::random(6);
       temp=corres[slot1];
       corres[slot1]=corres[slot2];
       corres[slot2]=temp;     
@@ -298,6 +305,10 @@ void U61_Layout::draw()
     {
       items[i].draw();
     }    
+
+  chat_display.draw(0,0,
+		    U61_Global::data.screen_w/4,
+		    U61_Global::data.screen_h/2);
 }
 
 /*--------------------------------------------------------------------------*/
@@ -425,6 +436,15 @@ int U61_Layout::get_next_player_id(int old_id,int own_id)
   return result;
 }
 
+/*--------------------------------------------------------------------------*/
+/*
+ * Queues an event to the chat_display object
+ */
+void U61_Layout::put_chat_letter(U61_Event evt)
+{
+  chat_display.put_chat_letter(evt);
+}
+
 /*--------------------------------------------------------------------------*/
 /*
  * returns true if one of the backgrounds has changed
