@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -43,6 +43,7 @@
 #include "layoutitem.h"
 #include "layout.h"
 #include "global.h"
+#include "macro.h"
 
 /*---------------------------------------------------------------------------
  constants
@@ -247,7 +248,7 @@ void U61_LayoutItem::draw()
       player->draw(x_map+U61_Global::data.offset_squares_x[size],
 		   y_map+U61_Global::data.offset_squares_y[size],size);
 
-      sprintf(buffer,"%s",player->get_name());
+      U61_MACRO_SPRINTF1(buffer,"%s",player->get_name());
       buffer[6]=0;
       U61_Global::data.font_info[size]->print_left
 	(x_info+U61_Global::data.offset_name_x[size],
@@ -256,7 +257,7 @@ void U61_LayoutItem::draw()
 
       if (player->is_active())
 	{
-	  sprintf(buffer,"%6d",player->get_score());
+	  U61_MACRO_SPRINTF1(buffer,"%6d",player->get_score());
 	  U61_Global::data.font_info[size]->print_left
 	    (x_info+U61_Global::data.offset_score_x[size],
 	     y_info+U61_Global::data.offset_score_y[size],
@@ -270,8 +271,8 @@ void U61_LayoutItem::draw()
 	  target=U61_Global::game.find_player_by_id(player->get_target_id());
 	  if (target!=NULL && target!=player) 
 	    {
-	      sprintf(buffer,"%s",target->get_name());
-	      buffer[6]=0;
+	      U61_MACRO_SPRINTF1(buffer,"%s",target->get_name());
+	      buffer[U61_DATA_PLAYER_NAME_SIZE]=0;
 	      U61_Global::data.font_info[size]->print_left
         	(x_info+U61_Global::data.offset_target_x[size],
 	         y_info+U61_Global::data.offset_target_y[size],
@@ -282,12 +283,12 @@ void U61_LayoutItem::draw()
 	  buffer[0]=0;
 	  while (i<U61_LAYOUTITEM_LIMIT_CURSE/2 && i<player->get_nb_curse(true))
 	    {
-	      strcat(buffer,U61_DATA_SYMBOL_CURSE_GOOD);
+	      U61_MACRO_STRCAT(buffer,U61_DATA_SYMBOL_CURSE_GOOD);
 	      ++i;
 	    }
 	  while (i<U61_LAYOUTITEM_LIMIT_CURSE && i<player->get_nb_curse(false))
 	    {
-	      strcat(buffer,U61_DATA_SYMBOL_CURSE_BAD);
+	      U61_MACRO_STRCAT(buffer,U61_DATA_SYMBOL_CURSE_BAD);
 	      ++i;
 	    }
 	  U61_Global::data.font_symbol[size]->print_left
@@ -299,7 +300,7 @@ void U61_LayoutItem::draw()
 	  buffer[0]=0;
 	  while (i<U61_LAYOUTITEM_LIMIT_ANTIDOTE && i<player->get_nb_antidote())
 	    {
-	      strcat(buffer,U61_DATA_SYMBOL_ANTIDOTE);
+	      U61_MACRO_STRCAT(buffer,U61_DATA_SYMBOL_ANTIDOTE);
 	      ++i;
 	    }
 	  U61_Global::data.font_symbol[size]->print_left
@@ -307,7 +308,7 @@ void U61_LayoutItem::draw()
 	     y_info+U61_Global::data.offset_antidotes_y[size],
 	     buffer);
 
-	  sprintf(buffer,player->get_curse_name());
+	  U61_MACRO_SPRINTF1(buffer,"%s",player->get_curse_name());
 	  buffer[10]=0;
 	  U61_Global::data.font_info[size]->print_left
 	    (x_info+U61_Global::data.offset_next_curse_x[size],
