@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -41,8 +41,8 @@
 
 
 /*---------------------------------------------------------------------------
- include
- ---------------------------------------------------------------------------*/
+  include
+  ---------------------------------------------------------------------------*/
 
 #include "global.h"
 #include "map.h"
@@ -50,13 +50,18 @@
 #include "time.h"
 #include "sound.h"
 #include "utils.h"
-#include "debug.h"
+#include "log.h"
 #include "music.h"
+#include "serial.h"
+#include "checksum.h"
+#include "macro.h"
 
 /*---------------------------------------------------------------------------
- constants
- ---------------------------------------------------------------------------*/
+  constants
+  ---------------------------------------------------------------------------*/
 
+#define U61_MAP_MIN_WIDTH 2
+#define U61_MAP_MIN_HEIGHT 5
 #define U61_MAP_SPEED_SCALE 1000
 #define U61_MAP_ACCEL_SCALE 100000
 #define U61_MAP_MAX_SYSTEM_SPEED 50
@@ -68,6 +73,7 @@
 #define U61_MAP_CHECK_TARGET_DELAY 100
 #define U61_MAP_CHECK_CURSE_DELAY 100
 #define U61_MAP_ANTICIPATE_MAX 1000
+#define U61_MAP_SERIALIZED_FOR_CHECKSUM_SIZE (U61_MAP_MAX_WIDTH*U61_MAP_MAX_HEIGHT*U61_SQUARE_SERIALIZED_SIZE + 2*U61_BLOCK_SERIALIZED_SIZE + U61_MAP_NB_GLOBAL * U61_SERIAL_INT_SERIALIZED_SIZE + U61_MAP_MAX_CURSE_ID*U61_REGISTEREDCURSE_SERIALIZED_SIZE + 20*U61_SERIAL_INT_SERIALIZED_SIZE + 2*U61_SERIAL_BOOL_SERIALIZED_SIZE)
 
 /*--------------------------------------------------------------------------*/
 /* 
@@ -75,24 +81,24 @@
  * have something more subtle than just a linear behavior
  */ 
 static int speeds[U61_MAP_MAX_SYSTEM_SPEED+1]=
-{
-  1,  2,  5,  10, 20, 25, 30, 35, 40, 45, 50,
-  55, 60, 65, 70, 75, 80, 85, 90, 95, 100,
-  105,110,115,120,125,130,135,140,145,150,
-  155,160,165,170,175,180,185,190,195,200,
-  220,240,260,280,300,350,400,450,500,1000
-};
+  {
+    1,  2,  5,  10, 20, 25, 30, 35, 40, 45, 50,
+    55, 60, 65, 70, 75, 80, 85, 90, 95, 100,
+    105,110,115,120,125,130,135,140,145,150,
+    155,160,165,170,175,180,185,190,195,200,
+    220,240,260,280,300,350,400,450,500,1000
+  };
 
 static int accels[U61_MAP_MAX_SYSTEM_ACCEL+1]=
-{
-  0,  2,  5,  10, 15, 20,
-  30, 40, 50, 75, 100,
-  150,200,250,500,1000
-};
+  {
+    0,  2,  5,  10, 15, 20,
+    30, 40, 50, 75, 100,
+    150,200,250,500,1000
+  };
 
 /*---------------------------------------------------------------------------
- functions
- ---------------------------------------------------------------------------*/
+  functions
+  ---------------------------------------------------------------------------*/
 
 /*--------------------------------------------------------------------------*/
 /* 
@@ -120,13 +126,13 @@ void U61_Map::clear()
 {
   int i,j;
 
-  for (j=0;j<U61_MAP_HEIGHT;++j)
+  for (j=0;j<U61_MAP_MAX_HEIGHT;++j)
     {
-      for (i=0;i<U61_MAP_WIDTH;++i)
+      for (i=0;i<U61_MAP_MAX_WIDTH;++i)
         {
 	  squares[i][j].disable();
 	  //squares[i][j].enable();
-	  squares[i][j].set_color((i+j)%8);
+	  //squares[i][j].set_color((i+j)%8);
         }
     }
   curse_chosen=false;
@@ -145,12 +151,14 @@ void U61_Map::begin()
   int i;
   U61_Event evt;
 
+  width=U61_MAP_MAX_WIDTH;
+  height=U61_MAP_MAX_HEIGHT;
+
   auto_events.clear();
   request_events.clear();
   
   clear();
  
-  name[0]=0; 
   block.reset();
   next_block.reset();
   block_requested=false;
@@ -212,6 +220,9 @@ void U61_Map::reset()
   active=false;
   score=0;
   silent=false;
+  name[0]='\0'; 
+  last_checksum=0;
+  checksum_requested=false;
 
   begin();
 }
@@ -232,10 +243,16 @@ bool U61_Map::is_active()
 /*
  * activates a map
  */
-void U61_Map::set_active(int time)
+void U61_Map::set_active(unsigned int time)
 {
-  begin();
   map_time=time;
+
+  begin();
+  /*
+   * We call the start user func, which will perform initializations.
+   */
+  start();
+
   active=true;
 }
 
@@ -247,6 +264,74 @@ void U61_Map::set_inactive()
 {
   active=false;
 }
+ 
+/*--------------------------------------------------------------------------*/
+/*
+ * Returns the width of the map.
+ */
+int U61_Map::get_width()
+{
+  return width;
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * Returns the height of the map.
+ */
+int U61_Map::get_height()
+{
+  return height;
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * changes the background of the map randomly
+ */
+void U61_Map::set_width(int w)
+{
+  if (w<U61_MAP_MIN_WIDTH) 
+    {
+      w=U61_MAP_MIN_WIDTH;
+    }
+  if (w>U61_MAP_MAX_WIDTH)
+    {
+      w=U61_MAP_MAX_WIDTH;
+    }
+
+  /*
+   * We re-center the map
+   */
+  shift((w-width)/2,0);
+
+  width=w;
+
+  check_map();
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * changes the background of the map randomly
+ */
+void U61_Map::set_height(int h)
+{
+  if (h<U61_MAP_MIN_HEIGHT) 
+    {
+      h=U61_MAP_MIN_HEIGHT;
+    }
+  if (h>U61_MAP_MAX_HEIGHT)
+    {
+      h=U61_MAP_MAX_HEIGHT;
+    }
+
+  /*
+   * We re-center the map
+   */
+  shift(0,h-height);
+
+  height=h;
+
+  check_map();
+}
 
 /*--------------------------------------------------------------------------*/
 /*
@@ -276,7 +361,7 @@ int U61_Map::get_background()
  */
 void U61_Map::set_square_color(int x,int y,int color)
 {
-  if (x>=0 && y>=0 && x<U61_MAP_WIDTH && y<U61_MAP_HEIGHT)
+  if (x>=0 && y>=0 && x<width && y<height)
     {
       if (color>=0 && color<U61_SQUARE_NB_COLORS)
         {
@@ -301,7 +386,7 @@ int U61_Map::get_square_color(int x,int y)
 {
   int color=-1;
 
-  if (x>=0 && y>=0 && x<U61_MAP_WIDTH && y<U61_MAP_HEIGHT)
+  if (x>=0 && y>=0 && x<width && y<height)
     {
       if (squares[x][y].is_enabled())
         {
@@ -488,7 +573,7 @@ void U61_Map::register_curse(int i, int length, bool good)
       registered_curse[i].end=map_time+length;
       registered_curse[i].active=true;
       registered_curse[i].good=good;
-      U61_LOG_DEBUG("Registering "<<i<<" begin="<<curse_begin[i]<<" end="<<curse_end[i]);
+      U61_LOG_DEBUG("Registering "<<i<<" begin="<<registered_curse[i].begin<<" end="<<registered_curse[i].end);
     }
 }
 
@@ -587,7 +672,7 @@ void U61_Map::delete_antidote()
  */
 void U61_Map::set_curse_x(int x)
 {
-  if (x>=0 && x<U61_MAP_WIDTH)
+  if (x>=0 && x<width)
     {
       curse_x=x;
     }
@@ -599,7 +684,7 @@ void U61_Map::set_curse_x(int x)
  */
 void U61_Map::set_curse_y(int y)
 {
-  if (y>=0 && y<U61_MAP_HEIGHT)
+  if (y>=0 && y<height)
     {
       curse_y=y;
     }
@@ -672,7 +757,7 @@ bool U61_Map::is_curse_available()
 {
   bool result=false;
 
-  if (curse_x>=0 && curse_y>=0 && curse_x<U61_MAP_WIDTH && curse_y<U61_MAP_HEIGHT
+  if (curse_x>=0 && curse_y>=0 && curse_x<width && curse_y<height
       && curse_state && curse_chosen
       && squares[curse_x][curse_y].is_enabled())
     {
@@ -756,6 +841,10 @@ void U61_Map::compute_next()
     {
       check_curse();
     }
+  if (!(map_time%U61_MAP_DELAY_BETWEEN_CHECKSUM))
+    {
+      start_checksum();
+    }
   if (!exists_explosion())
     {
       match_pattern();
@@ -795,7 +884,7 @@ void U61_Map::update_speed()
 /*
  * computes the level up to the givent time, no events are handled
  */
-void U61_Map::compute_up_to(int time)
+void U61_Map::compute_up_to(unsigned int time)
 {
   if (active)
     {
@@ -812,19 +901,20 @@ void U61_Map::compute_up_to(int time)
  */
 void U61_Map::handle_event(U61_Event *event)
 {
-  if (active)
+  if (start_time==0)
     {
-      if (start_time==0)
+      start_time=map_time;
+    }
+  if (event->time<map_time)
+    {
+      U61_LOG_DEBUG("Inconsitency in event dates detected with event "<<*event<<" and map_time="<<map_time);
+    }  
+  else
+    {
+      compute_up_to(event->time);
+      
+      if (active)
 	{
-	  start_time=map_time;
-	}
-      if (int(event->time)<map_time)
-        {
-	  U61_LOG_DEBUG("Inconsitency in event dates detected with event "<<*event<<" and map_time="<<map_time);
-        }  
-      else
-        {
-	  compute_up_to(event->time);
 	  switch (event->code)
             {
             case U61_EVENT_ROTATE_LEFT:
@@ -860,9 +950,6 @@ void U61_Map::handle_event(U61_Event *event)
             case U61_EVENT_SET_VICTIM:
               set_victim(event->par);
               break;
-            case U61_EVENT_NAME_LETTER:
-              name_add_letter(event->par);
-	      break;
             case U61_EVENT_NEW_CURSE:
               new_curse(event->par);
 	      break;
@@ -875,8 +962,32 @@ void U61_Map::handle_event(U61_Event *event)
             case U61_EVENT_USE_ANTIDOTE:
               use_antidote();
               break;
+            case U61_EVENT_NAME_LETTER:
+              name_add_letter(event->par);
+	      break;
+            case U61_EVENT_REQUEST_CHECKSUM:
+              request_checksum();
+	      break;
+            case U61_EVENT_VERIFY_CHECKSUM:
+              verify_checksum(event->par);
+	      break;
             }
-        }
+	}
+      else
+	{
+	  switch (event->code)
+            {
+            case U61_EVENT_NAME_LETTER:
+              name_add_letter(event->par);
+	      break;
+            case U61_EVENT_REQUEST_CHECKSUM:
+              request_checksum();
+	      break;
+            case U61_EVENT_VERIFY_CHECKSUM:
+              verify_checksum(event->par);
+	      break;
+	    }
+	}
     }
 
   U61_LOG_DEBUG("Event computed "<<*event);
@@ -894,7 +1005,7 @@ void U61_Map::handle_event(U61_Event *event)
  * since they are of no use when the map has been anticipated and/or
  * we don't want the list to grow and explose system memory...
  */
-void U61_Map::anticipate(int time)
+void U61_Map::anticipate(unsigned int time)
 {
   /*
    * we do not anticipate too much for this can lead to
@@ -976,7 +1087,14 @@ bool U61_Map::exists_auto_event()
  */
 void U61_Map::put_auto_event(U61_Event event)
 {
-  auto_events.push_back(event);
+  if (auto_events.size()<U61_MAP_MAX_AUTO_EVENTS)
+    {
+      auto_events.push_back(event);
+    }
+  else
+    {
+      U61_LOG_WARNING("Too many auto events!");
+    }
 }
 
 /*--------------------------------------------------------------------------*/
@@ -1008,11 +1126,18 @@ bool U61_Map::exists_request_event()
  */
 void U61_Map::put_request_event(U61_Event event)
 {
-  event.target=target_id;
-  event.author=player_id;
-  event.target=target_id;
-  event.time=0;
-  request_events.push_back(event);
+  if (auto_events.size()<U61_MAP_MAX_REQUEST_EVENTS)
+    {
+      event.target=target_id;
+      event.author=player_id;
+      event.target=target_id;
+      event.time=0;
+      request_events.push_back(event);
+    }
+  else
+    {
+      U61_LOG_WARNING("Too many request events!");
+    }
 }
 
 /*--------------------------------------------------------------------------*/
@@ -1032,12 +1157,41 @@ void U61_Map::draw(int x,int y, int size, bool anti)
 {
   int i,j;
   int draw_x,draw_y;
+  int di,dj;
+  U61_Square filler;
 
   if (active)
     {
-      for (j=0;j<U61_MAP_HEIGHT;++j)
+      di=(U61_MAP_MAX_WIDTH-width)/2;
+      dj=(U61_MAP_MAX_HEIGHT-height)/2;
+
+      /*
+       * First we draw the "filler" squares in case map is smaller
+       * than 10*25.
+       */    
+      filler.enable();
+      for (j=0;j<U61_MAP_MAX_HEIGHT;++j)
+	{
+	  for (i=0;i<U61_MAP_MAX_WIDTH;++i)
+	    {
+	      if (i<di || i>=di+width || j<dj || j>=dj+height)
+		{
+		  filler.draw_filler(x+i*U61_Global::data.square_w[size],
+				     y+j*U61_Global::data.square_h[size],
+				     size);
+		}
+	    }
+	}
+      
+      x+=di*U61_Global::data.square_w[size];
+      y+=dj*U61_Global::data.square_h[size];
+
+      /*
+       * Now we draw map contents
+       */    
+      for (j=0;j<height;++j)
         {
-	  for (i=0;i<U61_MAP_WIDTH;++i)
+	  for (i=0;i<width;++i)
             {
 	      draw_x=x+i*U61_Global::data.square_w[size];
 
@@ -1066,10 +1220,28 @@ void U61_Map::draw(int x,int y, int size, bool anti)
         {
 	  draw_anticipation(x,y,size);
         }
-      block.draw(x,y,size);
+      block.draw(x,y,size,width,height);
     }
 }
 
+/*--------------------------------------------------------------------------*/
+/*
+ * returns the width of the map on the screen
+ */
+int U61_Map::get_draw_width(int size)
+{
+  return U61_MAP_MAX_WIDTH*U61_Global::data.square_w[size];
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * returns the height of the map on the screen
+ */
+int U61_Map::get_draw_height(int size)
+{
+  return U61_MAP_MAX_HEIGHT*U61_Global::data.square_h[size];
+}
+
 /*--------------------------------------------------------------------------*/
 /*
  * sets the map in a mute state (does not generate sounds any more)
@@ -1200,7 +1372,7 @@ U61_Block U61_Map::get_next_block()
  */
 void U61_Map::blow_up_square(int x,int y)
 {
-  if (x>=0 && y>=0 && x<U61_MAP_WIDTH && y<U61_MAP_HEIGHT)
+  if (x>=0 && y>=0 && x<width && y<height)
     {
       squares[x][y].begin_explosion();
     }
@@ -1214,7 +1386,7 @@ bool U61_Map::is_square_exploding(int x,int y)
 {
   bool result=false;
 
-  if (x>=0 && y>=0 && x<U61_MAP_WIDTH && y<U61_MAP_HEIGHT)
+  if (x>=0 && y>=0 && x<width && y<height)
     {
       result= squares[x][y].is_exploding();
     }
@@ -1231,8 +1403,8 @@ bool U61_Map::exists_explosion()
   bool result=false;
   int x,y;
 
-  for (y=0;y<U61_MAP_HEIGHT;++y)
-    for (x=0;x<U61_MAP_WIDTH;++x)
+  for (y=0;y<height;++y)
+    for (x=0;x<width;++x)
       {
 	if (squares[x][y].is_exploding())
 	  {
@@ -1265,10 +1437,24 @@ void U61_Map::draw_anticipation(int x,int y,int size)
         }
 
       U61_Script::land(this,&prev);
-      prev.draw_anticipation(x,y,size); 
+      prev.draw_anticipation(x,y,size,width,height); 
     }
 }
 
+/*--------------------------------------------------------------------------*/
+/*
+ * calls the user "start" callback (at the beginning of each game)
+ * it is not possible
+ */
+void U61_Map::start()
+{
+  U61_LOG_DEBUG("start");
+
+  U61_Script::start(this,&block);
+
+  check_block();
+}
+
 /*--------------------------------------------------------------------------*/
 /*
  * tries to rotate the block on the left, but cancels the action if
@@ -1445,17 +1631,25 @@ void U61_Map::block_drop()
 
 /*--------------------------------------------------------------------------*/
 /*
- * Adds a letter to the map name
+ * Adds a letter to the map name. If the value of c is 0 then the name
+ * is "reseted" to "".
  */
 void U61_Map::name_add_letter(char c)
 {
   int len;
 
-  len=strlen(name);
-  if (len<U61_MAP_NAME_SIZE)
+  if (c)
     {
-      name[len]=c;
-      name[len+1]=0;
+      len=strlen(name);
+      if (len<U61_DATA_PLAYER_NAME_SIZE)
+	{
+	  name[len]=c;
+	  name[len+1]=0;
+	}
+    }
+  else
+    {
+      name[0]='\0';
     }
 }
 
@@ -1509,7 +1703,7 @@ bool U61_Map::is_block_ok(U61_Block *test)
        * the square must be in the map, or over the map,
        * if the block is brand new and didn't show himself yet
        */
-      if (x>=0 && x<U61_MAP_WIDTH && y<U61_MAP_HEIGHT)
+      if (x>=0 && x<width && y<height)
         {
 	  /*
 	   * if y>=0, there could potentially be a stable square
@@ -1540,7 +1734,7 @@ bool U61_Map::is_block_ok(U61_Block *test)
 void U61_Map::new_block(int shape)
 {
   block=next_block;
-  block.set_x(U61_MAP_WIDTH/2);
+  block.set_x(width/2);
   block_requested=false;
 
   next_block.reset();
@@ -1586,7 +1780,7 @@ void U61_Map::stabilize_block()
     {
       x=block.get_x()+block.get_item_x(i);
       y=block.get_y()+block.get_item_y(i);
-      if (x>=0 && y>=0 && x<U61_MAP_WIDTH && y<U61_MAP_HEIGHT)
+      if (x>=0 && y>=0 && x<width && y<height)
         {
 	  squares[x][y]=block.get_item(i)->square;
         }
@@ -1624,6 +1818,10 @@ void U61_Map::loose()
   U61_LOG_DEBUG("The game is lost");
 
   begin();
+  /*
+   * We call the start user func, which will perform initializations.
+   */
+  start();
 }
 
 /*--------------------------------------------------------------------------*/
@@ -1722,8 +1920,8 @@ void U61_Map::update_explosions()
   bool curse_exploded=false;
   U61_Event event;
 
-  for (y=0;y<U61_MAP_HEIGHT;++y)
-    for (x=0;x<U61_MAP_WIDTH;++x)
+  for (y=0;y<height;++y)
+    for (x=0;x<width;++x)
       { 
 	squares[x][y].update_explosion();
 	if (squares[x][y].is_explosion_ended())
@@ -1785,8 +1983,8 @@ void U61_Map::new_curse(int id)
   U61_LOG_DEBUG("New curse");
 
   nb_squares=0;
-  for (y=0;y<U61_MAP_HEIGHT;++y)
-    for (x=0;x<U61_MAP_WIDTH;++x)
+  for (y=0;y<height;++y)
+    for (x=0;x<width;++x)
       {
         if (squares[x][y].is_enabled() && 
 	    !squares[x][y].is_exploding())
@@ -1807,8 +2005,8 @@ void U61_Map::new_curse(int id)
        */
       chosen_pos=map_time%nb_squares;
       pos=0;
-      for (y=0;y<U61_MAP_HEIGHT;++y)
-        for (x=0;x<U61_MAP_WIDTH;++x)
+      for (y=0;y<height;++y)
+        for (x=0;x<width;++x)
 	  {
 	    if (squares[x][y].is_enabled() &&
 		!squares[x][y].is_exploding())
@@ -1984,7 +2182,7 @@ void U61_Map::check_block()
    * First step, we check that the block is not on the right or on
    * the left. Basically we want to put it "back in the middle"
    */
-  min_x=U61_MAP_WIDTH-1;
+  min_x=width-1;
   max_x=0;
   for (i=0;i<n;++i)
     {
@@ -2008,9 +2206,9 @@ void U61_Map::check_block()
   /*
    * we move the block to the left if needed
    */
-  if (max_x>U61_MAP_WIDTH-1)
+  if (max_x>width-1)
     {
-      block.set_x(block.get_x()-max_x+U61_MAP_WIDTH-1);
+      block.set_x(block.get_x()-max_x+width-1);
     }
 
   /*
@@ -2083,3 +2281,515 @@ void U61_Map::check_block()
     }
 }
 
+/*-------------------------------------------------------------------------*/
+/*
+ * This function is not as important as check_block but it can be usefull,
+ * especially when resizing the map. Basically it disactivates any square
+ * outside the map borders.
+ */
+void U61_Map::check_map()
+{
+  int i,j;
+
+  for (j=height;j<U61_MAP_MAX_HEIGHT;++j)
+    {
+      for (i=width;i<U61_MAP_MAX_WIDTH;++i)
+        {
+	  squares[i][j].disable();
+        }
+    }
+}
+
+/*-------------------------------------------------------------------------*/
+/*
+ * This function shifts the content of a map vertically or horizontally.
+ * It's mostly used when resizing a map, to recenter its contents. It's
+ * also available within scripts.
+ */
+void U61_Map::shift(int x, int y)
+{
+  int i,j;
+
+  if (x>0 && x<width)
+    {
+      for (j=0;j<height;++j)
+	{
+	  for (i=0;i<x;++i)
+	    {
+	      squares[i][j].disable();
+	    }
+	  for (i=width-1;i>=x;--i)
+	    {
+	      squares[i][j]=squares[i-x][j];  
+	    }
+	}
+    }
+
+  if (x<0 && x>-width)
+    {
+      for (j=0;j<height;++j)
+	{
+	  for (i=width+x;i<width;++i)
+	    {
+	      squares[i][j].disable();
+	    }
+	  for (i=0;i<width+x;++i)
+	    {
+	      squares[i][j]=squares[i-x][j];  
+	    }
+	}
+    }
+
+  if (y>0 && y<height)
+    {
+      for (i=0;i<width;++i)
+	{
+	  for (j=0;j<y;++j)
+	    {
+	      squares[i][j].disable();
+	    }
+	  for (j=height-1;j>=y;--j)
+	    {
+	      squares[i][j]=squares[i][j-y];  
+	    }
+	}
+    }
+
+  if (y<0 && y>-height)
+    {
+      for (i=0;i<width;++i)
+	{
+	  for (j=height+y;j<height;++j)
+	    {
+	      squares[i][j].disable();
+	    }
+	  for (j=0;j<height+y;++j)
+	    {
+	      squares[i][j]=squares[i][j-y];  
+	    }
+	}
+    }
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * Serializes a map into a byte strem
+ */
+bool U61_Map::serialize(unsigned char **buffer, unsigned char *buffer_limit)
+{
+  bool result=true;
+  unsigned char *buffer_start=*buffer;
+  int i,j,size;
+  U61_Event evt;
+  list<U61_Event> list_evt;
+
+  for (i=0;i<U61_MAP_MAX_WIDTH && result;++i)
+    for (j=0;j<U61_MAP_MAX_HEIGHT && result;++j)
+      {
+	result=result && squares[i][j].serialize(buffer,buffer_limit);
+      }
+
+  result=result && block.serialize(buffer,buffer_limit);
+  result=result && next_block.serialize(buffer,buffer_limit);
+
+  list_evt=auto_events;
+  size=list_evt.size();
+  result=result && U61_Serial::serialize_int(size,buffer,buffer_limit);
+  for (i=0;i<U61_MAP_MAX_AUTO_EVENTS && result;++i)
+    {
+      if (i<size)
+	{
+	  result=result && list_evt.front().serialize(buffer,buffer_limit);	  
+	  list_evt.pop_front();
+	}
+      else
+	{
+	  result=result && evt.serialize(buffer,buffer_limit);
+	}
+    }
+
+  list_evt=request_events;
+  size=list_evt.size();
+  result=result && U61_Serial::serialize_int(size,buffer,buffer_limit);
+  for (i=0;i<U61_MAP_MAX_REQUEST_EVENTS && result;++i)
+    {
+      if (i<size)
+	{
+	  result=result && list_evt.front().serialize(buffer,buffer_limit);	  
+	  list_evt.pop_front();
+	}
+      else
+	{
+	  result=result && evt.serialize(buffer,buffer_limit);
+	}
+    }
+
+  for (i=0;i<U61_MAP_NB_GLOBAL && result;++i)
+    {
+      result=result && U61_Serial::serialize_int(global_val[i],buffer,buffer_limit);
+    }
+
+  for (i=0;i<U61_MAP_MAX_CURSE_ID && result;++i)
+    {
+      result=result && registered_curse[i].serialize(buffer,buffer_limit);
+    }
+
+  result=result && U61_Serial::serialize_int(player_id,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_int(target_id,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_int(background,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_bool(active,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_bool(silent,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_int(map_time,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_int(width,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_int(height,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_bool(block_requested,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_bool(matching,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_int(curse_delay,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_int(speed_counter,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_int(accel_counter,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_int(speed,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_int(accel,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_int(base_speed,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_int(base_accel,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_int(score,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_int(curse_sent,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_int(curse_received,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_int(start_time,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_int(match_count,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_bool(anticipation_state,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_bool(preview_state,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_int(curse_x,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_int(curse_y,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_bool(curse_state,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_bool(curse_chosen,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_bool(curse_asked,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_int(curse_id,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_int(curse_counter,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_int(nb_antidote,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_short(last_checksum,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_bool(checksum_requested,buffer,buffer_limit);
+
+  if ((*buffer) - buffer_start != U61_MAP_SERIALIZED_SIZE)
+    {
+      U61_LOG_WARNING("Wrong buffer size for map serialization (diff="<<(buffer_start - (*buffer) + U61_MAP_SERIALIZED_SIZE)<<")!");
+    }
+
+  return result;
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * Un-serializes a map from a byte strem
+ */
+bool U61_Map::unserialize(unsigned char **buffer, unsigned char *buffer_limit)
+{
+  bool result=true;
+  unsigned char *buffer_start=*buffer;
+  int i,j,size;
+  U61_Event evt;
+
+  for (i=0;i<U61_MAP_MAX_WIDTH && result;++i)
+    for (j=0;j<U61_MAP_MAX_HEIGHT && result;++j)
+      {
+	result=result && squares[i][j].unserialize(buffer,buffer_limit);
+      }
+
+  result=result && block.unserialize(buffer,buffer_limit);
+  result=result && next_block.unserialize(buffer,buffer_limit);
+
+  auto_events.clear();
+  result=result && U61_Serial::unserialize_int(&size,buffer,buffer_limit);
+  for (i=0;i<U61_MAP_MAX_AUTO_EVENTS && result;++i)
+    {
+      result=result && evt.unserialize(buffer,buffer_limit);
+      if (i<size)
+	{
+	  auto_events.push_back(evt);
+	}
+    }
+
+  request_events.clear();
+  result=result && U61_Serial::unserialize_int(&size,buffer,buffer_limit);
+  for (i=0;i<U61_MAP_MAX_REQUEST_EVENTS && result;++i)
+    {
+      result=result && evt.unserialize(buffer,buffer_limit);
+      if (i<size)
+	{
+	  request_events.push_back(evt);
+	}
+    }
+
+  for (i=0;i<U61_MAP_NB_GLOBAL && result;++i)
+    {
+      result=result && U61_Serial::unserialize_int(&(global_val[i]),buffer,buffer_limit);
+    }
+
+  for (i=0;i<U61_MAP_MAX_CURSE_ID && result;++i)
+    {
+      result=result && registered_curse[i].unserialize(buffer,buffer_limit);
+    }
+
+  result=result && U61_Serial::unserialize_int(&player_id,buffer,buffer_limit);
+  result=result && U61_Serial::unserialize_int(&target_id,buffer,buffer_limit);
+  result=result && U61_Serial::unserialize_int(&background,buffer,buffer_limit);
+  result=result && U61_Serial::unserialize_bool(&active,buffer,buffer_limit);
+  result=result && U61_Serial::unserialize_bool(&silent,buffer,buffer_limit);
+  result=result && U61_Serial::unserialize_int((int *) &map_time,buffer,buffer_limit);
+  result=result && U61_Serial::unserialize_int(&width,buffer,buffer_limit);
+  result=result && U61_Serial::unserialize_int(&height,buffer,buffer_limit);
+  result=result && U61_Serial::unserialize_bool(&block_requested,buffer,buffer_limit);
+  result=result && U61_Serial::unserialize_bool(&matching,buffer,buffer_limit);
+  result=result && U61_Serial::unserialize_int(&curse_delay,buffer,buffer_limit);
+  result=result && U61_Serial::unserialize_int(&speed_counter,buffer,buffer_limit);
+  result=result && U61_Serial::unserialize_int(&accel_counter,buffer,buffer_limit);
+  result=result && U61_Serial::unserialize_int(&speed,buffer,buffer_limit);
+  result=result && U61_Serial::unserialize_int(&accel,buffer,buffer_limit);
+  result=result && U61_Serial::unserialize_int(&base_speed,buffer,buffer_limit);
+  result=result && U61_Serial::unserialize_int(&base_accel,buffer,buffer_limit);
+  result=result && U61_Serial::unserialize_int(&score,buffer,buffer_limit);
+  result=result && U61_Serial::unserialize_int(&curse_sent,buffer,buffer_limit);
+  result=result && U61_Serial::unserialize_int(&curse_received,buffer,buffer_limit);
+  result=result && U61_Serial::unserialize_int((int *) &start_time,buffer,buffer_limit);
+  result=result && U61_Serial::unserialize_int(&match_count,buffer,buffer_limit);
+  result=result && U61_Serial::unserialize_bool(&anticipation_state,buffer,buffer_limit);
+  result=result && U61_Serial::unserialize_bool(&preview_state,buffer,buffer_limit);
+  result=result && U61_Serial::unserialize_int(&curse_x,buffer,buffer_limit);
+  result=result && U61_Serial::unserialize_int(&curse_y,buffer,buffer_limit);
+  result=result && U61_Serial::unserialize_bool(&curse_state,buffer,buffer_limit);
+  result=result && U61_Serial::unserialize_bool(&curse_chosen,buffer,buffer_limit);
+  result=result && U61_Serial::unserialize_bool(&curse_asked,buffer,buffer_limit);
+  result=result && U61_Serial::unserialize_int(&curse_id,buffer,buffer_limit);
+  result=result && U61_Serial::unserialize_int(&curse_counter,buffer,buffer_limit);
+  result=result && U61_Serial::unserialize_int(&nb_antidote,buffer,buffer_limit);
+  result=result && U61_Serial::unserialize_short(&last_checksum,buffer,buffer_limit);
+  result=result && U61_Serial::unserialize_bool(&checksum_requested,buffer,buffer_limit);
+
+  /*
+   * We call change background() for background depends on local settings,
+   * since it's defined in the theme.
+   */
+  change_background();
+
+  if ((*buffer) - buffer_start != U61_MAP_SERIALIZED_SIZE)
+    {
+      U61_LOG_WARNING("Wrong buffer size for map unserialization (diff="<<(buffer_start - (*buffer) + U61_MAP_SERIALIZED_SIZE)<<")!");
+    }
+
+  return result;
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * Serializes a map into a byte stream, omitting informations which might
+ * interfere with checksum calculus, but still preserving a rather precise
+ * image of map state.
+ */
+bool U61_Map::serialize_for_checksum(unsigned char **buffer, unsigned char *buffer_limit)
+{
+  bool result=true;
+  unsigned char *buffer_start=*buffer;
+  int i,j;
+  U61_Event evt;
+  list<U61_Event> list_evt;
+
+  for (i=0;i<U61_MAP_MAX_WIDTH && result;++i)
+    for (j=0;j<U61_MAP_MAX_HEIGHT && result;++j)
+      {
+	result=result && squares[i][j].serialize(buffer,buffer_limit);
+      }
+
+  result=result && block.serialize(buffer,buffer_limit);
+  result=result && next_block.serialize(buffer,buffer_limit);
+
+  for (i=0;i<U61_MAP_NB_GLOBAL && result;++i)
+    {
+      result=result && U61_Serial::serialize_int(global_val[i],buffer,buffer_limit);
+    }
+
+  for (i=0;i<U61_MAP_MAX_CURSE_ID && result;++i)
+    {
+      result=result && registered_curse[i].serialize(buffer,buffer_limit);
+    }
+
+  result=result && U61_Serial::serialize_int(player_id,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_int(target_id,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_bool(active,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_int(map_time,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_int(width,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_int(height,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_int(curse_delay,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_int(speed_counter,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_int(accel_counter,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_int(speed,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_int(accel,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_int(base_speed,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_int(base_accel,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_int(score,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_int(curse_sent,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_int(curse_received,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_int(start_time,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_int(curse_x,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_int(curse_y,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_bool(curse_state,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_int(curse_id,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_int(nb_antidote,buffer,buffer_limit);
+
+  if ((*buffer) - buffer_start != U61_MAP_SERIALIZED_FOR_CHECKSUM_SIZE)
+    {
+      U61_LOG_WARNING("Wrong buffer size for map serialization in checksum mode (diff="<<(buffer_start - (*buffer) + U61_MAP_SERIALIZED_FOR_CHECKSUM_SIZE)<<")!");
+    }
+
+  return result;
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * Starts the checksum process.
+ */
+void U61_Map::start_checksum()
+{
+  U61_Event event;
+
+  event.code=U61_EVENT_REQUEST_CHECKSUM;
+  event.par=0;
+
+  put_auto_event(event);
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * Generates a REQUEST_CHECKSUM event.
+ */
+void U61_Map::request_checksum()
+{
+  U61_Event event;
+  unsigned char *buf;
+  unsigned char *buf_tmp;
+  short sig=0;
+  U61_Checksum checksum;
+
+  /*
+   * We need to test if we're already waiting for a verify checksum event
+   * to come, otherwise everything would be messed up for the "last_checksum"
+   * value would be overwritten...
+   */
+  if (!checksum_requested)
+    {
+      checksum_requested=true;
+
+      event.code=U61_EVENT_VERIFY_CHECKSUM;
+
+      buf=new unsigned char[U61_MAP_SERIALIZED_FOR_CHECKSUM_SIZE];
+      if (buf)
+	{
+	  buf_tmp=buf;
+	  this->serialize_for_checksum(&buf_tmp,buf_tmp+U61_MAP_SERIALIZED_FOR_CHECKSUM_SIZE);
+	  checksum.calc((char *) buf,U61_MAP_SERIALIZED_FOR_CHECKSUM_SIZE);
+	  sig=checksum.get_short_sig();
+
+	  U61_LOG_DEBUG("Generated checksum: "<<sig<<" map: "<<*this);
+#ifdef U61_DEF_DEBUG
+	  dump_on_log();
+#endif
+
+	  delete buf;
+	}
+
+      event.par=last_checksum=sig;
+
+      /*
+       * Now with this "put_auto_event" trick, only the map on the computer
+       * where the player is actually playing will send the event on the
+       * network, but everybody will receive it. Since last_checksum is
+       * checked _locally_ on every computer, matching last_checksum with
+       * the value in the event should reveal inconsistencies.
+       */
+      put_auto_event(event);
+    }
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * Verifies a checksum.
+ */
+void U61_Map::verify_checksum(short checksum)
+{
+  checksum_requested=false;
+
+  U61_LOG_DEBUG("Comparing checksums : "<<checksum<<" / "<<last_checksum);
+
+  if (checksum != last_checksum)
+    {
+      U61_LOG_WARNING("Wrong checksum for player \""<<name<<"\"!");
+    }
+}
+
+/*------------------------------------------------------------------------- */
+/*
+ * Dumps a map to the log, for debugging purpose.
+ * We don't use the ostream operator here since log messages are limited
+ * in size and we need to split the dump since it's too big.
+ */
+void U61_Map::dump_on_log()
+{
+  unsigned char buffer_bin[U61_MAP_SERIALIZED_SIZE];
+  char buffer_ascii[U61_MAP_SERIALIZED_SIZE*2+1];
+  unsigned char *buffer_tmp;
+  char buffer_log[73]; 
+  char *buffer_tmp2;
+
+  buffer_tmp=buffer_bin;
+  this->serialize(&buffer_tmp,buffer_tmp+U61_MAP_SERIALIZED_SIZE);
+  U61_Utils::to_hexascii(buffer_ascii,buffer_bin,U61_MAP_SERIALIZED_SIZE);
+
+  U61_LOG_DEBUG("------8<--------------------------- [begin dump map]")
+  for (buffer_tmp2=buffer_ascii;buffer_tmp2<buffer_ascii+sizeof(buffer_ascii);buffer_tmp2+=sizeof(buffer_log)-1)
+    {
+      U61_MACRO_STRCPY(buffer_log,buffer_tmp2);
+      U61_LOG_DEBUG(buffer_log);
+    }
+  U61_LOG_DEBUG("------8<--------------------------- [end dump map]")
+}
+
+/*------------------------------------------------------------------------- */
+/*
+ * Usefull to print a map using iostreams
+ */
+ostream &operator <<(ostream &o, U61_Map map)
+{
+  char square_buf[(U61_MAP_MAX_WIDTH + 1) * U61_MAP_MAX_HEIGHT + 2];
+  char square_tmp[2];
+  int i,j;
+  int c;
+
+  square_buf[0]='\0';
+  square_tmp[1]='\0';
+  for (j=0;j<map.get_height();++j)    
+    {
+      for (i=0;i<map.get_width();++i)
+	{
+	  c=map.get_square_color(i,j);
+	  if (c<0)
+	    {
+	      square_tmp[0]='_';
+	    }
+	  else
+	    {
+	      square_tmp[0]='0'+c;
+	    }
+
+	  U61_MACRO_STRCAT(square_buf,square_tmp);
+	}
+      if (j<map.get_height()-1)
+	{
+	  U61_MACRO_STRCAT(square_buf,"/");
+	}
+    }
+
+  return o<<"name=\""<<map.get_name()<<"\""
+	  <<" / time="<<map.get_time()
+	  <<" / score="<<map.get_score()
+	  <<" / size="<<map.get_width()<<"x"<<map.get_height()
+	  <<" / data="<<square_buf;
+}
+
