@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -39,8 +39,8 @@
  *              are determined by embedded LUA scripts
  */
 
-#ifndef INC_U61_MAP
-#define INC_U61_MAP
+#ifndef U61_INC_MAP
+#define U61_INC_MAP
 
 /*---------------------------------------------------------------------------
  includes
@@ -53,19 +53,32 @@
 #include "event.h"
 #include "data.h"
 #include "registeredcurse.h"
+#include "serial.h"
 
 /*---------------------------------------------------------------------------
  constants
  ---------------------------------------------------------------------------*/
 
-#define U61_MAP_WIDTH 10
-#define U61_MAP_HEIGHT 25
-#define U61_MAP_NAME_SIZE 8
+#define U61_MAP_MAX_WIDTH 10
+#define U61_MAP_MAX_HEIGHT 25
 #define U61_MAP_NB_GLOBAL 1000
 #define U61_MAP_MAX_CURSE_ID 1000
 #define U61_MAP_MAX_USER_SPEED 20
 #define U61_MAP_MAX_USER_ACCEL 10
 
+#define U61_MAP_MAX_AUTO_EVENTS        20
+#define U61_MAP_MAX_REQUEST_EVENTS     20
+
+#define U61_MAP_SERIALIZED_SIZE (U61_MAP_MAX_WIDTH*U61_MAP_MAX_HEIGHT*U61_SQUARE_SERIALIZED_SIZE + 2*U61_BLOCK_SERIALIZED_SIZE + 1*U61_SERIAL_INT_SERIALIZED_SIZE + U61_MAP_MAX_AUTO_EVENTS*U61_EVENT_SERIALIZED_SIZE + 1*U61_SERIAL_INT_SERIALIZED_SIZE + U61_MAP_MAX_REQUEST_EVENTS*U61_EVENT_SERIALIZED_SIZE + U61_MAP_NB_GLOBAL * U61_SERIAL_INT_SERIALIZED_SIZE + U61_MAP_MAX_CURSE_ID*U61_REGISTEREDCURSE_SERIALIZED_SIZE + 23*U61_SERIAL_INT_SERIALIZED_SIZE + 10*U61_SERIAL_BOOL_SERIALIZED_SIZE + 1*U61_SERIAL_SHORT_SERIALIZED_SIZE) 
+/*
+ * zlib's doc says we need a buffer 0,1%+12 bytes greater, we use
+ * 0,2%+24 bytes, just to be sure, in case we have a round error
+ * when adding number of bytes.
+ */
+#define U61_MAP_COMPRESSED_SIZE (U61_MAP_SERIALIZED_SIZE+U61_MAP_SERIALIZED_SIZE/500+24)
+
+#define U61_MAP_DELAY_BETWEEN_CHECKSUM 1500
+
 /*---------------------------------------------------------------------------
  classes
  ---------------------------------------------------------------------------*/
@@ -81,7 +94,7 @@ public:
     void clear();
     void begin();
     void handle_event(U61_Event *event);
-    void anticipate(int time);
+    void anticipate(unsigned int time);
 
     void poll_auto_events();
     U61_Event get_auto_event();
@@ -90,6 +103,11 @@ public:
     U61_Event get_request_event();
     bool exists_request_event();
 
+    int get_width();
+    int get_height();
+    void set_width(int w);
+    void set_height(int h);
+
     void change_background();
     int get_background();
     void set_square_color(int x,int y,int color);
@@ -132,11 +150,13 @@ public:
     void draw(int x,int y, 
               int size=U61_DATA_SIZE_BIG,
               bool anti=false);
+    int get_draw_width(int size);
+    int get_draw_height(int size);
 
     void mute();
 
     bool is_active();
-    void set_active(int time);
+    void set_active(unsigned int time);
     void set_inactive();
 
     void set_anticipation_state(bool state);
@@ -159,8 +179,13 @@ public:
 
     U61_Block get_next_block();
 
+    void shift(int x, int y);
+
+    bool serialize(unsigned char **buffer, unsigned char *buffer_limit);
+    bool unserialize(unsigned char **buffer, unsigned char *buffer_limit);
+
 protected:
-    U61_Square squares[U61_MAP_WIDTH][U61_MAP_HEIGHT];
+    U61_Square squares[U61_MAP_MAX_WIDTH][U61_MAP_MAX_HEIGHT];
     U61_Block block;
     U61_Block next_block;
     U61_Block block_anticipation;
@@ -168,14 +193,16 @@ protected:
     std::list<U61_Event> request_events;
     int global_val[U61_MAP_NB_GLOBAL];
     U61_RegisteredCurse registered_curse[U61_MAP_MAX_CURSE_ID];
-    char name[U61_MAP_NAME_SIZE+1];
+    char name[U61_DATA_PLAYER_NAME_SIZE+1];
 
     int player_id;
     int target_id;
     int background;
     bool active;
     bool silent;
-    int map_time;
+    unsigned int map_time;
+    int width;
+    int height;
     bool block_requested;
     bool matching;
     int curse_delay;
@@ -188,7 +215,7 @@ protected:
     int score;
     int curse_sent;
     int curse_received;
-    int start_time;
+    unsigned int start_time;
     int match_count;
     bool anticipation_state;
     bool preview_state;
@@ -200,15 +227,19 @@ protected:
     int curse_id;
     int curse_counter;
     int nb_antidote;
+    short last_checksum;
+    bool checksum_requested;
 
     void compute_next();
     void update_speed();
-    void compute_up_to(int time);
+    void compute_up_to(unsigned int time);
     void put_auto_event(U61_Event event);
     void put_request_event(U61_Event event);
 
     void draw_anticipation(int x,int y, int size=U61_DATA_SIZE_BIG);
 
+    void start();
+
     void block_rotate_left();
     void block_rotate_right();
     void block_move_left();
@@ -244,8 +275,18 @@ protected:
     void check_curse();
     void check_target();
     void check_block();
+    void check_map();
+
+    bool serialize_for_checksum(unsigned char **buffer, unsigned char *buffer_limit);
+    void start_checksum();
+    void request_checksum();
+    void verify_checksum(short checksum);
+
+    void dump_on_log();
 };
 
+ostream &operator <<(ostream &o, U61_Map map);
+
 #endif
 
 
