@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -36,10 +36,10 @@
 
 
 /*---------------------------------------------------------------------------
- includes
- ---------------------------------------------------------------------------*/
+  includes
+  ---------------------------------------------------------------------------*/
 
-#ifdef U61_MSVC
+#ifdef U61_DEF_MSVC
 #include <minmax.h>
 #endif
 
@@ -48,29 +48,50 @@
 #include "sound.h"
 #include "global.h"
 #include "menuinput.h"
-#include "debug.h"
+#include "log.h"
+#include "setup.h"
+#include "macro.h"
 
 /*---------------------------------------------------------------------------
- globals
- ---------------------------------------------------------------------------*/
+  constants
+  ---------------------------------------------------------------------------*/
+
+/*--------------------------------------------------------------------------*/
+/*
+ * Message are displayed for a limited time only, after, they enter
+ * a "blink" state.
+ */
+#define U61_MENU_INFO_DELAY  (5*U61_TIME_ONE_SECOND)
+#define U61_MENU_INFO_BLINK_DELAY  (U61_TIME_ONE_SECOND/2)
+
+/*---------------------------------------------------------------------------
+  globals
+  ---------------------------------------------------------------------------*/
+
+/*--------------------------------------------------------------------------*/
+/*
+ * Class attributes used to display messages (mostly during the connection)
+ */
+unsigned int U61_Menu::info_time_limit=0;
+char U61_Menu::info_text[U61_CONST_STRING_SIZE]={0};
 
 /*--------------------------------------------------------------------------*/
 /*
  * messages displayed at the bottom of the screen in the menus
  */
 char *U61_Menu::bottom_message[U61_MENU_BOTTOM_MESSAGE_NB]=
-{
-  "U61, another block-based game by U-Foot",
-  "Copyright (C) 2000-2002 Christian Mauduit",
-  "Home page http://www.ufoot.org/u61",
-  "Contact author : ufoot@ufoot.org",
-  "U61 uses ClanLib http://clanlib.org",
-  "Graphics by U-Woman http://www.absorbed.org",
-  "Free as in speech http://www.gnu.org",
-  "Available on sourceforge http://sourceforge.net",
-  "Game rules can be changed with lua scripts, try it...",
-  "Have a good day!",
-};
+  {
+    "U61, another block-based game by U-Foot",
+    "Copyright (C) 2000-2003 Christian Mauduit",
+    "Home page http://www.ufoot.org/u61",
+    "Contact author : ufoot@ufoot.org",
+    "U61 uses ClanLib http://clanlib.org",
+    "Default theme graphics by U-Woman http://www.absorbed.org",
+    "Free as in speech http://www.fsf.org",
+    "Available on Savannah http://savannah.gnu.org",
+    "Game rules can be changed with lua scripts, try it...",
+    "Have a good day!",
+  };
 
 /*--------------------------------------------------------------------------*/
 /*
@@ -87,8 +108,8 @@ int U61_Menu::bottom_message_width=-1000000;
 bool U61_Menu::shaded=false;
 
 /*---------------------------------------------------------------------------
- functions
- ---------------------------------------------------------------------------*/
+  functions
+  ---------------------------------------------------------------------------*/
 
 /*--------------------------------------------------------------------------*/
 /* 
@@ -100,6 +121,8 @@ U61_Menu::U61_Menu(U61_Menu *parent,char *top)
   top_message=top;
   message_active=true;
   drawn_once=false;
+  info_time_limit=0;
+  info_text[0]='\0';
 }
 
 /*--------------------------------------------------------------------------*/
@@ -108,14 +131,7 @@ U61_Menu::U61_Menu(U61_Menu *parent,char *top)
  */ 
 U61_Menu::~U61_Menu()
 {
-  int i;
-  int nb_item;
-
-  nb_item=items.size();
-  for (i=0;i<nb_item;++i)
-    {
-      delete items[i];
-    }
+  remove_items();
 }
 
 /*--------------------------------------------------------------------------*/
@@ -125,38 +141,79 @@ U61_Menu::~U61_Menu()
 U61_Menu *U61_Menu::action(int key)
 {
   U61_Menu *next_menu;
+  U61_Menu *item_menu;
+  int nb_item;
   int old_selection;
 
   next_menu=this;
+
+  nb_item=items.size();
   old_selection=selection;
-
-  if (!(items[selection]->is_modal()))
+	  
+  /*
+   * We test escape & f10 separately for they do not need
+   * a selected item to have a meaning
+   */
+  switch (key)
     {
-      switch (key)
-        {
-        case CL_KEY_UP:
-	  select_up();
-	  break;
-        case CL_KEY_DOWN:
-	  select_down();
-	  break;
-        case CL_KEY_ESCAPE:
-	  next_menu=back();
-	  break;
-        case CL_KEY_ENTER:
-	  next_menu=validate();
-	  break;
-        }
-    }
+    case CL_KEY_ESCAPE:
+      next_menu=back();
+      break;
+    case CL_KEY_F4:
+      U61_Global::game.chat_input.toggle();
+      break;
+    case CL_KEY_F10:
+      /*
+       * F10 causes a brutal exit without any warning, nice when
+       * developping the game or when your boss is coming.
+       */
+      U61_LOG_MESSAGE("User pressed F10, exit forced.");
+      U61_Global::game.quit();
+      break;
+    default:
+      if (0<=selection && selection<nb_item)
+	{
+	  if (!(items[selection]->is_modal()))
+	    {
+	      switch (key)
+		{
+		case CL_KEY_UP:
+		  select_up();
+		  break;
+		case CL_KEY_DOWN:
+		  select_down();
+		  break;
+		case CL_KEY_ENTER:
+		  /*
+		   * First we try the menu item validate function
+		   */
+		  item_menu=items[selection]->validate();
+		  if (item_menu)
+		    {
+		      /*
+		       * It returned a non-null value, therefore this
+		       * will be the next menu.
+		       */
+		      next_menu=item_menu;
+		    }
+		  else
+		    {
+		      next_menu=validate();
+		    }
+		  break;
+		}	    
+	    }
 
-  if (items[selection]->action(key))
-    {
-      U61_Sound::play_menu_move();
+	  if (items[selection]->action(key))
+	    {
+	      U61_Sound::play_menu_move();
+	    }
+	  
+	  items[selection]->update();
+	  items[selection]->select();
+	}
     }
 
-  items[selection]->update();
-  items[selection]->select();
-
   if (next_menu!=this)
     {
       U61_Sound::play_menu_validate();
@@ -164,9 +221,18 @@ U61_Menu *U61_Menu::action(int key)
   else
     {
       if (old_selection!=selection)
-        {
-	  U61_Sound::play_menu_move();
-        }
+	{
+	  if (items[selection]->selectable())
+	    {
+	      U61_Sound::play_menu_move();
+	    }
+	  else
+	    {
+	      selection=old_selection;
+
+	      select(old_selection);
+	    }
+	}
     }
 
   return next_menu;
@@ -174,27 +240,46 @@ U61_Menu *U61_Menu::action(int key)
 
 /*--------------------------------------------------------------------------*/
 /*
- * Called when the user has pressed the mouse
+ * Used to handle mouse moves
  */
-U61_Menu *U61_Menu::mouse_action(int mouse_y)
+void U61_Menu::mouse_move(int x, int y)
 {
-  U61_Menu *next_menu;
-  int nb_item;
-  int i;
+  bool mouse_in;
+  int mouse_item;
+  int mouse_char;
+  int old_selection;
 
-  next_menu=this;
+  old_selection = selection;
 
-  nb_item=items.size();
-  for (i=0;i<nb_item;++i)
+  draw_ex(false,
+	  x,y,
+	  &mouse_in,
+	  &mouse_item,
+	  &mouse_char);
+
+  if (mouse_in)
     {
-      if (items[i]->is_mouse_in(mouse_y))
-	{
-          select(i);
-          next_menu=action(CL_KEY_ENTER);
-	}
-    }  
+      U61_LOG_DEBUG("Mouse in menu item="<<mouse_item<<", char="<<mouse_char);
 
-  return next_menu;
+      if (items[mouse_item]->selectable())
+	{
+	  items[mouse_item]->update();
+	  items[mouse_item]->select(mouse_char);
+	  select(mouse_item);
+	}
+    }
+
+  if (old_selection!=selection)
+    {
+      if (items[selection]->selectable())
+	{
+	  U61_Sound::play_menu_move();
+	}
+      else
+	{
+	  select(old_selection);
+	}
+    }
 }
 
 /*--------------------------------------------------------------------------*/
@@ -220,100 +305,177 @@ U61_Menu *U61_Menu::back()
  * draws the current menu
  */
 void U61_Menu::draw()
+{
+  draw_ex(true);
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * Internal function used to draw the menu. It can also do a "dry run" of
+ * menu drawing, this is usefull to know where the mouse would have been.
+ */
+void U61_Menu::draw_ex(bool do_draw,
+		       int mouse_x,
+		       int mouse_y,
+		       bool *mouse_in,
+		       int *mouse_item,
+		       int *mouse_char)
 {
   int y;
   int i;
   int nb_item;
-  int last_item_height2;
-  int item_height;
   int max_disp_item;
   int first_disp_item;
   int real_disp_item;
+  bool info_active;
+  bool info_available;
+  unsigned int time;
 
-  if (!drawn_once)
+  if (mouse_in)
+    {
+      *mouse_in = false;
+    }
+
+  if (mouse_item)
+    {
+      *mouse_item = -1;
+    }
+
+  if (mouse_char)
+    {
+      *mouse_char = -1;
+    }
+
+  if (do_draw && shaded)
+    {
+      U61_Global::data.shade->put_screen(0,0);
+    }
+
+  nb_item=items.size();
+  time=U61_Time::for_effect();
+  info_available=(nb_item == 0 && strlen(info_text) > 0);	      
+  info_active=(info_available && 
+	       (info_time_limit>time ||
+		((time/U61_MENU_INFO_BLINK_DELAY)%2)));
+  
+  /*
+   * Ugly hack to make the score menu work: we initialize
+   * it the first time we draw the menu, that's to say after
+   * the game - but the menu had been created *before* the game
+   * so we can't know the score within the constructor
+   */
+  if (do_draw && !drawn_once)
     {
       before_first_draw();
       drawn_once=true;
     }
 
-  nb_item=items.size();
-  /*
-   * We calculate the number of displayable items. To amke the calculus,
-   * we assume that the average height of a menu item is the height of the
-   * font. Then we choose the highest odd number so that there's still room
-   * on the screen to draw the top and bottom messages, plus a gap between
-   * those messages and the real menu.
-   */
-  max_disp_item=(U61_Global::data.screen_h
-		 /U61_Global::data.font_menu->get_height()
-		 )-3;
-  if (!(max_disp_item%2))
-    {
-      ++max_disp_item;
-    }
-
-  if (nb_item<=max_disp_item)
+  if (nb_item>0 && !info_active)
     {
       /*
-       * This is the easy case, we have enough room to display all the items
-       * so the first item is number 0, and there are as many displayed items
-       * as there are items in the menu.
+       * We calculate the number of displayable items. To amke the calculus,
+       * we assume that the average height of a menu item is the height of the
+       * font. Then we choose the highest odd number so that there's still room
+       * on the screen to draw the top and bottom messages, plus a gap between
+       * those messages and the real menu.
        */
-      first_disp_item=0;
-      real_disp_item=nb_item;
-    }
-  else
-    {
-      /*
-       * Now this is the worst case, for all the items can't fit in a screen.
-       * We have to choose which items we display. The algorithm assumes that
-       * the maximum number of displayable items (max_nb_item) is odd.
-       */
-      real_disp_item=max_disp_item;
-      /*
-       * By default we suppose we are at the beginning
-       */
-      first_disp_item=0;
-      /*
-       * Now if we are too far in the menu, the selected item we'll be
-       * in the center of the screen
-       */
-      if (selection>max_disp_item/2)
+      max_disp_item=(U61_Global::data.screen_h
+		     /U61_Global::data.font_menu->get_height()
+		     )-3;
+      if (!(max_disp_item%2))
 	{
-	  first_disp_item=selection-max_disp_item/2;
+	  ++max_disp_item;
 	}
-      /*
-       * If we are at the end of the menu, always display the last item
-       * at the bottom of the screen
-       */
-      if (selection>nb_item-max_disp_item/2-1)
+
+      if (nb_item<=max_disp_item)
 	{
-	  first_disp_item=nb_item-max_disp_item;
+	  /*
+	   * This is the easy case, we have enough room to display all the items
+	   * so the first item is number 0, and there are as many displayed items
+	   * as there are items in the menu.
+	   */
+	  first_disp_item=0;
+	  real_disp_item=nb_item;
+	}
+      else
+	{
+	  /*
+	   * Now this is the worst case, for all the items can't fit in a screen.
+	   * We have to choose which items we display. The algorithm assumes that
+	   * the maximum number of displayable items (max_nb_item) is odd.
+	   */
+	  real_disp_item=max_disp_item;
+	  /*
+	   * By default we suppose we are at the beginning
+	   */
+	  first_disp_item=0;
+	  /*
+	   * Now if we are too far in the menu, the selected item we'll be
+	   * in the center of the screen
+	   */
+	  if (selection>max_disp_item/2)
+	    {
+	      first_disp_item=selection-max_disp_item/2;
+	    }
+	  /*
+	   * If we are at the end of the menu, always display the last item
+	   * at the bottom of the screen
+	   */
+	  if (selection>nb_item-max_disp_item/2-1)
+	    {
+	      first_disp_item=nb_item-max_disp_item;
+	    }
+	}
+
+      /*
+       * The first step is to calculate all the y coordinates for all the
+       * menus. Then we''l decide which ones we want to display and where,
+       * in case there are too many of them
+       */
+      y=U61_Global::data.screen_h/2;
+
+      y-=get_height(first_disp_item,real_disp_item)/2;
+
+      for (i=first_disp_item;i<first_disp_item+real_disp_item;++i)
+	{
+	  /*
+	   * we try and see in which item the mouse is
+	   */
+	  if (mouse_in && mouse_item && mouse_char)
+	    {
+	      if (!(*mouse_in))
+		{
+		  items[i]->test_mouse(y,
+				       mouse_x,
+				       mouse_y,
+				       mouse_in,
+				       mouse_char);
+		  if (*mouse_in)
+		    {
+		      *mouse_item=i;
+		    }
+		}
+	    }
+
+	  /*
+	   * Draw stuff if necessary only...
+	   */
+	  if (do_draw)
+	    {
+	      items[i]->draw(y);
+	    }
+
+	  y+=items[i]->get_height();
 	}
     }
 
-  if (shaded)
+  if (info_active)
     {
-      U61_Global::data.shade->put_screen(0,0);
-    }
-
-  /*
-   * The first step is to calculate all the y coordinates for all the
-   * menus. Then we''l decide which ones we want to display and where,
-   * in case there are too many of them
-   */
-  y=U61_Global::data.screen_h/2;
-
-  y-=get_height(first_disp_item,real_disp_item)/2;
-  last_item_height2=0;
-
-  for (i=first_disp_item;i<first_disp_item+real_disp_item;++i)
-    {
-      item_height=items[i]->get_height();
-
-      y+=last_item_height2+item_height/2;
-      items[i]->draw(y);
-      last_item_height2=item_height-item_height/2;
+      /*
+       * Special case, we don't display the normal menu items
+       * but a special "info" message.
+       */
+      draw_info();
     }
 
   /*
@@ -321,7 +483,7 @@ void U61_Menu::draw()
    * name of the menu and botton message is a scrolling general information
    * message
    */
-  if (message_active)
+  if (message_active || info_available)
     {
       draw_top_message();
       draw_bottom_message();
@@ -346,9 +508,9 @@ void U61_Menu::before_first_draw()
  * This function is called on a regular basis, and can be redefined
  * to handle dynamic menus such as the server list.
  */
-void U61_Menu::poll()
+U61_Menu *U61_Menu::poll()
 {
-
+  return this;
 }
 
 /*--------------------------------------------------------------------------*/
@@ -397,18 +559,27 @@ void U61_Menu::select(int num_item)
   int i;
 
   U61_LOG_DEBUG("Selecting item "<<num_item);
+
   nb_item=items.size();
-  for (i=0;i<nb_item;++i)
+
+  if (0<=num_item && num_item<nb_item)
     {
-      if (i==num_item)
-        {
-	  items[i]->select();
-        }
-      else
-        {
-	  items[i]->unselect();
-        }
+      if (items[num_item]->selectable())
+	{
+	  for (i=0;i<nb_item;++i)
+	    {
+	      if (i==num_item && items[i]->selectable())
+		{
+		  items[i]->select();
+		}
+	      else
+		{
+		  items[i]->unselect();
+		}
+	    }
+	}
     }
+
   selection=num_item;
 }
 
@@ -464,6 +635,26 @@ void U61_Menu::add_item(U61_MenuItem *item)
   items.push_back(item);
 }
 
+/*--------------------------------------------------------------------------*/
+/*
+ * removes all the menu items
+ */
+void U61_Menu::remove_items()
+{
+  int i;
+  int nb_item;
+
+  nb_item=items.size();
+  for (i=0;i<nb_item;++i)
+    {
+      delete items[i];
+    }
+
+  items.clear();
+
+  select(0);
+}
+
 /*--------------------------------------------------------------------------*/
 /*
  * draws the top message only
@@ -479,6 +670,22 @@ void U61_Menu::draw_top_message()
   U61_Global::data.font_menu->print_left(x,y,top_message);
 }
 
+/*--------------------------------------------------------------------------*/
+/*
+ * draws the "popup" information.
+ */
+void U61_Menu::draw_info()
+{
+  int x,y;
+
+  x=(U61_Global::data.screen_w-
+     U61_Global::data.font_menu->get_text_width(info_text))/2;
+  y=(U61_Global::data.screen_h-
+     U61_Global::data.font_menu->get_height())/2;
+
+  U61_Global::data.font_menu->print_left(x,y,info_text);
+}
+
 /*--------------------------------------------------------------------------*/
 /*
  * draws the bottom message only
@@ -540,3 +747,23 @@ void U61_Menu::unshade()
   shaded=false;
 }
 
+/*--------------------------------------------------------------------------*/
+/*
+ * Unshades what's behind the menu, (the contrary of shade())
+ * This is the default behavior
+ */
+void U61_Menu::display_info(char *text)
+{
+  if ((text!=NULL) && strlen(text)>0)
+    {
+      info_time_limit=U61_MENU_INFO_DELAY+U61_Time::for_effect();
+      U61_MACRO_STRCPY(info_text,text);
+    }
+  else
+    {
+      info_time_limit=0;
+      info_text[0]='\0';
+    }
+}
+
+
