@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -34,8 +34,8 @@
  *              it and handle menuitems in a rational way
  */
 
-#ifndef INC_U61_MENU
-#define INC_U61_MENU
+#ifndef U61_INC_MENU
+#define U61_INC_MENU
 
 /*---------------------------------------------------------------------------
  includes
@@ -44,7 +44,7 @@
 #include <ClanLib/core.h>
 
 #include "menuitem.h"
-
+#include "const.h"
 
 /*---------------------------------------------------------------------------
  constants
@@ -64,22 +64,24 @@ class U61_Menu
   virtual ~U61_Menu();
     
   U61_Menu *action(int key);
-  U61_Menu *mouse_action(int mouse_y);
+  void mouse_move(int x, int y);
   virtual U61_Menu *validate();
   virtual U61_Menu *back();
   void draw();
   virtual void before_first_draw();
-  virtual void poll();
+  virtual U61_Menu *poll();
   void select(int num_item);
   virtual void select_up();
   virtual void select_down();
   int get_height(int first, int nb);
   int get_width(int first, int nb);
   void add_item(U61_MenuItem *item);
+  void remove_items();
   U61_Menu *get_parent();
 
   void shade();
   void unshade();
+  static void display_info(char *text);
 
  protected:
   std::vector<U61_MenuItem *> items;
@@ -91,14 +93,24 @@ class U61_Menu
   static bool shaded;
   bool message_active;
 
-  void draw_top_message();
-  void draw_bottom_message();
-  void init_bottom_message();
+  static unsigned int info_time_limit;
+  static char info_text[U61_CONST_STRING_SIZE];
 
   static int current_bottom_message;
   static int bottom_message_start_time;
   static int bottom_message_width;
   static char *bottom_message[U61_MENU_BOTTOM_MESSAGE_NB];
+
+  void draw_top_message();
+  void draw_info();
+  void draw_bottom_message();
+  void init_bottom_message();
+  void draw_ex(bool do_draw,
+	       int mouse_x=-1,
+	       int mouse_y=-1,
+	       bool *mouse_in=NULL,
+	       int *mouse_item=NULL,
+	       int *mouse_char=NULL);
 };
 
 #endif
