@@ -6,32 +6,45 @@
  * U U   6 6  1   b b  y   U U   F   O O O O  T                             *
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
- * This project is protected by the GPL, see copying.txt                    *
+ * U61 is another block based game                                          *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
+ * This program is free software; you can redistribute it and/or            *
+ * modify it under the terms of the GNU General Public License              *
+ * as published by the Free Software Foundation; either version 2           *
+ * of the License, or (at your option) any later version.                   *
+ *                                                                          *
+ * This program is distributed in the hope that it will be useful,          *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
+ * GNU General Public License for more details.                             *
+ *                                                                          *
+ * You should have received a copy of the GNU General Public License        *
+ * along with this program; if not, write to the Free Software              *
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
+ *                                                                          *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
-/* name:        menuinput.cpp
- * author       U-Foot (ufoot@bigfoot.com)
- * description: input class which handles menu evemts
+/*
+ * file name:   menuinput.cpp
+ * author:      U-Foot (ufoot@ufoot.org / www.ufoot.org)
+ * description: input class which handles menu events from both keyboard
+ *              and mouse.
  */
 
 /*---------------------------------------------------------------------------
  includes
  ---------------------------------------------------------------------------*/
 
-#include "menuinput.h"
-#include "inputmonitor.h"
-#include "time.h"
-#include "debug.h"
+#include "completeinput.h"
+#include "log.h"
+#include "global.h"
 
 /*---------------------------------------------------------------------------
  constants
  ---------------------------------------------------------------------------*/
 
-#define U61_MENUINPUT_MOUSE_DELAY       50
-#define U61_MENUINPUT_REPEAT_FIRST      50
-#define U61_MENUINPUT_REPEAT_STANDARD   15
-
 /*---------------------------------------------------------------------------
  functions
  ---------------------------------------------------------------------------*/
@@ -40,9 +53,9 @@
 /* 
  * creation of a menu input object
  */ 
-U61_MenuInput::U61_MenuInput() : U61_Input(U61_INPUTMONITOR_NUM_KEYS)
+U61_MenuInput::U61_MenuInput() : U61_CompleteInput()
 {
-  init();
+
 }
 
 /*--------------------------------------------------------------------------*/
@@ -55,21 +68,13 @@ U61_MenuInput::~U61_MenuInput()
 
 /*--------------------------------------------------------------------------*/
 /*
- * initializations of the menu keys
+ * Polls the mouse for basic clicks
  */
-void U61_MenuInput::init()
+void U61_MenuInput::poll(int time)
 {
-  int i;
-  U61_Key key;
+  U61_CompleteInput::poll(time);
 
-  mouse_last_fired=-1;
-  mouse_y=-1;
-
-  for (i=0;i<U61_INPUTMONITOR_NUM_KEYS;++i)
-    {
-      key.init(i,i,U61_MENUINPUT_REPEAT_FIRST,U61_MENUINPUT_REPEAT_STANDARD);
-      set_key(i,key);
-    } 
+  poll_mouse();
 }
 
 /*--------------------------------------------------------------------------*/
@@ -78,40 +83,13 @@ void U61_MenuInput::init()
  */
 void U61_MenuInput::poll_mouse()
 {
-  if (CL_Mouse::middle_pressed() ||
-      CL_Mouse::left_pressed() ||
-      CL_Mouse::right_pressed())
+  if (U61_Global::mouse_monitor.left_pressed())
     {
-      mouse_y=CL_Mouse::get_y();
+      simulate(CL_KEY_ENTER);
     }
-  else
+  if (U61_Global::mouse_monitor.right_pressed())
     {
-      mouse_y=-1;
+      simulate(CL_KEY_ESCAPE);
     }
-
-  /*
-   * For now this mouse support just sucks, so I have disabled it,
-   * so that people do not think the GUI is buggy. It's better for
-   * everyone to use the keyboard only for now...
-   */
-  mouse_y=-1;
 }
 
-/*--------------------------------------------------------------------------*/
-/*
- * Returns the y position of the mouse if it has been clicked, or -1
- */
-int U61_MenuInput::get_mouse_y()
-{
-  int result=-1;
-  int time;
-
-  time=U61_Time::for_effect();
-  if (time-mouse_last_fired>U61_MENUINPUT_MOUSE_DELAY)
-    {
-      mouse_last_fired=time;
-      result=mouse_y;
-    }
-
-  return result;
-}
