@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -41,8 +41,8 @@
  *              and automatically centers itself
  */
 
-#ifndef INC_U61_MENUITEM
-#define INC_U61_MENUITEM
+#ifndef U61_INC_MENUITEM
+#define U61_INC_MENUITEM
 
 /*---------------------------------------------------------------------------
  includes
@@ -51,6 +51,7 @@
 #include <ClanLib/core.h>
 
 #include "letter.h"
+#include "textdisplayer.h"
 
 /*---------------------------------------------------------------------------
  constants
@@ -62,6 +63,8 @@
  classes
  ---------------------------------------------------------------------------*/
 
+class U61_Menu;
+
 class U61_MenuItem
 {
  public:
@@ -72,24 +75,36 @@ class U61_MenuItem
   virtual bool action(int key);
   virtual void update();
   void draw(int y);
+  void test_mouse(int y, 
+		  int mouse_x,
+		  int mouse_y,
+		  bool *mouse_in, 
+		  int *mouse_char);
   int get_width();
   int get_height();
   void set_text(char *str);
-  void put_text(char *buf);
+  void put_text(char *buf, int max);
+  void set_align(int al);
 
   void select_char(int i);
   void select_right(int start);
   void blink_char(int i);
-  virtual void select();
+  void select();
+  virtual void select(int pos);
   virtual void unselect();
   virtual bool is_modal();
+  virtual bool check_value();
 
-  bool is_mouse_in(int y);
+  virtual U61_Menu *validate();
+
+  virtual bool selectable();
  protected:
-  std::vector<U61_Letter> text;    
+  void unselect_ex();
+
+  U61_TextDisplayer text_displayer;
   bool selected;
-  int mouse_up_limit;
-  int mouse_down_limit;
+  int align;
+  int selection;
 };
 
 #endif
