@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -45,6 +45,7 @@
 #include "playermenu.h"
 #include "soundmenu.h"
 #include "graphicsmenu.h"
+#include "systemmenu.h"
 #include "button.h"
 #include "time.h"
 #include "global.h"
@@ -73,6 +74,7 @@ U61_MultiplayerMenu::U61_MultiplayerMenu(U61_Menu *parent)
   add_item(new U61_Button("Player 4 options"));
   add_item(new U61_Button("Graphics"));
   add_item(new U61_Button("Sound"));
+  add_item(new U61_Button("System"));
   select(0);
 }
 
@@ -113,6 +115,9 @@ U61_Menu *U61_MultiplayerMenu::validate()
     case 7:
       next_menu=new U61_SoundMenu(this, &(U61_Global::config));
       break;
+    case 8:
+      next_menu=new U61_SystemMenu(this, &(U61_Global::config));
+      break;
     }
 
   return next_menu;
