@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -42,7 +42,7 @@
 
 #include "music.h"
 #include "global.h"
-#include "debug.h"
+#include "log.h"
 #include "utils.h"
 
 /*---------------------------------------------------------------------------
@@ -94,7 +94,15 @@ void U61_Music::start()
 
   if (session==NULL)
     {
-      change(true);
+      if (last_chosen<0)
+	{
+	  change(true);
+	}
+      else
+	{
+	  session=new CL_SoundBuffer_Session
+	    (U61_Global::data.music[last_chosen]->prepare());
+	}
     }
 
   if (session!=NULL)
