@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -42,14 +42,18 @@
 #include "number.h"
 #include "editaddress.h"
 #include "editnumber.h"
+#include "edit.h"
+#include "text.h"
 #include "toggle.h"
 #include "config.h"
+#include "data.h"
+#include "editport.h"
+#include "const.h"
 
 /*---------------------------------------------------------------------------
  globals
  ---------------------------------------------------------------------------*/
 
-
 /*---------------------------------------------------------------------------
  functions
  ---------------------------------------------------------------------------*/
@@ -64,13 +68,19 @@ U61_NetworkOptionMenu::U61_NetworkOptionMenu(U61_Menu *parent,
 {
   config=conf;
   add_item(new U61_EditAddress("Server",conf->server_id));
-  add_item(new U61_Toggle("Anticipation mode is",&(conf->network_anticipation),
-                            "on","off"));
-  add_item(new U61_EditNumber("TCP/IP port",
-			      &(conf->network_port),1000,10000));
-  add_item(new U61_EditNumber("ClanLib channel",
-			      &(conf->network_channel),0,255));
-           
+  add_item(new U61_EditPort("TCP/IP port",
+			    &(conf->network_port)));
+  add_item(new U61_Edit("Comment",
+			conf->server_comment,
+			U61_CONST_METASERVER_COMMENT_SIZE));
+  add_item(new U61_Toggle("Use password :",
+			  &(conf->use_password),
+			  "yes","no"));
+  add_item(new U61_Edit("Password (*)",
+			conf->password,
+			U61_DATA_PASSWORD_SIZE));
+  add_item(new U61_Text("(*) choose a weak password"));
+
   select(0);
 }
 
