@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -42,7 +42,6 @@
 #include <ClanLib/core.h>
 #include <ClanLib/display.h>
 #include <ClanLib/sound.h>
-#include <ClanLib/network.h>
 #include <ClanLib/mikmod.h>
 
 #include <sys/types.h>
@@ -55,9 +54,11 @@
 
 #include "platform.h"
 #include "const.h"
-#include "debug.h"
+#include "log.h"
 #include "script.h"
 #include "sound.h"
+#include "setup.h"
+#include "macro.h"
 
 /*---------------------------------------------------------------------------
  variants
@@ -158,7 +159,7 @@ void U61_Platform::chdir(char *dir)
 /*
  * keeps the directory only in a /dir1/dir2/path.ext string
  */
-void U61_Platform::keep_dir_only(char *path)
+void U61_Platform::keep_dir_only(char path[U61_CONST_STRING_SIZE])
 {
   int len;
   bool found=false;
@@ -180,7 +181,7 @@ void U61_Platform::keep_dir_only(char *path)
 /*
  * keeps the file only in a /dir1/dir2/path.ext string
  */
-void U61_Platform::keep_file_only(char *path)
+void U61_Platform::keep_file_only(char path[U61_CONST_STRING_SIZE])
 {
   int pos;
   bool found=false;
@@ -191,8 +192,8 @@ void U61_Platform::keep_file_only(char *path)
     {
       if (path[pos]=='/')
 	{
-	  strcpy(tmp,path+pos+1);
-	  strcpy(path,tmp);
+	  U61_MACRO_STRCPY(tmp,path+pos+1);
+	  U61_MACRO_STRNCPY(path,tmp,U61_CONST_STRING_SIZE);
 	  found=true;
 	}
       pos--;
@@ -208,21 +209,21 @@ char *U61_Platform::get_user_path()
   static char buffer[U61_CONST_STRING_SIZE]; 
   int len;
 
-  strcpy (buffer,getenv("HOME"));
+  U61_MACRO_STRCPY (buffer,getenv("HOME"));
   len=strlen(buffer);
   if (len>0)
     {
       if (buffer[len-1]!='/')
         {
-	  strcat(buffer,"/");
+	  U61_MACRO_STRCAT(buffer,"/");
         } 
     }
   else
     {
-      strcpy(buffer,"./");
+      U61_MACRO_STRCPY(buffer,"./");
     }
 
-  strcat(buffer,".u61/");
+  U61_MACRO_STRCAT(buffer,".u61/");
 
   return buffer;
 }
@@ -237,7 +238,7 @@ char *U61_Platform::get_data_path()
   DIR *d;
   int len,i;
 
-  strcpy (buffer,U61_INSTALL_DATA_PATH);
+  U61_MACRO_STRCPY (buffer,U61_DEF_INSTALL_DATA_PATH);
   d=opendir(buffer);
   if (d==NULL)
     {  
@@ -254,7 +255,7 @@ char *U61_Platform::get_data_path()
 	       * We've found an existing directory named u61, we'd be
 	       * really unlucky if it wasn't the right one...
 	       */
-	      strcpy(buffer,standard_paths[i]);
+	      U61_MACRO_STRCPY(buffer,standard_paths[i]);
 	      i=NB_STANDARD_PATHS;
 	      closedir(d);
 	    }
@@ -270,7 +271,7 @@ char *U61_Platform::get_data_path()
     {
       if (buffer[len-1]!='/')
         {
-	  strcat(buffer,"/");
+	  U61_MACRO_STRCAT(buffer,"/");
         } 
     }
 
@@ -285,8 +286,8 @@ char *U61_Platform::get_builtin_theme_path()
 {
   static char buffer[U61_CONST_STRING_SIZE];
 
-  strcpy(buffer,get_data_path());
-  strcat(buffer,"theme/");
+  U61_MACRO_STRCPY(buffer,get_data_path());
+  U61_MACRO_STRCAT(buffer,"theme/");
 
   return buffer;
 }
@@ -299,8 +300,8 @@ char *U61_Platform::get_builtin_script_path()
 {
   static char buffer[U61_CONST_STRING_SIZE];
 
-  strcpy(buffer,get_data_path());
-  strcat(buffer,"script/");
+  U61_MACRO_STRCPY(buffer,get_data_path());
+  U61_MACRO_STRCAT(buffer,"script/");
 
   return buffer;
 }
@@ -313,8 +314,8 @@ char *U61_Platform::get_user_theme_path()
 {
   static char buffer[U61_CONST_STRING_SIZE];
 
-  strcpy(buffer,get_user_path());
-  strcat(buffer,"theme/");
+  U61_MACRO_STRCPY(buffer,get_user_path());
+  U61_MACRO_STRCAT(buffer,"theme/");
 
   return buffer;
 }
@@ -327,8 +328,8 @@ char *U61_Platform::get_user_script_path()
 {
   static char buffer[U61_CONST_STRING_SIZE];
 
-  strcpy(buffer,get_user_path());
-  strcat(buffer,"script/");
+  U61_MACRO_STRCPY(buffer,get_user_path());
+  U61_MACRO_STRCAT(buffer,"script/");
 
   return buffer;
 }
@@ -341,8 +342,8 @@ char *U61_Platform::get_config_file()
 {
   static char buffer[U61_CONST_STRING_SIZE]; 
 
-  strcpy (buffer,get_user_path());
-  strcat(buffer,"config.lua");
+  U61_MACRO_STRCPY(buffer,get_user_path());
+  U61_MACRO_STRCAT(buffer,"config.lua");
  
   return buffer;
 }
@@ -351,7 +352,7 @@ char *U61_Platform::get_config_file()
 /*
  * transforms "/usr/local/share/u61/script/classic.lua" into "classic"
  */
-void U61_Platform::strip_remote_script_path(char *path)
+void U61_Platform::strip_remote_script_path(char path[U61_CONST_STRING_SIZE])
 {
   char buffer[U61_CONST_STRING_SIZE];
   char *pos;
@@ -362,8 +363,8 @@ void U61_Platform::strip_remote_script_path(char *path)
   pos=strrchr(path,'/');
   if (pos!=NULL)
     {
-      strcpy(buffer,pos+1);
-      strcpy(path,buffer);
+      U61_MACRO_STRCPY(buffer,pos+1);
+      U61_MACRO_STRNCPY(path,buffer,U61_CONST_STRING_SIZE);
     }
 
   /*
@@ -380,18 +381,18 @@ void U61_Platform::strip_remote_script_path(char *path)
 /*
  * transforms "classic" into "/home/ufoot/.u61/script/classic.lua"
  */
-void U61_Platform::unstrip_remote_script_path(char *path)
+void U61_Platform::unstrip_remote_script_path(char path[U61_CONST_STRING_SIZE])
 {
   char buffer[U61_CONST_STRING_SIZE];
 
   // Gets the user script path "/home/ufoot/.u61/script/"
-  strncpy(buffer,get_user_script_path(),sizeof(buffer)-1);
+  U61_MACRO_STRCPY(buffer,get_user_script_path());
   // Add the filename "classic"
-  strcat(buffer,path);
+  U61_MACRO_STRCAT(buffer,path);
   // Add the extension ".lua"
-  strcat(buffer,".lua");
+  U61_MACRO_STRCAT(buffer,".lua");
 
-  strncpy(path,buffer,sizeof(buffer)-1);
+  U61_MACRO_STRNCPY(path,buffer,U61_CONST_STRING_SIZE);
 }
 
 /*--------------------------------------------------------------------------*/
@@ -506,19 +507,9 @@ void U61_Platform::exec(char *file)
   char buffer[U61_CONST_STRING_SIZE];
   int len;
 
-  U61_LOG_MESSAGE("Restarting...");
+  U61_LOG_MESSAGE("Restarting program...");
 
-  U61_Sound::wait_no_playback();
-
-  U61_Platform::deinit();
-  U61_Script::close();
-  CL_SetupMikMod::deinit();
-  CL_SetupNetwork::deinit();    
-  CL_SetupSound::deinit();
-  CL_SetupDisplay::deinit();
-
-  U61_Debug::deinit();
-  CL_SetupCore::deinit();
+  U61_Setup::deinit();
 
   /*
    * First we try to launch the executable described
@@ -534,16 +525,16 @@ void U61_Platform::exec(char *file)
    * this should replace u61 by /usr/local/bin/u61
    * for instance.
    */
-  strcpy(buffer,U61_INSTALL_BIN_PATH);
+  U61_MACRO_STRCPY(buffer,U61_DEF_INSTALL_BIN_PATH);
   len=strlen(buffer);
   if (len>0)
     {
       if (buffer[len-1]!='/')
         {
-	  strcat(buffer,"/");
+	  U61_MACRO_STRCAT(buffer,"/");
         } 
     }
-  strcat(buffer,file);
+  U61_MACRO_STRCAT(buffer,file);
   execl(buffer,buffer,NULL);
 
   /*
@@ -551,9 +542,9 @@ void U61_Platform::exec(char *file)
    * Since we have uninitialized ClanLib, we *must*
    * quit, or there's a core dump arround the corner...
    */
-  U61_Debug::init();
+  U61_Log::init();
   U61_LOG_ERROR("Unable to execute "<<file<<" or "<<buffer);
-  U61_Debug::deinit();
+  U61_Log::deinit();
 
   exit(1);
 }
@@ -570,8 +561,8 @@ void U61_Platform::open_score_file()
   
   if (score_file==NULL)
     {
-      strcpy (buffer,get_user_path());
-      strcat(buffer,"score.txt");
+      U61_MACRO_STRCPY (buffer,get_user_path());
+      U61_MACRO_STRCAT(buffer,"score.txt");
 
       score_file=fopen(buffer,"a");
     }
@@ -613,7 +604,7 @@ void U61_Platform::dump_score(char *text)
        */
       loctime = localtime (&curtime);
      
-      sprintf(buffer_date,asctime(loctime));
+      U61_MACRO_SPRINTF0(buffer_date,asctime(loctime));
       /*
        * We get rid of the trailing '\n'
        */
@@ -636,13 +627,13 @@ void U61_Platform::open_log_files()
   
   create_user_dir();  
 
-#ifdef U61_DEBUG
+#ifdef U61_DEF_DEBUG
   char buffer[U61_CONST_STRING_SIZE];
 
   if (debug_file==NULL)
     {
-      strcpy (buffer,get_user_path());
-      strcat(buffer,"debug.log");
+      U61_MACRO_STRCPY (buffer,get_user_path());
+      U61_MACRO_STRCAT(buffer,"debug.log");
 
       debug_file=fopen(buffer,"w");
     }
@@ -656,7 +647,7 @@ void U61_Platform::open_log_files()
  */
 void U61_Platform::close_log_files()
 {
-#ifdef U61_DEBUG
+#ifdef U61_DEF_DEBUG
   if (debug_file!=NULL)
     {
       fclose(debug_file);
@@ -674,7 +665,7 @@ void U61_Platform::log_to_error_file(char *text)
 {
   syslog(LOG_ERR,"error: %s",text);
 
-#ifdef U61_DEBUG
+#ifdef U61_DEF_DEBUG
   if (debug_file!=NULL)
     {
       fprintf(debug_file,"*;error  ;%25s;%-5d;%s\n","unknown",0,text);
@@ -690,7 +681,7 @@ void U61_Platform::log_to_warning_file(char *text)
 {
   syslog(LOG_WARNING,"warning: %s",text);
 
-#ifdef U61_DEBUG
+#ifdef U61_DEF_DEBUG
   if (debug_file!=NULL)
     {
       fprintf(debug_file,"*;warning;%25s;%-5d;%s\n","unknown",0,text);
@@ -704,7 +695,7 @@ void U61_Platform::log_to_warning_file(char *text)
  */
 void U61_Platform::log_to_message_file(char *text)
 {
-#ifdef U61_DEBUG
+#ifdef U61_DEF_DEBUG
   if (debug_file!=NULL)
     {
       fprintf(debug_file," ;message;%25s;%-5d;%s\n","unknown",0,text);
@@ -718,7 +709,7 @@ void U61_Platform::log_to_message_file(char *text)
  */
 void U61_Platform::log_to_debug_file(char *text,char *file,int line)
 {
-#ifdef U61_DEBUG
+#ifdef U61_DEF_DEBUG
   if (debug_file!=NULL)
     {
       fprintf(debug_file," ;debug  ;%25s;%-5d;%s\n",file,line,text);
