@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -39,23 +39,31 @@
  includes
  ---------------------------------------------------------------------------*/
 
-#ifdef U61_MSVC
+#ifdef U61_DEF_MSVC
 
 #include <stdlib.h>
 #include <time.h>
 #include <process.h>
 #include <direct.h>
+#include <winsock.h>
 #include <ClanLib/core.h>
-#include <ClanLib/network.h>
 #include <ClanLib/sound.h>
 #include <ClanLib/display.h>
 #include <ClanLib/mikmod.h>
 
 #include "platform.h"
 #include "const.h"
-#include "debug.h"
+#include "log.h"
 #include "script.h"
 #include "sound.h"
+#include "setup.h"
+#include "macro.h"
+
+/*---------------------------------------------------------------------------
+ constants
+ ---------------------------------------------------------------------------*/
+
+#define U61_PLATFORMWINDOWS_DESIRED_WINSOCK_VERSION 0x0101
 
 /*---------------------------------------------------------------------------
  variants
@@ -160,7 +168,7 @@ void *_al_findfirst(char *name, int attrib)
 	}
 
 	//   do_uconvert(info->info.name, U_ASCII, nameret, U_CURRENT, -1);
-	strcpy(nameret,info->info.name);
+	U61_MACRO_STRCPY(nameret,info->info.name);
 
 	if (aret)
 	*aret = info->info.attrib;
@@ -195,7 +203,7 @@ int _al_findnext(void *dta, char *nameret, int *aret)
   }
 
   //   do_uconvert(info->info.name, U_ASCII, nameret, U_CURRENT, -1);
-  strcpy(nameret,info->info.name);
+  U61_MACRO_STRNCPY(nameret,info->info.name,U61_CONST_STRING_SIZE);
 
   if (aret)
     *aret = info->info.attrib;
@@ -232,7 +240,15 @@ void _al_findclose(void *dta)
  */
 void U61_Platform::init()
 {
+  int winerr;
+  WSADATA wsadata;
 
+  winerr = WSAStartup (U61_PLATFORMWINDOWS_DESIRED_WINSOCK_VERSION, &wsadata);
+
+  if (winerr != 0)
+    {
+      U61_LOG_WARNING("Unable to initialize Winsock API, network will *NOT* work.");
+    }
 }
 
 /*--------------------------------------------------------------------------*/
@@ -241,7 +257,7 @@ void U61_Platform::init()
  */
 void U61_Platform::deinit()
 {
-  
+  WSACleanup();
 }
 
 /*--------------------------------------------------------------------------*/
@@ -276,7 +292,7 @@ void U61_Platform::chdir(char *dir)
 /*
  * keeps the directory only in a /dir1/dir2/path.ext string
  */
-void U61_Platform::keep_dir_only(char *path)
+void U61_Platform::keep_dir_only(char path[U61_CONST_STRING_SIZE])
 {
   int len;
   bool found=false;
@@ -298,7 +314,7 @@ void U61_Platform::keep_dir_only(char *path)
 /*
  * keeps the file only in a /dir1/dir2/path.ext string
  */
-void U61_Platform::keep_file_only(char *path)
+void U61_Platform::keep_file_only(char path[U61_CONST_STRING_SIZE])
 {
   int pos;
   bool found=false;
@@ -309,8 +325,8 @@ void U61_Platform::keep_file_only(char *path)
     {
       if (path[pos]=='\\' || path[pos]=='/')
 	{
-	  strcpy(tmp,path+pos+1);
-	  strcpy(path,tmp);
+	  U61_MACRO_STRCPY(tmp,path+pos+1);
+	  U61_MACRO_STRNCPY(path,tmp,U61_CONST_STRING_SIZE);
 	  found=true;
 	}
       pos--;
@@ -325,7 +341,7 @@ char *U61_Platform::get_user_path()
 {
   static char buffer[U61_CONST_STRING_SIZE]; 
 
-  strcpy(buffer,"data\\user\\");
+  U61_MACRO_STRCPY(buffer,"data\\user\\");
 
   return buffer;
 }
@@ -338,7 +354,7 @@ char *U61_Platform::get_data_path()
 {
   static char buffer[U61_CONST_STRING_SIZE]; 
 
-  strcpy(buffer,"data\\builtin\\");	
+  U61_MACRO_STRCPY(buffer,"data\\builtin\\");	
 
   return buffer;
 }
@@ -351,8 +367,8 @@ char *U61_Platform::get_builtin_theme_path()
 {
   static char buffer[U61_CONST_STRING_SIZE];
 
-  strcpy(buffer,get_data_path());
-  strcat(buffer,"theme\\");
+  U61_MACRO_STRCPY(buffer,get_data_path());
+  U61_MACRO_STRCAT(buffer,"theme\\");
 
   return buffer;
 }
@@ -365,8 +381,8 @@ char *U61_Platform::get_builtin_script_path()
 {
   static char buffer[U61_CONST_STRING_SIZE];
 
-  strcpy(buffer,get_data_path());
-  strcat(buffer,"script\\");
+  U61_MACRO_STRCPY(buffer,get_data_path());
+  U61_MACRO_STRCAT(buffer,"script\\");
 
   return buffer;
 }
@@ -379,8 +395,8 @@ char *U61_Platform::get_user_theme_path()
 {
   static char buffer[U61_CONST_STRING_SIZE];
 
-  strcpy(buffer,get_user_path());
-  strcat(buffer,"theme\\");
+  U61_MACRO_STRCPY(buffer,get_user_path());
+  U61_MACRO_STRCAT(buffer,"theme\\");
 
   return buffer;
 }
@@ -393,8 +409,8 @@ char *U61_Platform::get_user_script_path()
 {
   static char buffer[U61_CONST_STRING_SIZE];
 
-  strcpy(buffer,get_user_path());
-  strcat(buffer,"script\\");
+  U61_MACRO_STRCPY(buffer,get_user_path());
+  U61_MACRO_STRCAT(buffer,"script\\");
 
   return buffer;
 }
@@ -407,8 +423,8 @@ char *U61_Platform::get_config_file()
 {
   static char buffer[U61_CONST_STRING_SIZE]; 
 
-  strcpy (buffer,get_user_path());
-  strcat(buffer,"config.txt");
+  U61_MACRO_STRCPY (buffer,get_user_path());
+  U61_MACRO_STRCAT(buffer,"config.txt");
  
   return buffer;
 }
@@ -417,7 +433,7 @@ char *U61_Platform::get_config_file()
 /*
  * transforms "C:\\GAMES\\U61\\builtin-theme\\classic.lua" into "classic"
  */
-void U61_Platform::strip_remote_script_path(char *path)
+void U61_Platform::strip_remote_script_path(char path[U61_CONST_STRING_SIZE])
 {
   char buffer[U61_CONST_STRING_SIZE];
   char *pos;
@@ -428,8 +444,8 @@ void U61_Platform::strip_remote_script_path(char *path)
   pos=strrchr(path,'\\');
   if (pos!=NULL)
     {
-      strcpy(buffer,pos+1);
-      strcpy(path,buffer);
+      U61_MACRO_STRCPY(buffer,pos+1);
+      U61_MACRO_STRNCPY(path,buffer,U61_CONST_STRING_SIZE);
     }
 
   /*
@@ -446,15 +462,15 @@ void U61_Platform::strip_remote_script_path(char *path)
 /*
  * transforms  "classic" into "C:\\GAMES\\U61\\builtin-theme\\classic.lua"
  */
-void U61_Platform::unstrip_remote_script_path(char *path)
+void U61_Platform::unstrip_remote_script_path(char path[U61_CONST_STRING_SIZE])
 {
   char buffer[U61_CONST_STRING_SIZE];
 
-  strncpy(buffer,get_user_script_path(),sizeof(buffer)-1);
-  strcat(buffer,path);
-  strcat(buffer,".lua");
+  U61_MACRO_STRCPY(buffer,get_user_script_path());
+  U61_MACRO_STRCAT(buffer,path);
+  U61_MACRO_STRCAT(buffer,".lua");
 
-  strncpy(path,buffer,sizeof(buffer)-1);
+  U61_MACRO_STRNCPY(path,buffer,U61_CONST_STRING_SIZE);
 }
 
 /*--------------------------------------------------------------------------*/
@@ -466,7 +482,7 @@ bool U61_Platform::open_dir(char *dir, char *ext)
   bool ok=false;
   char buffer[U61_CONST_STRING_SIZE];
 
-  sprintf(buffer,"%s*%s",dir,ext);
+  U61_MACRO_SPRINTF2(buffer,"%s*%s",dir,ext);
   find_info=_al_findfirst(buffer,0);
   if (find_info!=NULL)
     {
@@ -542,19 +558,9 @@ void U61_Platform::exec(char *file)
   char buffer[U61_CONST_STRING_SIZE];
   //  int len;
 
-  U61_LOG_MESSAGE("Restarting...");
+  U61_LOG_MESSAGE("Restarting program...");
 
-  U61_Sound::wait_no_playback();
-
-  U61_Platform::deinit();
-  U61_Script::close();
-  CL_SetupMikMod::deinit();
-  CL_SetupNetwork::deinit();    
-  CL_SetupSound::deinit();
-  CL_SetupDisplay::deinit();
-
-  U61_Debug::deinit();
-  CL_SetupCore::deinit();
+  U61_Setup::deinit();
 
   /*
    * We try to launch the executable described
@@ -569,9 +575,9 @@ void U61_Platform::exec(char *file)
    * Since we have uninitialized ClanLib, we *must*
    * quit, or there's a core dump arround the corner...
    */
-  U61_Debug::init();
+  U61_Log::init();
   U61_LOG_ERROR("Unable to execute "<<file<<" or "<<buffer);
-  U61_Debug::deinit();
+  U61_Log::deinit();
 
   exit(1);
 }
@@ -588,8 +594,8 @@ void U61_Platform::open_score_file()
 
   if (score_file==NULL)
     {
-      strcpy (buffer,get_user_path());
-      strcat(buffer,"score.txt");
+      U61_MACRO_STRCPY (buffer,get_user_path());
+      U61_MACRO_STRCAT(buffer,"score.txt");
 
       score_file=fopen(buffer,"a");
     }
@@ -631,7 +637,7 @@ void U61_Platform::dump_score(char *text)
        */
       loctime = localtime (&curtime);
      
-      sprintf(buffer_date,asctime(loctime));
+      U61_MACRO_SPRINTF0(buffer_date,asctime(loctime));
       /*
        * We get rid of the trailing '\n'
        */
@@ -656,17 +662,17 @@ void U61_Platform::open_log_files()
 
   if (trace_file==NULL)
     {
-      strcpy (buffer,get_user_path());
-      strcat(buffer,"trace.txt");
+      U61_MACRO_STRCPY (buffer,get_user_path());
+      U61_MACRO_STRCAT(buffer,"trace.txt");
 
       trace_file=fopen(buffer,"w");
     }
 
-#ifdef U61_DEBUG
+#ifdef U61_DEF_DEBUG
   if (debug_file==NULL)
     {
-      strcpy (buffer,get_user_path());
-      strcat(buffer,"debug.txt");
+      U61_MACRO_STRCPY (buffer,get_user_path());
+      U61_MACRO_STRCAT(buffer,"debug.txt");
 
       debug_file=fopen(buffer,"w");
     }
@@ -687,7 +693,7 @@ void U61_Platform::close_log_files()
   trace_file=NULL;
 
 
-#ifdef U61_DEBUG
+#ifdef U61_DEF_DEBUG
   if (debug_file!=NULL)
     {
       fclose(debug_file);
@@ -707,7 +713,7 @@ void U61_Platform::log_to_error_file(char *text)
     {
       fprintf(trace_file,"ERROR! %s\n",text);
     }
-#ifdef U61_DEBUG
+#ifdef U61_DEF_DEBUG
   if (debug_file!=NULL)
     {
       fprintf(debug_file,"*;error  ;%25s;%-5d;%s\n","unknown",0,text);
@@ -725,7 +731,7 @@ void U61_Platform::log_to_warning_file(char *text)
     {
       fprintf(trace_file,"WARNING! %s\n",text);
     }
-#ifdef U61_DEBUG
+#ifdef U61_DEF_DEBUG
   if (debug_file!=NULL)
     {
       fprintf(debug_file,"*;warning;%25s;%-5d;%s\n","unknown",0,text);
@@ -743,7 +749,7 @@ void U61_Platform::log_to_message_file(char *text)
     {
       fprintf(trace_file,"%s\n",text);
     }
-#ifdef U61_DEBUG
+#ifdef U61_DEF_DEBUG
   if (debug_file!=NULL)
     {
       fprintf(debug_file," ;message;%25s;%-5d;%s\n","unknown",0,text);
@@ -757,7 +763,7 @@ void U61_Platform::log_to_message_file(char *text)
  */
 void U61_Platform::log_to_debug_file(char *text,char *file,int line)
 {
-#ifdef U61_DEBUG
+#ifdef U61_DEF_DEBUG
   if (debug_file!=NULL)
     {
       fprintf(debug_file," ;debug  ;%25s;%-5d;%s\n",file,line,text);
@@ -765,4 +771,4 @@ void U61_Platform::log_to_debug_file(char *text,char *file,int line)
 #endif
 }
 
-#endif //U61_MSVC
+#endif //U61_DEF_MSVC
