@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -54,6 +54,7 @@
  ---------------------------------------------------------------------------*/
 
 #include <string.h>
+#include <zlib.h>
 
 #include "player.h"
 #include "time.h"
@@ -62,8 +63,18 @@
 #include "networksource.h"
 #include "networkcopy.h"
 #include "global.h"
-#include "debug.h"
+#include "log.h"
 #include "sound.h"
+#include "macro.h"
+
+/*---------------------------------------------------------------------------
+ constants
+ ---------------------------------------------------------------------------*/
+
+/*
+ * Delay after which one considers that a checksum is "missing".
+ */
+#define U61_PLAYER_MAX_DELAY_BETWEEN_CHECKSUM (U61_MAP_DELAY_BETWEEN_CHECKSUM*3+1000)
 
 /*---------------------------------------------------------------------------
  globals
@@ -80,10 +91,24 @@
  */ 
 U61_Player::U61_Player()
 {
+  /*
+   * Note: we don't use reset for it's not possible to
+   * call map::reset() until all the map backgrounds have
+   * been loaded, and the U61_Player objects are created
+   * as global variables, long before the backgrounds are loaded...
+   */
+  id=0;
+  available=false;
+  local=true;
+  history_done=true;
+
   source=NULL;
   copy=NULL;
 
-  reset();
+  anticipation=NULL;
+  key_start_stop=NULL;
+
+  ready_state=U61_PLAYER_STATE_NOT_READY;
 }
 
 /*--------------------------------------------------------------------------*/
@@ -101,7 +126,9 @@ U61_Player::~U61_Player()
  */
 void U61_Player::reset()
 {
+  id=0;
   available=false;
+  local=true;
   history_done=true;
 
   if (source!=NULL)
@@ -114,6 +141,22 @@ void U61_Player::reset()
       delete copy;
       copy=NULL;
     }
+
+  last.reset();
+  sure.reset();
+  history.reset();
+  history_time=U61_TIME_START;
+  U61_MACRO_MEMSET0(history_zlib_buffer);
+  history_zlib_buffer_size=0;
+  history_zlib_buffer_cpt=0;
+
+  last_event_time=U61_TIME_START;
+  last_checksum_time=U61_TIME_START;
+
+  anticipation=NULL;
+  key_start_stop=NULL;
+
+  ready_state=U61_PLAYER_STATE_NOT_READY;
 }
 
 /*--------------------------------------------------------------------------*/
@@ -226,11 +269,13 @@ int U61_Player::get_background()
  * initialized with this function
  */
 void U61_Player::init_local(U61_Input *input,
+			    U61_Input *chat_input,
                             U61_Dispatcher *d,
 			    int id,
+			    bool prep_hist,
 			    int time,
 			    U61_PlayerConfig *conf,
-			    bool start)
+			    bool ready)
 {
   U61_LocalSource *src; 
   U61_LocalCopy *cpy;
@@ -240,6 +285,8 @@ void U61_Player::init_local(U61_Input *input,
   local=true;
 
   set_id(id);
+  prepare_history=prep_hist;
+
   sure.init(id,
 	    U61_Global::game.initial_speed,
 	    U61_Global::game.acceleration,
@@ -247,11 +294,15 @@ void U61_Player::init_local(U61_Input *input,
   copy_sure_to_last();
 
   anticipation=&(conf->anticipation);
+  key_start_stop=&(conf->key_start_stop);
 
-  src=new U61_LocalSource(input,&sure,d,id,conf->name);
-  if (start)
+  src=new U61_LocalSource(input,chat_input,&sure,d,id,conf->name);
+
+  src->send_nothing(time);
+  src->send_name(time);
+  if (ready)
     {
-      src->send_start_stop(time);
+      src->send_ready(time);
     }
 
   cpy=new U61_LocalCopy(id,d);
@@ -267,7 +318,8 @@ void U61_Player::init_local(U61_Input *input,
  * sets the player as a network player
  */
 void U61_Player::init_network(U61_Dispatcher *d,
-                              int id)
+                              int id,
+			      bool prep_hist)
 {
   U61_NetworkSource *src; 
   U61_NetworkCopy *cpy;
@@ -277,6 +329,8 @@ void U61_Player::init_network(U61_Dispatcher *d,
   local=false;;
 
   set_id(id);
+  prepare_history=prep_hist;
+
   sure.init(id,
 	    U61_Global::game.initial_speed,
 	    U61_Global::game.acceleration,
@@ -285,6 +339,7 @@ void U61_Player::init_network(U61_Dispatcher *d,
   copy_sure_to_last();
 
   anticipation=&(U61_Global::config.network_anticipation);
+  key_start_stop=NULL;
 
   src=new U61_NetworkSource(d,&sure,id);
   cpy=new U61_NetworkCopy(id,d);
@@ -336,6 +391,7 @@ void U61_Player::pump_events()
   bool changed=false;
   U61_Event evt;
   bool kill_needed=false;
+  bool history_copied=false;
 
   source->poll();
 
@@ -344,6 +400,24 @@ void U61_Player::pump_events()
     {
       changed=true;
       evt=source->get();
+
+      /*
+       * In case we have an event which is older
+       * than the latest processed event, we copy
+       * the "sure" map to the "history" map which will
+       * be used when sending info to network clients
+       */
+      if (prepare_history && last_event_time<evt.time && !history_copied)
+	{
+	  copy_sure_to_history();
+	  /*
+	   * We flag history_copied as true so that we do not copy
+	   * sure to history for each event...
+	   */
+	  history_copied=true;
+	}
+      last_event_time=evt.time;
+
       switch (evt.code)
 	{
 	case U61_EVENT_KILL:
@@ -358,17 +432,66 @@ void U61_Player::pump_events()
         case U61_EVENT_HISTORY_END:
 	  history_done=true;
 	  break;
+	case U61_EVENT_READY:
+	  if (ready_state!=U61_PLAYER_STATE_READY_PLAYING)
+	    {
+	      ready_state=U61_PLAYER_STATE_READY_WAITING;
+	    }
+	  break;
+	case U61_EVENT_READY_ALL:
+	  if (ready_state!=U61_PLAYER_STATE_READY_PLAYING)
+	    {
+	      U61_Global::game.notify_session_start();
+
+	      ready_state=U61_PLAYER_STATE_READY_PLAYING;
+	      if (is_local())
+		{
+		  ((U61_LocalSource *) source)->send_start_stop(evt.time);
+		  ((U61_LocalSource *) source)->send_name(evt.time);
+		}
+	    }
+	  break;
+	case U61_EVENT_CHAT_LETTER:
+	  U61_Global::layout.put_chat_letter(evt);
+	  break;
+	case U61_EVENT_VERIFY_CHECKSUM:
+	  last_checksum_time=evt.time;
+	  break;
 	}
+
       /*
-       * It's important to put this code after the "switch"
+       * It's important to put this code after the previous "switch"
        * or else scores are not correctly handled
        */
-      handle_event(&evt);
-      copy->put(evt);
+      switch (evt.code)
+	{
+	case U61_EVENT_SERIAL_BEGIN:
+	  if (!serial_begin(evt))
+	    {
+	      kill_needed=true;
+	    }
+	  break;
+	case U61_EVENT_SERIAL_DATA:
+	  if (!serial_data(evt))
+	    {
+	      kill_needed=true;
+	    }
+	  break;
+	case U61_EVENT_SERIAL_END:
+	  if (!serial_end(evt))
+	    {
+	      kill_needed=true;
+	    }
+	  break;
+	default:
+	  handle_event(&evt);
+	  copy->put(evt);
+	}
     }
 
   while (sure.exists_request_event())
     {
+      changed=true;
       evt=sure.get_request_event();
       copy->put(evt);
     }
@@ -380,6 +503,8 @@ void U61_Player::pump_events()
 
   copy->flush();
 
+  check_missing_checksum();
+
   if (kill_needed)
     {
       kill();
@@ -392,23 +517,20 @@ void U61_Player::pump_events()
  */
 void U61_Player::handle_event(U61_Event *evt)
 {
-  if (sure.is_active())
+  if (evt->code==U61_EVENT_START_STOP)
     {
-      if (evt->code==U61_EVENT_START_STOP)
+      if (sure.is_active())
 	{
 	  stop();
 	}
       else
 	{
-	  sure.handle_event(evt);
+	  start(evt->time);
 	}
     }
   else
     {
-      if (evt->code==U61_EVENT_START_STOP)
-	{
-	  start(evt->time);
-	} 
+      sure.handle_event(evt);
     }
 }
 
@@ -456,10 +578,53 @@ void U61_Player::kill()
  */
 void U61_Player::draw(int x,int y,int size)
 {
+  U61_TextDisplayer message(U61_Global::data.font_info[size]);
+  bool draw_message = false;
+  char buffer[U61_CONST_STRING_SIZE];
+
   if (history_done)
     {
       last.anticipate(U61_Time::for_event());
-      last.draw(x,y,size,*anticipation);
+      last.draw(x,y,size,(anticipation!=NULL) && *anticipation);
+    }
+
+  switch (ready_state)
+    {
+    case U61_PLAYER_STATE_NOT_READY:
+      if (local && key_start_stop)
+	{
+	  U61_MACRO_SPRINTF1(buffer,
+			     "Press %s when ready",
+			     U61_Global::input_monitor.to_label(*key_start_stop));
+	}
+      else
+	{
+	  U61_MACRO_STRCPY(buffer,
+			   "Not ready...");
+	}
+      message.set_text(buffer);	
+      draw_message = true;
+      break;
+    case U61_PLAYER_STATE_READY_WAITING:
+      message.set_text("Ready!");
+      draw_message = true;
+      break;
+    case U61_PLAYER_STATE_READY_PLAYING:
+      break;
+    default:
+      break;
+    }
+
+  if (draw_message)
+    {
+      int message_y,message_w;
+
+      message_y = y+(last.get_draw_height(size)-message.get_height())/2;
+      message_w = last.get_draw_width(size);
+
+      message.draw(x,message_y,
+		   U61_TEXTDISPLAYER_ALIGN_CENTER,
+		   message_w);			  
     }
 }
 
@@ -473,6 +638,17 @@ void U61_Player::copy_sure_to_last()
   last.mute();
 }
 
+/*--------------------------------------------------------------------------*/
+/*
+ * copies the sure map to the history map
+ */
+void U61_Player::copy_sure_to_history()
+{
+  history_time=last_event_time;
+
+  history=sure;
+}
+
 /*--------------------------------------------------------------------------*/
 /*
  * copies the sure map to the last map
@@ -489,3 +665,152 @@ void U61_Player::register_score()
 				      sure.get_curse_received());
     }
 }
+
+/*--------------------------------------------------------------------------*/
+/*
+ * Returns the "history" map, which is usefull for serializing the map
+ * into events within the U61_History object.
+ */
+U61_Map *U61_Player::get_history()
+{
+  return &history;
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * Returns the time when the "history" map has been built.
+ */
+unsigned int U61_Player::get_history_time()
+{
+  return history_time;
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * Returns the time when the "history" map has been built.
+ */
+bool U61_Player::serial_begin(U61_Event evt)
+{
+  bool ok=false;
+
+  if (evt.par>U61_MAP_COMPRESSED_SIZE)
+    {
+      U61_LOG_ERROR("Network header reports that compressed map is too big ("<<evt.par<<" vs "<<U61_MAP_COMPRESSED_SIZE<<")!");
+    }
+  else
+    {
+      U61_MACRO_MEMSET0(history_zlib_buffer);
+      history_zlib_buffer_size=evt.par;
+      history_zlib_buffer_cpt=0;
+
+      ok=true;
+    }
+
+  return ok;
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * Returns the time when the "history" map has been built.
+ */
+bool U61_Player::serial_data(U61_Event evt)
+{
+  bool ok=false;
+  unsigned char *buffer;
+
+  if (history_zlib_buffer_cpt>=((history_zlib_buffer_size-1)/(int) sizeof(short))+1)
+    {
+      U61_LOG_ERROR("Too many network packets for compressed map ("<<(history_zlib_buffer_cpt+1)<<" vs "<<(((history_zlib_buffer_size-1)/sizeof(short))+1)<<")!");
+    }
+  else
+    {
+      buffer=history_zlib_buffer+history_zlib_buffer_cpt*sizeof(short);
+      /*
+       * We use serialization functions for the conversion char[2]->short
+       * depends on machine endianess.
+       */
+      U61_Serial::serialize_short(evt.par,&buffer,buffer+sizeof(short));     
+
+      history_zlib_buffer_cpt++;
+
+      ok=true;
+    }
+
+  return ok;
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * Returns the time when the "history" map has been built.
+ */
+bool U61_Player::serial_end(U61_Event evt)
+{
+  bool ok=false;
+  unsigned char *buf;
+  unsigned char *buf_tmp;
+  unsigned long buf_size;
+  int zerr;
+
+  if (history_zlib_buffer_cpt!=((history_zlib_buffer_size-1)/(int) sizeof(short))+1)
+    {
+      U61_LOG_ERROR("Wrong network packets number for compressed map ("<<history_zlib_buffer_cpt<<" vs "<<(((history_zlib_buffer_size-1)/sizeof(short))+1)<<")!");
+
+    }
+  else
+    {
+      U61_LOG_DEBUG("received "<<history_zlib_buffer_size<<" bytes serialized map, going to uncompress it...");
+
+      buf_size=U61_MAP_SERIALIZED_SIZE;
+      buf=new unsigned char[buf_size];
+      if (buf)
+	{
+	  zerr=uncompress(buf,&buf_size,history_zlib_buffer,history_zlib_buffer_size);
+
+	  if (zerr!=Z_OK)
+	    {
+	      U61_LOG_ERROR("Unable to uncompress serialized map (zlib error "<<zerr<<":\""<<zError(zerr)<<"\")!");
+	    }
+	  else
+	    {
+	      /*
+	       * Now we're going to restore the map for good, since we've
+	       * collected all the network packets.
+	       */ 
+	      buf_tmp=buf;	      
+	      if (!sure.unserialize(&buf_tmp,buf_tmp+U61_MAP_SERIALIZED_SIZE))
+		{
+		  U61_LOG_ERROR("Serialized map is not correct!");
+		}
+	      else
+		{
+		  /*
+		   * Everything's OK, map should have been restored
+		   */
+		  ok=true;
+		}
+	    }
+
+	  delete buf;
+	}
+    }
+
+  return ok;
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * Checks if there's been a checksum sent recently.
+ */
+void U61_Player::check_missing_checksum()
+{
+  if (last_checksum_time<=U61_TIME_START || !is_active())
+    {
+      last_checksum_time=last_event_time;
+    }
+
+  if (last_checksum_time+U61_PLAYER_MAX_DELAY_BETWEEN_CHECKSUM<last_event_time)
+    {
+      last_checksum_time=last_event_time;
+      U61_LOG_WARNING("Missing checksum for player \""<<get_name()<<"\"!")
+    }
+}
