@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -41,34 +41,31 @@
 
 
 /*---------------------------------------------------------------------------
- includes
- ---------------------------------------------------------------------------*/
+  includes
+  ---------------------------------------------------------------------------*/
 
 #include "protocol.h"
 #include "packet.h"
 #include "version.h"
 #include "const.h"
-#include "debug.h"
+#include "log.h"
+#include "macro.h"
 
 /*---------------------------------------------------------------------------
- variants
- ---------------------------------------------------------------------------*/
-
-#define U61_PROTOCOL_READY_MESSAGE  "Ready"
+  variants
+  ---------------------------------------------------------------------------*/
 
 /*---------------------------------------------------------------------------
- functions
- ---------------------------------------------------------------------------*/
+  functions
+  ---------------------------------------------------------------------------*/
 
 /*--------------------------------------------------------------------------*/
 /*
  * creates a protocol
  */
-U61_Protocol::U61_Protocol(int p,int c)
+U61_Protocol::U61_Protocol(int p)
 {
   port=p;
-  channel=c;
-  session=NULL;
 
   reset();
 }
@@ -79,11 +76,7 @@ U61_Protocol::U61_Protocol(int p,int c)
  */
 U61_Protocol::~U61_Protocol()
 {
-  if (session!=NULL)
-    {
-      delete session; 
-      session=NULL;
-    }
+
 }
 
 /*--------------------------------------------------------------------------*/
@@ -163,7 +156,7 @@ void U61_Protocol::reset()
 {
   stage=U61_PROTOCOL_BEGIN;
   step=0;
-  diagnostic="unknown problem ";
+  U61_MACRO_STRCPY(diagnostic,"Unknown problem");
   connecting=false;
   failed=false;
 }
@@ -198,9 +191,16 @@ int U61_Protocol::send_program(U61_Connection *conn)
 
   U61_LOG_DEBUG("Sending program");
 
-  if (conn->send(U61_CONST_PROGRAM,strlen(U61_CONST_PROGRAM)))
+  if (conn!=NULL)
     {
-      result=U61_PROTOCOL_DONE;
+      if (conn->send(U61_CONST_PROGRAM,strlen(U61_CONST_PROGRAM)))
+	{
+	  result=U61_PROTOCOL_DONE;
+	}
+    }
+  else
+    {
+      result=U61_PROTOCOL_FAILED;
     }
 
   return result;
@@ -217,26 +217,32 @@ int U61_Protocol::recv_program(U61_Connection *conn)
 
   U61_LOG_DEBUG("Receiving program");
 
-  if (conn->peek())
+  if (conn!=NULL)
     {
-      if (conn->recv(buffer,sizeof(buffer)-1))
+      if (conn->peek_str())
 	{
-          if (strcmp(U61_CONST_PROGRAM,buffer)==0)
+	  if (conn->recv(buffer,sizeof(buffer)-1))
 	    {
-	      result=U61_PROTOCOL_DONE;
-	      U61_LOG_DEBUG("Received program "<<buffer<<"");
-	    }
-          else
-	    {
-	      result=U61_PROTOCOL_FAILED;
-              diagnostic="Wrong program, expecting \"";
-	      diagnostic+=U61_CONST_PROGRAM;
-	      diagnostic+="\" and received \"";
-	      diagnostic+=buffer;
-	      diagnostic+="\"";
+	      if (strcmp(U61_CONST_PROGRAM,buffer)==0)
+		{
+		  result=U61_PROTOCOL_DONE;
+		  U61_LOG_DEBUG("Received program "<<buffer<<"");
+		}
+	      else
+		{
+		  result=U61_PROTOCOL_FAILED;
+		  U61_MACRO_SPRINTF2(diagnostic,
+				     "Wrong program, expecting \"%s\" and received \"%s\"",
+				     U61_CONST_PROGRAM,
+				     buffer);
+		}
 	    }
 	}
     }
+  else
+    {
+      result=U61_PROTOCOL_FAILED;
+    }
 
   return result;
 }
@@ -255,11 +261,18 @@ int U61_Protocol::send_version(U61_Connection *conn)
 
   U61_LOG_DEBUG("Sending version");
 
-  U61_Version::get_version_text(version);
-
-  if (conn->send(version,strlen(version)))
+  if (conn!=NULL)
     {
-      result=U61_PROTOCOL_DONE;
+      U61_Version::get_version_text(version);
+
+      if (conn->send(version,strlen(version)))
+	{
+	  result=U61_PROTOCOL_DONE;
+	}
+    }
+  else
+    {
+      result=U61_PROTOCOL_FAILED;
     }
 
   return result;
@@ -277,27 +290,33 @@ int U61_Protocol::recv_version(U61_Connection *conn)
 
   U61_LOG_DEBUG("Receiving version");
 
-  U61_Version::get_version_text(version);
-  if (conn->peek())
+  if (conn!=NULL)
     {
-      if (conn->recv(buffer,sizeof(buffer)-1))
+      U61_Version::get_version_text(version);
+      if (conn->peek_str())
 	{
-          if (strcmp(version,buffer)==0)
+	  if (conn->recv(buffer,sizeof(buffer)-1))
 	    {
-	      result=U61_PROTOCOL_DONE;
-	      U61_LOG_DEBUG("Received version "<<buffer<<"");
-	    }
-          else
-	    {
-              diagnostic="Wrong game version, expecting \"";
-	      diagnostic+=version;
-	      diagnostic+="\" and received \"";
-	      diagnostic+=buffer;
-	      diagnostic+="\"";
-	      result=U61_PROTOCOL_FAILED;
+	      if (strcmp(version,buffer)==0)
+		{
+		  result=U61_PROTOCOL_DONE;
+		  U61_LOG_DEBUG("Received version "<<buffer<<"");
+		}
+	      else
+		{
+		  result=U61_PROTOCOL_FAILED;
+		  U61_MACRO_SPRINTF2(diagnostic,
+				     "Wrong version, expecting \"%s\" and received \"%s\"",
+				     version,
+				     buffer);
+		}
 	    }
 	}
     }
+  else
+    {
+      result=U61_PROTOCOL_FAILED;
+    }
 
   return result;
 }
@@ -312,10 +331,17 @@ int U61_Protocol::send_ready(U61_Connection *conn)
 
   U61_LOG_DEBUG("Sending ready message");
 
-  if (conn->send(U61_PROTOCOL_READY_MESSAGE,
-		 strlen(U61_PROTOCOL_READY_MESSAGE)))
+  if (conn!=NULL)
     {
-      result=U61_PROTOCOL_DONE;
+      if (conn->send(U61_PROTOCOL_READY_MESSAGE,
+		     strlen(U61_PROTOCOL_READY_MESSAGE)))
+	{
+	  result=U61_PROTOCOL_DONE;
+	}
+    }
+  else
+    {
+      result=U61_PROTOCOL_FAILED;
     }
 
   return result;
@@ -332,37 +358,45 @@ int U61_Protocol::recv_ready(U61_Connection *conn)
 
   U61_LOG_DEBUG("Receiving ready message");
 
-  if (conn->peek())
+  if (conn!=NULL)
     {
-      if (conn->recv(buffer,sizeof(buffer)-1))
+      if (conn->peek_str())
 	{
-          if (strcmp(U61_PROTOCOL_READY_MESSAGE,buffer)==0)
+	  if (conn->recv(buffer,sizeof(buffer)-1))
 	    {
-	      result=U61_PROTOCOL_DONE;
-	      U61_LOG_DEBUG("Received ID "<<buffer<<"");
-	    }
-          else
-	    {
-	      result=U61_PROTOCOL_FAILED;
-              diagnostic="Wrong ready message, expecting \"";
-	      diagnostic+=U61_PROTOCOL_READY_MESSAGE;
-	      diagnostic+="\" and received \"";
-	      diagnostic+=buffer;
-	      diagnostic+="\"";
+	      if (strcmp(U61_PROTOCOL_READY_MESSAGE,buffer)==0)
+		{
+		  result=U61_PROTOCOL_DONE;
+		  U61_LOG_DEBUG("Received ID "<<buffer<<"");
+		}
+	      else
+		{
+		  result=U61_PROTOCOL_FAILED;
+		  U61_MACRO_SPRINTF2(diagnostic,
+				     "Wrong message, expecting \"%s\" and received \"%s\"",
+				     U61_PROTOCOL_READY_MESSAGE,
+				     buffer);
+		}
 	    }
 	}
     }
+  else
+    {
+      result=U61_PROTOCOL_FAILED;
+    }
 
   return result;
 }
 
 /*--------------------------------------------------------------------------*/
 /*
- * Prints the diagnostic of the last network error 
+ * Returns the last error message
  */
-void U61_Protocol::print_diagnostic()
+char *U61_Protocol::get_diagnostic()
 {
-  U61_LOG_WARNING("Network error : "<<diagnostic.c_str());
+  diagnostic[sizeof(diagnostic)-1]='\0';
+
+  return diagnostic;
 }
 
 /*--------------------------------------------------------------------------*/
