@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -40,17 +40,17 @@
  */
 
 
-#ifndef INC_U61_PROTOCOL
-#define INC_U61_PROTOCOL
+#ifndef U61_INC_PROTOCOL
+#define U61_INC_PROTOCOL
 
 /*---------------------------------------------------------------------------
  includes
  ---------------------------------------------------------------------------*/
 
 #include <ClanLib/core.h>
-#include <ClanLib/network.h>
 
 #include "connection.h"
+#include "const.h"
 
 /*---------------------------------------------------------------------------
  constants
@@ -63,13 +63,16 @@
 #define U61_PROTOCOL_BEGIN 0
 #define U61_PROTOCOL_END -1
 
-#define U61_PROTOCOL_PROGRAM_SENT 20
+#define U61_PROTOCOL_PROGRAM_SENT     20
 #define U61_PROTOCOL_PROGRAM_RECEIVED 21
-#define U61_PROTOCOL_VERSION_SENT 22
+#define U61_PROTOCOL_VERSION_SENT     22
 #define U61_PROTOCOL_VERSION_RECEIVED 23
-#define U61_PROTOCOL_READY_SENT 50
-#define U61_PROTOCOL_READY_RECEIVED 51
-#define U61_PROTOCOL_TIMEOUT 10
+#define U61_PROTOCOL_READY_SENT       60
+#define U61_PROTOCOL_READY_RECEIVED   61
+#define U61_PROTOCOL_TIMEOUT          10
+
+#define U61_PROTOCOL_OK_MESSAGE     "OK"
+#define U61_PROTOCOL_READY_MESSAGE  "Ready"
 
 /*---------------------------------------------------------------------------
  classes
@@ -78,7 +81,7 @@
 class U61_Protocol
 {
  public:
-  U61_Protocol(int p,int c);
+  U61_Protocol(int p);
   virtual ~U61_Protocol();
 
   virtual void check_connection();
@@ -89,7 +92,7 @@ class U61_Protocol
   bool done();
   int send_all();
   int recv_all();
-  void print_diagnostic();
+  char  *get_diagnostic();
   bool is_connecting();
 
  protected:
@@ -97,11 +100,10 @@ class U61_Protocol
   int stage;
   int step;
   int port;
-  int channel;
-  std::string diagnostic;
+  char diagnostic[U61_CONST_STRING_SIZE];
   bool failed;
 
-  CL_NetSession *session;
+  //  CL_NetSession *session;
 
   int send_program(U61_Connection *conn);
   int recv_program(U61_Connection *conn);
