@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -39,12 +39,15 @@
  includes
  ---------------------------------------------------------------------------*/
 
+#include <string.h>
+
 #include "savescriptmenu.h"
 #include "button.h"
 #include "edit.h"
 #include "global.h"
 #include "platform.h"
-#include "debug.h"
+#include "log.h"
+#include "macro.h"
 
 /*---------------------------------------------------------------------------
  globals
@@ -65,7 +68,9 @@ U61_SaveScriptMenu::U61_SaveScriptMenu(U61_Menu *parent)
 
   add_item(new U61_Button("Do not save the script"));
   add_item(new U61_Button("Save the script"));
-  add_item(new U61_Edit("Name",U61_Global::remote_script.name_buffer));
+  add_item(new U61_Edit("Name",
+			U61_Global::remote_script.name_buffer,
+			U61_DATA_SCRIPT_NAME_SIZE));
 
   select(0);
 }
@@ -107,7 +112,7 @@ bool U61_SaveScriptMenu::save()
   
   U61_Global::data.chdir_to_initial_dir();
   
-  strncpy(buffer,U61_Global::remote_script.name_buffer,sizeof(buffer)-1);
+  U61_MACRO_STRCPY(buffer,U61_Global::remote_script.name_buffer);
   U61_Platform::unstrip_remote_script_path(buffer);
 
   U61_LOG_MESSAGE("Saving script to \""<<buffer<<"\".");
@@ -128,3 +133,25 @@ bool U61_SaveScriptMenu::save()
   return ok;
 }
 
+/*--------------------------------------------------------------------------*/
+/*
+ * Function called on a regular basis
+ */
+U61_Menu *U61_SaveScriptMenu::poll()
+{
+  U61_Menu *new_menu;
+
+  new_menu = U61_Menu::poll();
+  if (strlen(U61_Global::remote_script.code_buffer) == 0 ||
+      strlen(U61_Global::remote_script.code_buffer) == 0)   
+    {
+      /*
+       * No script defined, it's no use to try and save it,
+       * so we directly jump to the parent menu.
+       */
+      new_menu=back();
+    }
+
+  return new_menu;
+}
+
