@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -39,6 +39,7 @@
  ---------------------------------------------------------------------------*/
 
 #include "scoreline.h"
+#include "macro.h"
 
 /*---------------------------------------------------------------------------
  constants
@@ -74,7 +75,7 @@ U61_ScoreLine::~U61_ScoreLine()
  */
 void U61_ScoreLine::reset()
 {
-  strcpy(name,"");
+  U61_MACRO_STRCPY(name,"");
   nb_games=0;
   max_score=0;
   total_score=0;
@@ -127,7 +128,7 @@ char *U61_ScoreLine::get_name()
  */
 void U61_ScoreLine::set_name(char *n)
 {
-  strcpy(name,n);
+  U61_MACRO_STRCPY(name,n);
 }
 
 /*--------------------------------------------------------------------------*/
