@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -51,7 +51,7 @@
  * This akward stuff is to handle the fact that Debian renamed lua lua40
  * and also moved the header files
  */
-#ifdef U61_LUA40
+#ifdef U61_DEF_LUA40
 extern "C" {
 #include <lua40/lua.h>
 #include <lua40/lualib.h>
@@ -66,10 +66,11 @@ extern "C" {
 #include "script.h"
 #include "const.h"
 #include "utils.h"
-#include "debug.h"
+#include "log.h"
 #include "global.h"
 #include "version.h"
 #include "script.h"
+#include "macro.h"
 
 /*---------------------------------------------------------------------------
  constants
@@ -77,6 +78,7 @@ extern "C" {
 
 #define U61_SCRIPT_LUA_GET_PROGRAM "user_get_program"
 #define U61_SCRIPT_LUA_GET_VERSION "user_get_version"
+#define U61_SCRIPT_LUA_START "user_start"
 #define U61_SCRIPT_LUA_NEW_SHAPE "user_new_shape"
 #define U61_SCRIPT_LUA_DO_SHAPE "user_do_shape"
 #define U61_SCRIPT_LUA_ROTATE_LEFT "user_rotate_left"
@@ -111,6 +113,8 @@ extern "C" {
 #define U61_SCRIPT_C_SET_ITEM_COLOR "u61_set_item_color"
 #define U61_SCRIPT_C_GET_WIDTH "u61_get_width"
 #define U61_SCRIPT_C_GET_HEIGHT "u61_get_height"
+#define U61_SCRIPT_C_SET_WIDTH "u61_set_width"
+#define U61_SCRIPT_C_SET_HEIGHT "u61_set_height"
 #define U61_SCRIPT_C_GET_SQUARE_COLOR "u61_get_square_color"
 #define U61_SCRIPT_C_SET_SQUARE_COLOR "u61_set_square_color"
 #define U61_SCRIPT_C_GET_TIME "u61_get_time"
@@ -142,6 +146,8 @@ extern "C" {
 #define U61_SCRIPT_C_ADD_ANTIDOTE "u61_add_antidote"
 #define U61_SCRIPT_C_DELETE_ANTIDOTE "u61_delete_antidote"
 #define U61_SCRIPT_C_IS_BLOCK_OK "u61_is_block_ok"
+#define U61_SCRIPT_C_CLEAR_MAP "u61_clear_map"
+#define U61_SCRIPT_C_SHIFT_MAP "u61_shift_map"
 
 #define U61_SCRIPT_RANDOM_MAX 0x7FFFFFFF
 #define U61_SCRIPT_STACK_SIZE 2048
@@ -293,13 +299,12 @@ char *U61_Script::get_global_str(char *name,char *def)
 
   top=lua_gettop(lua_state);
 
-  strcpy(result,def);
+  U61_MACRO_STRCPY(result,def);
 
   lua_getglobal(lua_state,name);
   if (lua_isstring(lua_state,-1))
     {
-      strncpy(result,lua_tostring(lua_state,-1),U61_CONST_STRING_SIZE-1);
-      result[U61_CONST_STRING_SIZE-1]='\0';
+      U61_MACRO_STRCPY(result,lua_tostring(lua_state,-1));
     }    
 
   lua_settop(lua_state,top);
@@ -346,7 +351,9 @@ bool U61_Script::init()
 {
   bool bSuccess=true;
 
+  lua_baselibopen(lua_state);
   lua_mathlibopen(lua_state);
+  lua_strlibopen(lua_state);
   record_all();
 
   return bSuccess;
@@ -553,7 +560,7 @@ bool U61_Script::execute_lua_int_func_int(int *result,
 	}
       break;
     default:
-      U61_LOG_WARNING("Lua error executing \""<<func<<"("<<par1<<")\", code is"<<err<<".");
+      U61_LOG_WARNING("Lua error executing \""<<func<<"("<<par1<<")\", code is "<<err<<".");
       break;
     }
 
@@ -567,7 +574,7 @@ bool U61_Script::execute_lua_int_func_int(int *result,
  * calls a user defined function which takes no argument
  * and returns a string
  */
-bool U61_Script::execute_lua_string_func_void(char *result, 
+bool U61_Script::execute_lua_string_func_void(char result[U61_CONST_STRING_SIZE], 
 					      char *func)
 {
   bool success=false;
@@ -586,8 +593,7 @@ bool U61_Script::execute_lua_string_func_void(char *result,
     case 0:
       if (lua_isstring(lua_state,-1))
 	{
-	  strncpy(result,lua_tostring(lua_state,-1),U61_CONST_STRING_SIZE);
-	  result[U61_CONST_STRING_SIZE-1]='\0';
+	  U61_MACRO_STRNCPY(result,lua_tostring(lua_state,-1),U61_CONST_STRING_SIZE);
 	  success=true;
 	}
       else
@@ -596,7 +602,7 @@ bool U61_Script::execute_lua_string_func_void(char *result,
 	}
       break;
     default:
-      U61_LOG_WARNING("Lua error executing \""<<func<<"()\", code is"<<err<<".");
+      U61_LOG_WARNING("Lua error executing \""<<func<<"()\", code is "<<err<<".");
       break;
     }
 
@@ -610,7 +616,7 @@ bool U61_Script::execute_lua_string_func_void(char *result,
  * calls a user defined function which takes an integer as argument
  * and returns a string
  */
-bool U61_Script::execute_lua_string_func_int(char *result,
+bool U61_Script::execute_lua_string_func_int(char result[U61_CONST_STRING_SIZE],
 					     char *func, 
 					     int par1)
 {
@@ -631,8 +637,7 @@ bool U61_Script::execute_lua_string_func_int(char *result,
     case 0:
       if (lua_isstring(lua_state,-1))
 	{
-	  strncpy(result,lua_tostring(lua_state,-1),U61_CONST_STRING_SIZE);
-	  result[U61_CONST_STRING_SIZE-1]='\0';
+	  U61_MACRO_STRNCPY(result,lua_tostring(lua_state,-1),U61_CONST_STRING_SIZE);
 	  success=true;
 	}
       else
@@ -641,7 +646,7 @@ bool U61_Script::execute_lua_string_func_int(char *result,
 	}
       break;
     default:
-      U61_LOG_WARNING("Lua error executing \""<<func<<"("<<par1<<")\", code is"<<err<<".");
+      U61_LOG_WARNING("Lua error executing \""<<func<<"("<<par1<<")\", code is "<<err<<".");
       break;
     }
 
@@ -684,6 +689,25 @@ char *U61_Script::get_version()
   return (result);
 }
 
+/*--------------------------------------------------------------------------*/
+/*
+ * calls the user defined function which can performs various initializations
+ * at the beginning of a game.
+ */
+void U61_Script::start(U61_Map *m, U61_Block *b)
+{
+  secure_pointers();
+
+  current_map=m;
+  current_block=b;
+  map_read=true;
+  map_write=true;
+  block_read=true;
+  block_write=true;
+
+  execute_lua_void_func_void(U61_SCRIPT_LUA_START);
+}
+
 /*--------------------------------------------------------------------------*/
 /*
  * calls the user defined function that returns the code for the next block.
@@ -1303,7 +1327,7 @@ static int get_width(lua_State *ls)
 
   if (current_map!=NULL && map_read)
     {
-      lua_pushnumber(ls,U61_MAP_WIDTH);
+      lua_pushnumber(ls,current_map->get_width());
       result=1;
     }
 
@@ -1321,13 +1345,53 @@ static int get_height(lua_State *ls)
 
   if (current_map!=NULL && map_read)
     {
-      lua_pushnumber(ls,U61_MAP_HEIGHT);
+      lua_pushnumber(ls,current_map->get_height());
       result=1;
     }
 
   return result;
 }
 
+/*--------------------------------------------------------------------------*/
+/*
+ * this function is accessible from lua script 
+ * it sets the width of the map
+ */
+static int set_width(lua_State *ls)
+{
+  int result=0;
+  int w;
+
+  if (current_map!=NULL && map_write)
+    {
+      w=(int) lua_tonumber(ls,-1);
+
+      current_map->set_width(w);
+    }
+
+  return result;
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * this function is accessible from lua script 
+ * it sets the height of the map
+ */
+static int set_height(lua_State *ls)
+{
+  int result=0;
+  int h;
+
+  if (current_map!=NULL && map_write)
+    {
+      h=(int) lua_tonumber(ls,-1);
+
+      current_map->set_height(h);
+    }
+
+  return result;
+}
+
 /*--------------------------------------------------------------------------*/
 /*
  * this function is accessible from lua script 
@@ -1941,6 +2005,44 @@ static int is_block_ok(lua_State *ls)
   return result;
 }
 
+/*--------------------------------------------------------------------------*/
+/*
+ * this function is accessible from lua script 
+ * it clears a map from all its squares
+ */
+static int clear_map(lua_State *ls)
+{
+  int result=0;
+
+  if (current_map!=NULL && map_write)
+    {
+      current_map->clear();
+    }
+
+  return result;
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * this function is accessible from lua script 
+ * it shifts a map on both vertical and/or horizontal ways
+ */
+static int shift_map(lua_State *ls)
+{
+  int result=0;
+  int x,y;
+
+  if (current_map!=NULL && map_write)
+    {
+      x=(int) lua_tonumber(ls,-2);
+      y=(int) lua_tonumber(ls,-1);
+
+      current_map->shift(x,y);
+    }
+
+  return result;
+}
+
 /*--------------------------------------------------------------------------*/
 /*
  * registers all C functions that can be used from Lua
@@ -1963,6 +2065,8 @@ void U61_Script::record_all()
   lua_register(lua_state,U61_SCRIPT_C_SET_ITEM_COLOR,set_item_color);
   lua_register(lua_state,U61_SCRIPT_C_GET_WIDTH,get_width);
   lua_register(lua_state,U61_SCRIPT_C_GET_HEIGHT,get_height);
+  lua_register(lua_state,U61_SCRIPT_C_SET_WIDTH,set_width);
+  lua_register(lua_state,U61_SCRIPT_C_SET_HEIGHT,set_height);
   lua_register(lua_state,U61_SCRIPT_C_GET_SQUARE_COLOR,get_square_color);
   lua_register(lua_state,U61_SCRIPT_C_SET_SQUARE_COLOR,set_square_color);
   lua_register(lua_state,U61_SCRIPT_C_GET_TIME,get_time);
@@ -1994,6 +2098,8 @@ void U61_Script::record_all()
   lua_register(lua_state,U61_SCRIPT_C_ADD_ANTIDOTE,add_antidote);
   lua_register(lua_state,U61_SCRIPT_C_DELETE_ANTIDOTE,delete_antidote);
   lua_register(lua_state,U61_SCRIPT_C_IS_BLOCK_OK,is_block_ok);
+  lua_register(lua_state,U61_SCRIPT_C_CLEAR_MAP,clear_map);
+  lua_register(lua_state,U61_SCRIPT_C_SHIFT_MAP,shift_map);
 }
 
 
