@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -40,8 +40,8 @@
  */
 
 
-#ifndef INC_U61_SCRIPT
-#define INC_U61_SCRIPT
+#ifndef U61_INC_SCRIPT
+#define U61_INC_SCRIPT
 
 /*---------------------------------------------------------------------------
  includes
@@ -75,6 +75,7 @@ class U61_Script
 
   static char *get_program();
   static char *get_version();
+  static void start(U61_Map *m, U61_Block *b);
   static int new_shape(U61_Map *m,U61_Block *b);
   static void do_shape(U61_Map *m,U61_Block *b,int type);
   static void rotate_left(U61_Map *m,U61_Block *b);
@@ -107,9 +108,9 @@ class U61_Script
   static bool execute_lua_int_func_int(int *result, 
 				       char *func, 
 				       int par1);
-  static bool execute_lua_string_func_void(char *result, 
+  static bool execute_lua_string_func_void(char result[U61_CONST_STRING_SIZE], 
 					   char *func);
-  static bool execute_lua_string_func_int(char *result,
+  static bool execute_lua_string_func_int(char result[U61_CONST_STRING_SIZE],
 					  char *func, 
 					  int par1);
 };
