@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -36,51 +36,57 @@
 
 
 /*---------------------------------------------------------------------------
- includes
- ---------------------------------------------------------------------------*/
+  includes
+  ---------------------------------------------------------------------------*/
 
 #include "serverprotocol.h"
 #include "global.h"
 #include "time.h"
 #include "platform.h"
-#include "debug.h"
+#include "log.h"
+#include "macro.h"
+#include "checksum.h"
 
 /*---------------------------------------------------------------------------
- constants
- ---------------------------------------------------------------------------*/
+  constants
+  ---------------------------------------------------------------------------*/
 
 #define U61_SERVERPROTOCOL_LISTENING           10
 #define U61_SERVERPROTOCOL_CLIENT_ACCEPTED     11
-#define U61_SERVERPROTOCOL_SCRIPT_NAME_SENT    30
-#define U61_SERVERPROTOCOL_SCRIPT_SENT         31
-#define U61_SERVERPROTOCOL_ID0_SENT            40
-#define U61_SERVERPROTOCOL_ID1_SENT            41
-#define U61_SERVERPROTOCOL_INITIAL_SPEED_SENT  42
-#define U61_SERVERPROTOCOL_ACCELERATION_SENT   43
-#define U61_SERVERPROTOCOL_CURSE_DELAY_SENT    44
-#define U61_SERVERPROTOCOL_TIME_SENT           45
+#define U61_SERVERPROTOCOL_PASSWORD_RECEIVED   30
+#define U61_SERVERPROTOCOL_PASSWORD_OK_SENT    31
+#define U61_SERVERPROTOCOL_SCRIPT_NAME_SENT    40
+#define U61_SERVERPROTOCOL_SCRIPT_SENT         41
+#define U61_SERVERPROTOCOL_ID0_SENT            50
+#define U61_SERVERPROTOCOL_ID1_SENT            51
+#define U61_SERVERPROTOCOL_INITIAL_SPEED_SENT  52
+#define U61_SERVERPROTOCOL_ACCELERATION_SENT   53
+#define U61_SERVERPROTOCOL_CURSE_DELAY_SENT    54
+#define U61_SERVERPROTOCOL_TIME_SENT           55
 
 /*---------------------------------------------------------------------------
- functions
- ---------------------------------------------------------------------------*/
+  functions
+  ---------------------------------------------------------------------------*/
 
 /*--------------------------------------------------------------------------*/
 /*
  * creates a server protocol
  */
-U61_ServerProtocol::U61_ServerProtocol(U61_ServerDispatcher *d, int p,int ch)
-  : U61_Protocol(p,ch)
+U61_ServerProtocol::U61_ServerProtocol(U61_ServerDispatcher *d, int p)
+  : U61_Protocol(p)
 {
-  int i;
+  int i,j;
 
   dispatcher=d;
-  new_connection=0;
+  new_connection=-1;
 
   for (i=0;i<U61_GAME_NB_NET_GROUPS-1;++i)
     {
       connection[i]=NULL;
-      id0[i]=-1;
-      id1[i]=-1;
+      for (j=0;j<U61_GAME_NB_PLAYERS_PER_NET_GROUP;++j)
+	{
+	  id[i][j]=-1;
+	}	
     }
 }
 
@@ -100,12 +106,6 @@ U61_ServerProtocol::~U61_ServerProtocol()
           connection[i]=NULL;
 	}
     }
-    
-  if (session!=NULL)
-    {
-      delete session;
-      session=NULL;
-    }
 }
 
 /*--------------------------------------------------------------------------*/
@@ -115,60 +115,42 @@ U61_ServerProtocol::~U61_ServerProtocol()
  */
 void U61_ServerProtocol::check_connection()
 {
-  int i;
+  int i,j;
   U61_Player *p;
-  CL_NetComputer client;
   U61_Event evt;  
-  bool client_left=false;
 
-  if (session!=NULL)
+  for (i=0;i<U61_GAME_NB_NET_GROUPS-1;++i)
     {
-      try 
+      if (connection[i]!=NULL)
 	{
-	  client=(CL_NetComputer) session->receive_computer_leave();
-	  client_left=true;
-	}
-      catch (CL_Error e)
-	{
-	  U61_LOG_DEBUG("no client leaved the game, ClanLib says: "<<e.message.c_str());
-	}
-
-      if (client_left)
-	{
-	  for (i=0;i<U61_GAME_NB_NET_GROUPS-1;++i)
+	  if (!connection[i]->is_alive())
 	    {
-	      if (connection[i]!=NULL
-		  && connection[i]->is_associated(&client))
+	      if (dispatcher!=NULL)
 		{
-		  if (dispatcher!=NULL)
-		    {
-		      dispatcher->close(connection[i]);
-		    }
-		  delete connection[i];
+		  dispatcher->close(connection[i]);
+		}
+	      delete connection[i];
+	      connection[i]=NULL;
+	      
+	      for (j=0;j<U61_GAME_NB_PLAYERS_PER_NET_GROUP;++j)
+		{
+		  evt.time=0;
+		  evt.author=evt.target=id[i][j];
+		  
+		  evt.code=U61_EVENT_READY;
+		  dispatcher->put_out(evt);   
 		  
 		  evt.code=U61_EVENT_KILL;
-		  evt.time=0;
-		  evt.author=evt.target=id0[i];
 		  dispatcher->put_out(evt);   
-		  evt.author=evt.target=id1[i];
-		  dispatcher->put_out(evt);
 		  
-		  p=U61_Global::game.find_player_by_id(id0[i]);
+		  p=U61_Global::game.find_player_by_id(id[i][j]);
 		  if (p!=NULL)
 		    {
 		      p->kill();
 		    }
-		  p=U61_Global::game.find_player_by_id(id1[i]);
-		  if (p!=NULL)
-		    {
-		      p->kill();
-		    }
-		  
-		  connection[i]=NULL;
-		  id0[i]=-1;
-		  id1[i]=-1;  
-		} 
-	    } 
+		  id[i][j]=-1;
+		}		  
+	    }
 	}
     }
 }
@@ -192,20 +174,27 @@ int U61_ServerProtocol::send_next_packet()
 	}
       break;
     case U61_SERVERPROTOCOL_CLIENT_ACCEPTED:
-      if ((result=send_program(connection[new_connection]))==U61_PROTOCOL_DONE)
+      if ((result=new_connection>=0 ? send_program(connection[new_connection]) : U61_PROTOCOL_FAILED)==U61_PROTOCOL_DONE)
 	{
 	  stage=U61_PROTOCOL_PROGRAM_SENT;
 	  U61_LOG_DEBUG("stage=PROGRAM_SENT");
 	}
       break;
     case U61_PROTOCOL_PROGRAM_RECEIVED:
-      if ((result=send_version(connection[new_connection]))==U61_PROTOCOL_DONE)
+      if ((result=new_connection>=0 ? send_version(connection[new_connection]) : U61_PROTOCOL_FAILED)==U61_PROTOCOL_DONE)
 	{
 	  stage=U61_PROTOCOL_VERSION_SENT;
 	  U61_LOG_DEBUG("stage=VERSION_SENT");
 	}
       break;
-    case U61_PROTOCOL_VERSION_RECEIVED:
+    case U61_SERVERPROTOCOL_PASSWORD_RECEIVED:
+      if ((result=send_password_ok())==U61_PROTOCOL_DONE)
+	{
+	  stage=U61_SERVERPROTOCOL_PASSWORD_OK_SENT;
+	  U61_LOG_DEBUG("stage=PASSWORD_OK_SENT");
+	}
+      break;
+    case U61_SERVERPROTOCOL_PASSWORD_OK_SENT:
       if ((result=send_script_name())==U61_PROTOCOL_DONE)
 	{
 	  stage=U61_SERVERPROTOCOL_SCRIPT_NAME_SENT;
@@ -262,17 +251,26 @@ int U61_ServerProtocol::send_next_packet()
 	}
       break;
     case U61_SERVERPROTOCOL_TIME_SENT:
-      if ((result=send_ready(connection[new_connection]))==U61_PROTOCOL_DONE)
+      if ((result=new_connection>=0 ? send_ready(connection[new_connection]) : U61_PROTOCOL_FAILED)==U61_PROTOCOL_DONE)
 	{
 	  stage=U61_PROTOCOL_READY_SENT;
 	  U61_LOG_DEBUG("stage=READY_SENT");
 	}
       break;
     case U61_PROTOCOL_READY_RECEIVED:
-      dispatcher->open(connection[new_connection]);
+      if (new_connection>=0) 
+	{
+	  result=U61_PROTOCOL_DONE;
+	  dispatcher->open(connection[new_connection]);
+	  new_connection=-1;
+	  U61_LOG_DEBUG("stage=LISTENING");
+	  stage=U61_SERVERPROTOCOL_LISTENING;
+	}
+      else
+	{
+	  result=U61_PROTOCOL_FAILED;
+	}
 
-      U61_LOG_DEBUG("stage=LISTENING");
-      stage=U61_SERVERPROTOCOL_LISTENING;
       break;
     }
 
@@ -301,21 +299,28 @@ int U61_ServerProtocol::recv_next_packet()
 	}
       break;
     case U61_PROTOCOL_PROGRAM_SENT:
-      if ((result=recv_program(connection[new_connection]))==U61_PROTOCOL_DONE)
+      if ((result=new_connection>=0 ? recv_program(connection[new_connection]) : U61_PROTOCOL_FAILED)==U61_PROTOCOL_DONE)
 	{
 	  stage=U61_PROTOCOL_PROGRAM_RECEIVED;
 	  U61_LOG_DEBUG("stage=PROGRAM_RECEIVED");
 	}
       break;
     case U61_PROTOCOL_VERSION_SENT:
-      if ((result=recv_version(connection[new_connection]))==U61_PROTOCOL_DONE)
+      if ((result=new_connection>=0 ? recv_version(connection[new_connection]) : U61_PROTOCOL_FAILED)==U61_PROTOCOL_DONE)
 	{
 	  stage=U61_PROTOCOL_VERSION_RECEIVED;
 	  U61_LOG_DEBUG("stage=VERSION_RECEIVED");
 	}
       break;
+    case U61_PROTOCOL_VERSION_RECEIVED:
+      if ((result=new_connection>=0 ? recv_password() : U61_PROTOCOL_FAILED)==U61_PROTOCOL_DONE)
+	{
+	  stage=U61_SERVERPROTOCOL_PASSWORD_RECEIVED;
+	  U61_LOG_DEBUG("stage=PASSWORD_RECEIVED");
+	}
+      break;
     case U61_PROTOCOL_READY_SENT:
-      if ((result=recv_ready(connection[new_connection]))==U61_PROTOCOL_DONE)
+      if ((result=new_connection>=0 ? recv_ready(connection[new_connection]) : U61_PROTOCOL_FAILED)==U61_PROTOCOL_DONE)
 	{
 	  stage=U61_PROTOCOL_READY_RECEIVED;
 	  U61_LOG_DEBUG("stage=READY_RECEIVED");
@@ -337,22 +342,18 @@ int U61_ServerProtocol::start_listening()
 {
   int result=U61_PROTOCOL_LATER;
 
-  try 
-    {
-      session=new CL_NetSession(U61_CONST_PROGRAM,port);
-    }
-  catch (CL_Error e)
-    {
-      result=U61_PROTOCOL_FAILED;
-      diagnostic+=e.message;
-      session=NULL;
-    }
-
-  if (session!=NULL)
+  if (listening_sock.listen(port))
     {
       result=U61_PROTOCOL_DONE;
-    };
-  
+    }
+  else
+    {
+      U61_MACRO_SPRINTF0(diagnostic,
+			 "Unable to bind socket");
+
+      result=U61_PROTOCOL_FAILED;
+    }
+
   return result;
 }
 
@@ -365,38 +366,33 @@ int U61_ServerProtocol::start_listening()
 int U61_ServerProtocol::accept_client()
 {
   int result=U61_PROTOCOL_LATER;
-  CL_NetComputer computer;
+  U61_BufferedSocket *accepted_sock;
   
-  try 
+  accepted_sock=listening_sock.accept();
+
+  if (accepted_sock)
     {
-      if (!((computer=(CL_NetComputer) 
-	     session->receive_computer_join())==
-	    CL_NetComputer(NULL)))
+      if ((new_connection=get_free_connection())<0)
 	{
-	  if ((new_connection=get_free_connection())<0)
+	  result=U61_PROTOCOL_FAILED;
+	}
+      else
+	{
+	  if ((connection[new_connection]=new U61_Connection(accepted_sock))!=NULL)
+	    {	      
+	      result=U61_PROTOCOL_DONE;
+	    }
+	  else
 	    {
 	      result=U61_PROTOCOL_FAILED;
 	    }
-	  else
-	    {  
-	      /*
-	       * There's a connection available to talk with the client
-	       * so we can discuss with him.
-	       * It should be important to check when the computer
-	       * object is deleted by the way...
-	       */
-	      connection[new_connection]=new 
-		U61_Connection(session,computer,
-			       channel+new_connection,true);        
-	      result=U61_PROTOCOL_DONE;
-	    }
 	}
-    }
-  catch (CL_Error e)
-    {
-      /*
-       * Here we trap any error raised by receive_computer_join
-       */
+
+      if (result != U61_PROTOCOL_DONE)
+	{
+	  new_connection=-1;
+	  delete accepted_sock;
+	}
     }
   
   return result;
@@ -424,6 +420,81 @@ int U61_ServerProtocol::get_free_connection()
   return result;
 }
 
+/*--------------------------------------------------------------------------*/
+/*
+ * Receives the password.
+ */
+int U61_ServerProtocol::recv_password()
+{
+  int result=U61_PROTOCOL_LATER;
+  char ascii_sig_client[U61_CONST_STRING_SIZE];
+  char *ascii_sig_server;
+  U61_Checksum checksum;
+
+  if (new_connection>=0 && connection[new_connection]!=NULL)
+    {
+      if (connection[new_connection]->peek_str())
+	{
+	  if (connection[new_connection]->recv(ascii_sig_client,
+					       sizeof(ascii_sig_client)-1))
+	    {
+	      if (!U61_Global::config.use_password)
+		{
+		  result=U61_PROTOCOL_DONE;
+		  U61_LOG_DEBUG("No password check.");
+		}
+	      else
+		{
+		  checksum.calc(U61_Global::config.password,
+				strlen(U61_Global::config.password));
+		  ascii_sig_server=checksum.get_ascii_sig();
+		  
+		  if (strcmp(ascii_sig_client,ascii_sig_server)==0)
+		    {
+		      result=U61_PROTOCOL_DONE;
+		      U61_LOG_DEBUG("Password is OK");
+		    }
+		  else
+		    {
+		      result=U61_PROTOCOL_FAILED;
+		      U61_LOG_WARNING("Wrong password.");
+		    }
+		}
+	    }
+	}      
+    }
+  else
+    {
+      result=U61_PROTOCOL_FAILED;
+    }
+
+  return result;
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * Sends the message "password is OK"
+ */
+int U61_ServerProtocol::send_password_ok()
+{
+  int result=U61_PROTOCOL_LATER;
+
+  if (new_connection>=0 && connection[new_connection]!=NULL)
+    {
+      if (connection[new_connection]->send(U61_PROTOCOL_OK_MESSAGE,
+					   strlen(U61_PROTOCOL_OK_MESSAGE)))
+	{
+	  result=U61_PROTOCOL_DONE;
+	}
+    }
+  else
+    {
+      result=U61_PROTOCOL_FAILED;
+    }
+ 
+  return result;
+}
+
 /*--------------------------------------------------------------------------*/
 /*
  * Sends the script name to the client, we remove the path from the
@@ -434,13 +505,19 @@ int U61_ServerProtocol::send_script_name()
   int result=U61_PROTOCOL_LATER;
   char script_name[U61_CONST_STRING_SIZE];
 
-  strncpy(script_name,
-	  U61_Global::config.script_file,
-	  sizeof(script_name)-1);
-  U61_Platform::strip_remote_script_path(script_name);
-  if (connection[new_connection]->send(script_name,sizeof(script_name)-1))
+  if (new_connection>=0 && connection[new_connection]!=NULL)
     {
-      result=U61_PROTOCOL_DONE;
+      U61_MACRO_STRCPY(script_name,
+		       U61_Global::config.script_file);
+      U61_Platform::strip_remote_script_path(script_name);
+      if (connection[new_connection]->send(script_name,sizeof(script_name)-1))
+	{
+	  result=U61_PROTOCOL_DONE;
+	}
+    }
+  else
+    {
+      result=U61_PROTOCOL_FAILED;
     }
  
   return result;
@@ -459,35 +536,42 @@ int U61_ServerProtocol::send_script()
   int c;
   int pos=0;
 
-  U61_Global::data.chdir_to_initial_dir();
-
-  strncpy(script_name,U61_Global::config.script_file,sizeof(script_name)-1);
-
-  f=fopen(script_name,"r");
-  if (f!=NULL)
+  if (new_connection>=0 && connection[new_connection]!=NULL)
     {
-      while ((c=fgetc(f))!=EOF && pos<U61_CONST_SCRIPT_SIZE)
+      U61_Global::data.chdir_to_initial_dir();
+
+      U61_MACRO_STRCPY(script_name,
+		       U61_Global::config.script_file);
+
+      f=fopen(script_name,"r");
+      if (f!=NULL)
 	{
-	  script[pos++]=c;
+	  while ((c=fgetc(f))!=EOF && pos<(int) sizeof(script)-1)
+	    {
+	      script[pos++]=c;
+	    }
+	  fclose(f);
+	  if (connection[new_connection]->send(script,sizeof(script)-1))
+	    {
+	      result=U61_PROTOCOL_DONE;
+	    }
 	}
-      fclose(f);
-      if (connection[new_connection]->send(script,sizeof(script)-1))
+      else
 	{
-	  result=U61_PROTOCOL_DONE;
+	  U61_LOG_DEBUG("Unable to open \""<<script_name<<"\".");
+	  U61_MACRO_SPRINTF1(diagnostic,
+			     "Could not open script file \"%s\"",
+			     script_name);
+	  result=U61_PROTOCOL_FAILED;
 	}
+
+      U61_Global::data.chdir_to_data_dir();
     }
   else
     {
-      U61_LOG_WARNING("Unable to open \""<<script_name<<"\".");
-
-      diagnostic="Could not open script file \"";
-      diagnostic+=script_name;
-      diagnostic+="\"";
       result=U61_PROTOCOL_FAILED;
     }
 
-  U61_Global::data.chdir_to_data_dir();
-
   return result;
 }
 
@@ -498,13 +582,21 @@ int U61_ServerProtocol::send_script()
 int U61_ServerProtocol::send_id0()
 {
   int result=U61_PROTOCOL_LATER;
-  int id;
+  int id0;
 
-  id=U61_Global::game.find_free_player_id0();
-  if (connection[new_connection]->send(id))
+  if (new_connection>=0 && connection[new_connection]!=NULL)
     {
-      id0[new_connection]=id;
-      result=U61_PROTOCOL_DONE;
+      id0=U61_Global::game.find_free_player_id0();
+      if (connection[new_connection]->send(id0))
+	{
+	  id[new_connection][0]=id0;
+	  connection[new_connection]->authorize_player_id(id0,true);
+	  result=U61_PROTOCOL_DONE;
+	}
+    }
+  else
+    {
+      result=U61_PROTOCOL_FAILED;
     }
  
   return result;
@@ -517,13 +609,21 @@ int U61_ServerProtocol::send_id0()
 int U61_ServerProtocol::send_id1()
 {
   int result=U61_PROTOCOL_LATER;
-  int id;
+  int id1;
 
-  id=U61_Global::game.find_free_player_id1();
-  if (connection[new_connection]->send(id))
+  if (new_connection>=0 && connection[new_connection]!=NULL)
     {
-      id1[new_connection]=id;
-      result=U61_PROTOCOL_DONE;
+      id1=U61_Global::game.find_free_player_id1();
+      if (connection[new_connection]->send(id1))
+	{
+	  id[new_connection][1]=id1;
+	  connection[new_connection]->authorize_player_id(id1,true);
+	  result=U61_PROTOCOL_DONE;
+	}
+    }
+  else
+    {
+      result=U61_PROTOCOL_FAILED;
     }
  
   return result;
@@ -538,9 +638,16 @@ int U61_ServerProtocol::send_initial_speed()
 {
   int result=U61_PROTOCOL_LATER;
 
-  if (connection[new_connection]->send(U61_Global::config.initial_speed))
+  if (new_connection>=0 && connection[new_connection]!=NULL)
     {
-      result=U61_PROTOCOL_DONE;
+      if (connection[new_connection]->send(U61_Global::config.initial_speed))
+	{
+	  result=U61_PROTOCOL_DONE;
+	}
+    }
+  else
+    {
+      result=U61_PROTOCOL_FAILED;
     }
  
   return result;
@@ -555,9 +662,16 @@ int U61_ServerProtocol::send_acceleration()
 {
   int result=U61_PROTOCOL_LATER;
 
-  if (connection[new_connection]->send(U61_Global::config.acceleration))
+  if (new_connection>=0 && connection[new_connection]!=NULL)
     {
-      result=U61_PROTOCOL_DONE;
+      if (connection[new_connection]->send(U61_Global::config.acceleration))
+	{
+	  result=U61_PROTOCOL_DONE;
+	}
+    }
+  else
+    {
+      result=U61_PROTOCOL_FAILED;
     }
  
   return result;
@@ -572,9 +686,16 @@ int U61_ServerProtocol::send_curse_delay()
 {
   int result=U61_PROTOCOL_LATER;
 
-  if (connection[new_connection]->send(U61_Global::config.curse_delay))
+  if (new_connection>=0 && connection[new_connection]!=NULL)
     {
-      result=U61_PROTOCOL_DONE;
+      if (connection[new_connection]->send(U61_Global::config.curse_delay))
+	{
+	  result=U61_PROTOCOL_DONE;
+	}
+    }
+  else
+    {
+      result=U61_PROTOCOL_FAILED;
     }
  
   return result;
@@ -589,9 +710,16 @@ int U61_ServerProtocol::send_time()
 {
   int result=U61_PROTOCOL_LATER;
 
-  if (connection[new_connection]->send(U61_Time::for_event()))
+  if (new_connection>=0 && connection[new_connection]!=NULL)
     {
-      result=U61_PROTOCOL_DONE;
+      if (connection[new_connection]->send(U61_Time::for_event()))
+	{
+	  result=U61_PROTOCOL_DONE;
+	}
+    }
+  else
+    {
+      result=U61_PROTOCOL_FAILED;
     }
  
   return result;
@@ -606,10 +734,10 @@ void U61_ServerProtocol::handle_error(int *result)
 {
   if ((*result)==U61_PROTOCOL_FAILED)
     {
-      U61_LOG_WARNING("Network server protocol error on connection "
-		      <<new_connection
-		      <<" at stage "
-		      <<stage);
+      U61_LOG_DEBUG("Network server protocol error on connection "
+		    <<new_connection
+		    <<" at stage "
+		    <<stage);
 
       if (stage!=U61_PROTOCOL_BEGIN)
 	{
@@ -628,9 +756,10 @@ void U61_ServerProtocol::handle_error(int *result)
 	{
 	  delete connection[new_connection];
 	  connection[new_connection]=NULL;
+	  new_connection=-1;
 	}
 
-      if (session==NULL)
+      if (! listening_sock.is_alive())
 	{
 	  /*
 	   * session is NULL, this means we have not successfully passed
@@ -651,3 +780,4 @@ void U61_ServerProtocol::handle_error(int *result)
     }
 }
 
+
