@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -35,8 +35,8 @@
  */
 
 
-#ifndef INC_U61_SERVERPROTOCOL
-#define INC_U61_SERVERPROTOCOL
+#ifndef U61_INC_SERVERPROTOCOL
+#define U61_INC_SERVERPROTOCOL
 
 /*---------------------------------------------------------------------------
  includes
@@ -47,6 +47,7 @@
 #include "protocol.h"
 #include "serverdispatcher.h"
 #include "game.h"
+#include "bufferedsocket.h"
 
 /*---------------------------------------------------------------------------
  classes
@@ -55,7 +56,7 @@
 class U61_ServerProtocol : public U61_Protocol
 {
  public:
-  U61_ServerProtocol(U61_ServerDispatcher *d, int p,int ch);
+  U61_ServerProtocol(U61_ServerDispatcher *d, int p);
   ~U61_ServerProtocol();
 
   virtual void check_connection();
@@ -64,14 +65,16 @@ class U61_ServerProtocol : public U61_Protocol
 
  protected:
   U61_ServerDispatcher *dispatcher;
+  U61_BufferedSocket listening_sock;
   U61_Connection *connection[U61_GAME_NB_NET_GROUPS-1]; 
-  int id0[U61_GAME_NB_NET_GROUPS-1];
-  int id1[U61_GAME_NB_NET_GROUPS-1];
+  int id[U61_GAME_NB_NET_GROUPS-1][U61_GAME_NB_PLAYERS_PER_NET_GROUP];
   int new_connection;
 
   int start_listening();
   int accept_client();    
   int get_free_connection();
+  int recv_password();
+  int send_password_ok();
   int send_script_name();
   int send_script();
   int send_id0();
