@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -40,7 +40,8 @@
 
 #include "sound.h"
 #include "global.h"
-#include "debug.h"
+#include "log.h"
+#include "system.h"
 
 /*---------------------------------------------------------------------------
  variants
@@ -184,6 +185,7 @@ void U61_Sound::wait_no_playback()
 	 (U61_Global::data.sound_player_loose &&
 	  U61_Global::data.sound_player_loose->is_playing()))
     {
-      CL_System::keep_alive(100);
+      CL_System::sleep(100);
+      U61_System::keep_alive();
     }    
 }
