@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -45,6 +45,8 @@
 #include "global.h"
 #include "time.h"
 #include "utils.h"
+#include "serial.h"
+#include "log.h"
 
 /*---------------------------------------------------------------------------
  constants
@@ -327,3 +329,59 @@ void U61_Square::draw_curse(int x, int y,int size)
     }
 }
 
+/*--------------------------------------------------------------------------*/
+/*
+ * draws a filler square
+ */
+void U61_Square::draw_filler(int x, int y,int size)
+{
+  if (state)
+    {
+      U61_Global::data.filler[size]->put_screen
+	(x,y,(cycle+cycle_offset)%U61_SQUARE_NB_BITMAP);
+    }
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * Serializes a square into a byte strem
+ */
+bool U61_Square::serialize(unsigned char **buffer, unsigned char *buffer_limit)
+{
+  bool result=true;
+  unsigned char *buffer_start=*buffer;
+
+  result=result && U61_Serial::serialize_int(explosion_counter,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_int(color,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_bool(state,buffer,buffer_limit);
+  result=result && U61_Serial::serialize_bool(exploding,buffer,buffer_limit);
+
+  if ((*buffer) - buffer_start != U61_SQUARE_SERIALIZED_SIZE)
+    {
+      U61_LOG_WARNING("Wrong buffer size for square serialization (diff="<<(buffer_start - (*buffer) + U61_SQUARE_SERIALIZED_SIZE)<<")!");
+    }
+
+  return result;
+}
+
+/*--------------------------------------------------------------------------*/
+/*
+ * Un-serializes a square from a byte strem
+ */
+bool U61_Square::unserialize(unsigned char **buffer, unsigned char *buffer_limit)
+{
+  bool result=true;
+  unsigned char *buffer_start=*buffer;
+
+  result=result && U61_Serial::unserialize_int(&explosion_counter,buffer,buffer_limit);
+  result=result && U61_Serial::unserialize_int(&color,buffer,buffer_limit);
+  result=result && U61_Serial::unserialize_bool(&state,buffer,buffer_limit);
+  result=result && U61_Serial::unserialize_bool(&exploding,buffer,buffer_limit);
+
+  if ((*buffer) - buffer_start != U61_SQUARE_SERIALIZED_SIZE)
+    {
+      U61_LOG_WARNING("Wrong buffer size for square unserialization (diff="<<(buffer_start - (*buffer) + U61_SQUARE_SERIALIZED_SIZE)<<")!");
+    }
+
+  return result;
+}
