@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -38,8 +38,8 @@
  */
 
 
-#ifndef INC_U61_SQUARE
-#define INC_U61_SQUARE
+#ifndef U61_INC_SQUARE
+#define U61_INC_SQUARE
 
 /*---------------------------------------------------------------------------
  includes
@@ -55,7 +55,9 @@
  ---------------------------------------------------------------------------*/
 
 #define U61_SQUARE_NB_COLORS                8
-    
+  
+#define U61_SQUARE_SERIALIZED_SIZE (2*U61_SERIAL_INT_SERIALIZED_SIZE + 2*U61_SERIAL_BOOL_SERIALIZED_SIZE)
+  
 /*---------------------------------------------------------------------------
  classes
  ---------------------------------------------------------------------------*/
@@ -82,6 +84,7 @@ class U61_Square
   void draw(int x,int y, int size=U61_DATA_SIZE_BIG);
   void draw_anticipation(int x,int y, int size=U61_DATA_SIZE_BIG);
   void draw_curse(int x,int y, int size=U61_DATA_SIZE_BIG);
+  void draw_filler(int x,int y, int size=U61_DATA_SIZE_BIG);
 
   void begin_explosion();
   void end_explosion();
@@ -90,6 +93,9 @@ class U61_Square
   bool is_explosion_ended();
   bool is_exploding();
     
+  bool serialize(unsigned char **buffer, unsigned char *buffer_limit);
+  bool unserialize(unsigned char **buffer, unsigned char *buffer_limit);
+
  protected:
   int cycle_offset;
   int explosion_counter;
