@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -42,6 +42,7 @@
 #include <string.h>
 
 #include "text.h"
+#include "log.h"
 
 /*---------------------------------------------------------------------------
  globals
@@ -65,6 +66,17 @@ U61_Text::U61_Text(char *label) : U61_MenuItem()
 /*
  * select function, it does nothing so the text is never animated
  */
-void U61_Text::select()
+void U61_Text::select(int sel)
 {
+  selection = sel;
 }
+
+/*--------------------------------------------------------------------------*/
+/*
+ * Returns false since this kind of item must *not* be selected
+ */
+bool U61_Text::selectable()
+{
+  return false;
+}
+
