@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -35,8 +35,8 @@
 
 
 
-#ifndef INC_U61_TEXT
-#define INC_U61_TEXT
+#ifndef U61_INC_TEXT
+#define U61_INC_TEXT
 
 /*---------------------------------------------------------------------------
  includes
@@ -54,8 +54,9 @@ class U61_Text: public U61_MenuItem
 public:
     U61_Text(char *label);
     
-    virtual void select();
+    virtual void select(int pos);
 protected:
+    virtual bool selectable();
 };
 
 #endif
