@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -43,6 +43,7 @@
 #include "menuinput.h"
 #include "toggle.h"
 #include "const.h"
+#include "macro.h"
 
 /*---------------------------------------------------------------------------
  globals
@@ -92,9 +93,7 @@ void U61_Toggle::update()
     /*
      * we want the text in the item to look like "blablabla is on"
      */
-    strcpy(buffer,name);
-    strcat(buffer," ");
-    strcat(buffer,end);
+    U61_MACRO_SPRINTF2(buffer,"%s %s",name,end);
 
     /*
      * puts the prepared string in the label
@@ -137,10 +136,12 @@ bool U61_Toggle::action(int key)
 /*
  * selection of the item (only selects the value)
  */
-void U61_Toggle::select()
+void U61_Toggle::select(int sel)
 {
     int start;
 
+    selection = sel;
+
     start=strlen(name);
 
     select_right(start);
