@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -34,8 +34,8 @@
  */
 
 
-#ifndef INC_U61_UTILS
-#define INC_U61_UTILS
+#ifndef U61_INC_UTILS
+#define U61_INC_UTILS
 
 /*---------------------------------------------------------------------------
  includes
@@ -61,6 +61,10 @@ public:
     static bool set_videomode(int w,int h,bool fullscreen);
 
     static bool file_exists(char *filename);
+
+    static void to_hexascii(char *hexascii_data,
+			    unsigned char *binary_data, 
+			    int binary_data_size);
 protected:
 };
 
