@@ -7,7 +7,7 @@
  *  U     6   1   bb   y    U    F    O   O   T                             *
  *                                                                          *
  * U61 is another block based game                                          *
- * Copyright (C) 2000 Christian Mauduit (ufoot@ufoot.org / www.ufoot.org)   *
+ * Copyright (C) 2000-2003 Christian Mauduit (ufoot@ufoot.org)              *
  *                                                                          *
  * This program is free software; you can redistribute it and/or            *
  * modify it under the terms of the GNU General Public License              *
@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software              *
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA*
  *                                                                          *
- * This project is also available on SourceForge  (http://sourceforge.net)  *
+ * This project is also available on Savannah (http://savannah.gnu.org)     *
  ****************************************************************************/
 
 /*
@@ -34,8 +34,8 @@
  */
 
 
-#ifndef INC_U61_VERSION
-#define INC_U61_VERSION
+#ifndef U61_INC_VERSION
+#define U61_INC_VERSION
 
 /*---------------------------------------------------------------------------
  includes
@@ -43,6 +43,8 @@
 
 #include <ClanLib/core.h>
 
+#include "const.h"
+
 /*---------------------------------------------------------------------------
  classes
  ---------------------------------------------------------------------------*/
@@ -50,7 +52,7 @@
 class U61_Version
 {
 public:
-    static void get_version_text(char *buffer);
+    static void get_version_text(char buffer[U61_CONST_STRING_SIZE]);
     static int get_version_code();
 
 protected:
